﻿XIncludeFile #PB_Compiler_Home+"Inc\HID_Module.pbi"

Structure HID_Param
  *ID         ; Идентификатор USB HID соединения.
  IsConnect.a ; 0 - связи нет, иначе есть.
EndStructure

CompilerIf Defined(DEV_BROADCAST_DEVICEINTERFACE, #PB_Structure)=0
  Structure DEV_BROADCAST_DEVICEINTERFACE
    dbcc_size.l
    dbcc_devicetype.l
    dbcc_reserved.l
    dbcc_classguid.GUID
    dbcc_name.c[1]
  EndStructure
CompilerEndIf


Global gHID.HID_Param
Global NewList gOW_SN_List.OW_SN()

#USB_PID = $1003
#USB_VID = $2017

#HID_PackInSize = 65   ; Размер пакета.
#HID_PackOutSize = 9

Enumeration
  #HID_Ret_Unknown   ; Неопределенный тип ошибки.
  #HID_Ret_OK        ; Ошибок нет.
  #HID_Ret_NoConnect ; Нет связи с устройством.
  #HID_Ret_ErrWtite  ; Ошибка передачи.
  #HID_Ret_WtiteTimeout ;  Превышен таймаут при передаче.
  #HID_Ret_ErrRead   ; Ошибка приема.
  #HID_Ret_ReadTimeout  ;  Превышен таймаут при приеме.
  #HID_Ret_ErrBuff
  
  #HID_DevErr_Cmd    ; Команда не поддерживается.
EndEnumeration

Enumeration
  #HID_Cmd_GetSensor = 1
  #HID_Cmd_GetTemp
EndEnumeration

Procedure HID_Init(hWnd)
  HID::HID_Init(OSVersion())
  
  If HID::RegHidChange(hWnd) = 0
    MessageRequester(#ProgName, "Error calling function 'HID::RegHidChange'"+Chr(10)+
                                "Please restart the program.")
    End
  EndIf
EndProcedure

Procedure HID_End()
  HID::HID_End()
EndProcedure


Procedure HID_DevIO(*OutBuff, *InBuff, LenOut=#HID_PackOutSize, LenIn=#HID_PackInSize, TimeoutMS=1000)
  Protected Res = #HID_Ret_NoConnect, Ret
  
  If *OutBuff And *InBuff
    With gHID
      If \IsConnect And \ID
        Ret = HID::WriteDevice(\ID, *OutBuff, LenOut, TimeoutMS)
        If Ret = #HID_PackOutSize
          Ret = HID::ReadDevice(\ID, *InBuff, LenIn, TimeoutMS)
          If Ret = #HID_PackInSize
            Res = #HID_Ret_OK
          ElseIf Ret = -1
            Res = #HID_Ret_ReadTimeout
          Else
            Res = #HID_Ret_ErrRead
          EndIf
        ElseIf Ret = -1
          Res = #HID_Ret_WtiteTimeout
        Else
          Res = #HID_Ret_ErrWtite
        EndIf
      EndIf
    EndWith
  Else
    Res = #HID_Ret_ErrBuff
  EndIf
  
  ProcedureReturn Res
EndProcedure

Procedure HID_GetSensor()
  Protected Dim InBuff.a(#HID_PackInSize)
  Protected Dim OutBuff.a(#HID_PackOutSize)
  Protected Ret, CountDev.a=0, i, Count=0
  
  ClearList(gOW_SN_List())
  
  With gHID
    If \IsConnect And \ID    
      
      Repeat
        OutBuff(0)=0
        OutBuff(1)=#HID_Cmd_GetSensor
        OutBuff(2)=Count
        
        Ret = HID_DevIO(@OutBuff(), @InBuff())
        If Ret = #HID_Ret_OK And InBuff(1) = OutBuff(1)
          
          If Count=0
            CountDev = InBuff(2)
          EndIf
          
          For i=1 To 7
            If CountDev>0
              If AddElement(gOW_SN_List())
                CopyMemory(@InBuff(i*8+1), @gOW_SN_List()\Type, 8)
                If GetCRC(@gOW_SN_List()\Type, 7) <> gOW_SN_List()\CRC
                  DeleteElement(gOW_SN_List())
                EndIf
              EndIf
            Else
              Break
            EndIf
            CountDev-1
          Next          
          
        Else
          Break
        EndIf
        
        Count+1
      Until CountDev<=0
    EndIf
  EndWith
  
EndProcedure


Procedure HID_GetTemp()
  Protected Dim InBuff.a(#HID_PackInSize)
  Protected Dim OutBuff.a(#HID_PackOutSize)
  
  With gHID
    If \IsConnect And \ID    
      
      OutBuff(0)=0
      OutBuff(1)=#HID_Cmd_GetTemp
       
      If HID_DevIO(@OutBuff(), @InBuff()) = #HID_Ret_OK And InBuff(1) = OutBuff(1)
        ListIcon_AddTemp(#ListIcon_Temp, @InBuff(3), ListSize(gOW_SN_List()))
      EndIf
      
    EndIf
  EndWith
EndProcedure


Procedure HID_DeviceTest(DevState=-1)
  Protected Test
  
  With gHID
    
    If \ID
      If DevState <> 0
        Delay(100)
        Test=HID::TestDevice_Path(\ID)
      Else
        Test=0
      EndIf
    Else
      Test=HID::TestDevice(#USB_PID, #USB_VID)
    EndIf
    
    
    If Bool(Test) <> \IsConnect
      If Test
        If \ID = 0
          \ID = HID::OpenDevice(#USB_PID, #USB_VID)
          Test = Bool(\ID>0)
        Else
          Test = HID::ReOpenDevice(\ID)
        EndIf
        
        If Test
          \IsConnect=#True
          ;Debug "IsConnect=#True"
          StatusBarText(0, 0, "USB thermometer is connected")
          If DevState<>-1
            Delay(400)
          EndIf
          HID_GetSensor()
          ListIcon_AddDev(#ListIcon_Temp, gOW_SN_List())
          HID_GetTemp()
        EndIf
        
      Else
        \IsConnect=#False
        ;Debug "IsConnect=#False"
        StatusBarText(0, 0, "USB thermometer not connected")
        HID::DevCloseHandle(\ID)
        ClearGadgetItems(#ListIcon_Temp)
        ClearList(gOW_SN_List())
        ;ListIcon_AddTemp(#ListIcon_Temp, 0, ListSize(gOW_SN_List()))
      EndIf
    EndIf
    
  EndWith
EndProcedure

Procedure HID_ReConnect(uMsg, *Dev.DEV_BROADCAST_DEVICEINTERFACE)
  Protected Path.s
  
  If uMsg = #DBT_DEVICEARRIVAL Or uMsg = #DBT_DEVICEREMOVECOMPLETE
    If gHID\ID = 0
      HID_DeviceTest()
    Else
      
      If *Dev And *Dev\dbcc_devicetype = #DBT_DEVTYP_DEVICEINTERFACE
        Path = PeekS(@*Dev\dbcc_name)
        If Path<>"" And LCase(HID::GetDevPath(gHID\ID)) = LCase(Path)
          HID_DeviceTest(Bool(uMsg = #DBT_DEVICEARRIVAL))
        EndIf
      EndIf
      
    EndIf
  EndIf
  
EndProcedure
; IDE Options = PureBasic 5.70 LTS (Windows - x86)
; Folding = --
; EnableXP
; CompileSourceDirectory