﻿DeclareModule HID

;-DeclareModule

Structure HID_CAPS
  Usage.w
  UsagePage.w
  InputReportByteLength.w
  OutputReportByteLength.w
  FeatureReportByteLength.w
  Reserved.w[17]
  NumberLinkCollectionNodes.w
  NumberInputButtonCaps.w
  NumberInputValueCaps.w
  NumberInputDataIndices.w
  NumberOutputButtonCaps.w
  NumberOutputValueCaps.w
  NumberOutputDataIndices.w
  NumberFeatureButtonCaps.w
  NumberFeatureValueCaps.w
  NumberFeatureDataIndices.w
EndStructure

Structure HID_Sub_DeviceInfo
  VendorID.u
  ProductID.u
  VersionNumber.u
  NumInputBuffers.u
  InputReportByteLength.u
  OutputReportByteLength.u
  FeatureReportByteLength.u
  Manufacturer.s
  Product.s
  SerialNumber.s
EndStructure

Structure HID_DeviceInfo
  CountDevice.w              ; Число обнаруженных HID устройств
  DeviceInfo.HID_Sub_DeviceInfo[258]
EndStructure


Structure HID_Attributes
  VID.u
  PID.u
  VersionNumber.u
EndStructure

Declare HID_Init(Version = #PB_OS_Windows_XP)
Declare HID_End()
Declare RegHidChange(hWnd)
Declare OpenDevice(PID.u, VID.u, VersionNumber.w=-1, Index.u=0)
Declare CloseDevice(*Dev)
Declare DeviceID(*Dev)
Declare DevCloseHandle(*Dev)
Declare TestDevice(PID.u, VID.u, VersionNumber.w=-1, Index.u=0)
Declare TestDevice_Path(*Dev)
Declare.s GetDevPath(*Dev)
Declare ReOpenDevice(*Dev)
Declare DeviceInfo(*Info.HID_DeviceInfo)
Declare ReadDevice(*Dev, *Buffer, Len, TimeOut=-1)
Declare WriteDevice(*Dev, *Buffer, Len, TimeOut=-1)
Declare GetFeature(*Dev, *Buffer, Len)
Declare SetFeature(*Dev, *Buffer, Len)
Declare GetInputReport(*Dev, *Buffer, Len)
Declare SetOutputReport(*Dev, *Buffer, Len)
Declare GetCaps(hDevice, *Capabilities.HID_CAPS)
 ; Узнаём по хендлу устройства, его PID, VID и номер версии.
Declare GetAttributes(*Dev, *DeviceInfo.HID_Attributes)
Declare GetNumInputBuffers(*Dev)
Declare.s GetManufacturerString(*Dev)
Declare.s GetProductString(*Dev)
Declare.s GetSerialNumberString(*Dev)
Declare.s GetIndexedString(*Dev, Index)

EndDeclareModule




Module HID


Structure HIDD_ATTRIBUTES
  Size.l
  VendorID.u
  ProductID.u
  VersionNumber.u
EndStructure

Structure PSP_DEVICE_INTERFACE_DETAIL_DATA
  cbSize.l
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    DevicePath.l
  CompilerElse 
    DevicePath.c
  CompilerEndIf
EndStructure

CompilerIf Defined(SP_DEVICE_INTERFACE_DATA, #PB_Structure)=0
  Structure SP_DEVICE_INTERFACE_DATA
    cbSize.l
    InterfaceClassGuid.GUID
    Flags.l
    Reserved.l
  EndStructure
CompilerEndIf

CompilerIf Defined(DEV_BROADCAST_DEVICEINTERFACE, #PB_Structure)=0
  Structure DEV_BROADCAST_DEVICEINTERFACE
    dbcc_size.l
    dbcc_devicetype.l
    dbcc_reserved.l
    dbcc_classguid.GUID
    dbcc_name.i
  EndStructure
CompilerEndIf

Structure DevInfo
  hDev.i
  Ov.OVERLAPPED
  DevPath.s
EndStructure

EnableExplicit

CompilerIf #PB_Compiler_Version < 560
  Define.i
CompilerEndIf

Prototype pCancelIoEx(hFile, *lpOverlapped)

Define hKernel32=0, CancelIoEx.pCancelIoEx=0

CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
  Import "hid_x86.lib"
CompilerElseIf #PB_Compiler_Processor = #PB_Processor_x64
  Import "hid_x64.lib"
CompilerElse
  CompilerError "Процессор не поддерживается"
CompilerEndIf


  HidD_GetHidGuid(*HidGuid.GUID)
  HidD_GetAttributes(*HidDeviceObject, *Attributes.HIDD_ATTRIBUTES)
  HidD_GetPreparsedData(*HidDeviceObject, *PreparsedData)
  HidP_GetCaps(*PreparsedData, *Capabilities.HID_CAPS)
  HidD_FreePreparsedData(PreparsedData)
  HidD_GetNumInputBuffers(HidHandle, *NumInputBuffers)
  HidD_GetManufacturerString(HidHandle, *Buffer, Len)
  HidD_GetProductString(HidHandle, *Buffer, Len)
  HidD_GetSerialNumberString(HidHandle, *Buffer, Len)
  HidD_GetIndexedString(HidHandle, Index, *Buffer, Len)
  HidD_GetFeature(HidHandle, *Buffer, Len)
  HidD_SetFeature(HidHandle, *Buffer, Len)
  HidD_GetInputReport(HidHandle, *Buffer, Len)
  HidD_SetOutputReport(HidHandle, *Buffer, Len)

EndImport

Procedure HID_Init(Version = #PB_OS_Windows_XP)
  Shared hKernel32, CancelIoEx
  
  If Version >= #PB_OS_Windows_Vista
    hKernel32 = LoadLibrary_("kernel32.dll")
    If hKernel32
      CancelIoEx = GetProcAddress_(hKernel32, ?l_CancelIoEx)
      If CancelIoEx=0
        FreeLibrary_(hKernel32)
        hKernel32 = 0
      EndIf
    Else
      CancelIoEx = 0
    EndIf
  Else
    hKernel32 = 0
    CancelIoEx = 0
  EndIf
  
  ProcedureReturn
  
  DataSection
    l_CancelIoEx:
    ! db "CancelIoEx", 0, 0
  EndDataSection
  
EndProcedure

Procedure HID_End()
  Shared hKernel32
  
  If hKernel32
    FreeLibrary_(hKernel32)
    hKernel32 = 0
  EndIf
  
EndProcedure


Procedure FunctInfo(hDevice, *Info.HID_DeviceInfo)
  Protected Attributes.HIDD_ATTRIBUTES, i
  Protected HIDP_CAPS.HID_CAPS
    
  Attributes\Size = SizeOf(HIDD_ATTRIBUTES)
  
  If HidD_GetAttributes(hDevice, @Attributes)
    
    i = *Info\CountDevice
    *Info\CountDevice+1
    
    *Info\DeviceInfo[i]\VendorID        = Attributes\VendorID
    *Info\DeviceInfo[i]\ProductID       = Attributes\ProductID
    *Info\DeviceInfo[i]\VersionNumber   = Attributes\VersionNumber
    
    *Info\DeviceInfo[i]\Manufacturer    = GetManufacturerString(hDevice)
    *Info\DeviceInfo[i]\Product         = GetProductString(hDevice)
    *Info\DeviceInfo[i]\SerialNumber    = GetSerialNumberString(hDevice)
    *Info\DeviceInfo[i]\NumInputBuffers = GetNumInputBuffers(hDevice)
    GetCaps(hDevice, @HIDP_CAPS)
    *Info\DeviceInfo[i]\InputReportByteLength   = HIDP_CAPS\InputReportByteLength
    *Info\DeviceInfo[i]\OutputReportByteLength  = HIDP_CAPS\OutputReportByteLength
    *Info\DeviceInfo[i]\FeatureReportByteLength = HIDP_CAPS\FeatureReportByteLength
    
  EndIf
  
EndProcedure


Procedure RegHidChange(hWnd)
  Protected Res = 0
  Protected Dev.DEV_BROADCAST_DEVICEINTERFACE
  
  If hWnd
    Dev\dbcc_size = SizeOf(DEV_BROADCAST_DEVICEINTERFACE)
    Dev\dbcc_devicetype = #DBT_DEVTYP_DEVICEINTERFACE
    HidD_GetHidGuid(@Dev\dbcc_classguid)
    
    Res=RegisterDeviceNotification_(hWnd, @Dev, 0)
  EndIf
  
  ProcedureReturn Res  
EndProcedure

; Получение доступа к HID устройству
Procedure Open_HID_Device(PID.u, VID.u, VersionNumber.w=-1, Index.u=0, *Funct=0,
                          *Info.HID_DeviceInfo=0, *Dev.DevInfo=0)
  Protected HidGuid.Guid
  Protected devInfoData.SP_DEVICE_INTERFACE_DATA
  Protected Attributes.HIDD_ATTRIBUTES
  Protected *detailData.PSP_DEVICE_INTERFACE_DETAIL_DATA
  Protected Length.l, CurrentIndex.w, hDevInfo
  Protected i, Result, DevicePath.s
  Protected Required, hDevice=0, FlagsAndAttributes = 0
  
  devInfoData\cbSize = SizeOf(SP_DEVICE_INTERFACE_DATA)

  HidD_GetHidGuid(@HidGuid)
  
  hDevInfo=SetupDiGetClassDevs_(@HidGuid,0,0, #DIGCF_PRESENT|#DIGCF_DEVICEINTERFACE)
  If hDevInfo=0
    ProcedureReturn 0
  EndIf
  
  If *Dev
    FlagsAndAttributes = #FILE_FLAG_OVERLAPPED
  EndIf
  
  For i=0 To 255
    
    Result=SetupDiEnumDeviceInterfaces_(hDevInfo, 0, @HidGuid, i, @devInfoData)
    If Result
      Result = SetupDiGetDeviceInterfaceDetail_(hDevInfo, @devInfoData, 0, 0,@Length, 0)
      *detailData=AllocateMemory(Length)
      *detailData\cbSize=SizeOf(PSP_DEVICE_INTERFACE_DETAIL_DATA)
      Result = SetupDiGetDeviceInterfaceDetail_(hDevInfo, @devInfoData, *detailData, Length+1, @Required, 0)
      
      DevicePath.s=PeekS(@*detailData\DevicePath)
      FreeMemory(*detailData)
      
      hDevice=CreateFile_(@DevicePath, #GENERIC_READ|#GENERIC_WRITE, #FILE_SHARE_READ|#FILE_SHARE_WRITE,
                          0, #OPEN_EXISTING, FlagsAndAttributes, 0)
                          ;@Security, #OPEN_EXISTING, FlagsAndAttributes, 0)
      
      If hDevice<>#INVALID_HANDLE_VALUE
        
        If *Funct And *Info
          
          CallFunctionFast(*Funct, hDevice, *Info)
          CloseHandle_(hDevice)
          
        Else
          
          Attributes\Size = SizeOf(HIDD_ATTRIBUTES)
          Result = HidD_GetAttributes(hDevice, @Attributes)
          
          If Attributes\ProductID=PID And Attributes\VendorID=VID And 
             (Attributes\VersionNumber=VersionNumber Or VersionNumber=-1 ) 
            If CurrentIndex=Index
              
              If *Dev
                *Dev\DevPath = DevicePath
              EndIf
              
              SetupDiDestroyDeviceInfoList_(hDevInfo)
              
              ProcedureReturn hDevice
            Else
              CurrentIndex+1
              CloseHandle_(hDevice)
            EndIf 
          Else
            CloseHandle_(hDevice)
          EndIf 
          
        EndIf
        
      EndIf
    Else
      Break 
    EndIf
  Next i
  
  SetupDiDestroyDeviceInfoList_(hDevInfo)
  ProcedureReturn 0
EndProcedure

Procedure OpenDevice(PID.u, VID.u, VersionNumber.w=-1, Index.u=0)
  Protected hDevice
  Protected *Dev.DevInfo = 0
  
  *Dev = AllocateMemory(SizeOf(DevInfo))
  If *Dev
    InitializeStructure(*Dev, DevInfo)
    hDevice = Open_HID_Device(PID, VID, VersionNumber, Index, 0, 0, *Dev)
    If hDevice
      *Dev\hDev = hDevice
      FillMemory(@*Dev\Ov, SizeOf(*Dev\Ov), 0)
      *Dev\Ov\hEvent = CreateEvent_(0, 0, 0, 0)
      If *Dev\Ov\hEvent = 0
        CloseHandle_(hDevice)
        hDevice = 0
        Goto m1
      EndIf
    Else
      m1:
      ClearStructure(*Dev, DevInfo)
      FreeMemory(*Dev)
      *Dev = 0
    EndIf
  EndIf
  
  ProcedureReturn *Dev
EndProcedure

Procedure CloseDevice(*Dev.DevInfo) ; Закрытие HID устройства.
  Shared CancelIoEx
  Protected hDevice
  Protected Res = #False
  
  If *Dev
    
    hDevice = DeviceID(*Dev)
    *Dev\hDev = #INVALID_HANDLE_VALUE
    
    If hDevice
      If CancelIoEx
        CancelIoEx(hDevice, *Dev\Ov)
        CancelIoEx(hDevice, 0)
      Else
        CancelIo_(hDevice)
      EndIf
      Res = CloseHandle_(hDevice)
    EndIf
    
    If *Dev\Ov\hEvent
      CloseHandle_(*Dev\Ov\hEvent)
      *Dev\Ov\hEvent = 0
    EndIf
    
    ClearStructure(*Dev, DevInfo)
    FreeMemory(*Dev)
  EndIf
  
  ProcedureReturn Res
EndProcedure


Procedure DeviceID(*Dev.DevInfo)
  If *Dev 
    If *Dev>20000 Or *Dev<-20000
      If *Dev\hDev<>#INVALID_HANDLE_VALUE
        ProcedureReturn *Dev\hDev
      Else
        ProcedureReturn 0
      EndIf
    Else
      ProcedureReturn *Dev ; Это дескриптор, а не указатель.
    EndIf
  Else
    ProcedureReturn 0
  EndIf
EndProcedure


Procedure DevCloseHandle(*Dev.DevInfo)
  Shared CancelIoEx
  Protected Res = #False
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice
    Res = #True
    *Dev\hDev = #INVALID_HANDLE_VALUE
    If CancelIoEx
      CancelIoEx(hDevice, *Dev\Ov)
      CancelIoEx(hDevice, 0)
    Else
      CancelIo_(hDevice)
    EndIf
    CloseHandle_(hDevice)
  EndIf
  
  ProcedureReturn Res
EndProcedure

Procedure TestDevice(PID.u, VID.u, VersionNumber.w=-1, Index.u=0)
  Protected hDevice, Result=#False
  
  hDevice=Open_HID_Device(PID, VID, VersionNumber, Index)
  If hDevice
    Result=#True
    CloseHandle_(hDevice)
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure TestDevice_Path(*Dev.DevInfo)
  Protected Res = #False, hDevice
  
  If *Dev And *Dev\DevPath<>""
    hDevice=CreateFile_(@*Dev\DevPath, #GENERIC_READ|#GENERIC_WRITE, #FILE_SHARE_READ|#FILE_SHARE_WRITE,
                        0, #OPEN_EXISTING, 0, 0)
    If hDevice<>#INVALID_HANDLE_VALUE
      Res = #True
      CloseHandle_(hDevice)
    EndIf
  EndIf
  
  ProcedureReturn Res
EndProcedure

Procedure.s GetDevPath(*Dev.DevInfo)
  If *Dev 
    ProcedureReturn *Dev\DevPath
  Else
    ProcedureReturn ""
  EndIf
EndProcedure

Procedure ReOpenDevice(*Dev.DevInfo)
  Protected Res = #False, hDevice
  
  If *Dev And *Dev\DevPath<>""
    
    If *Dev\hDev<>#INVALID_HANDLE_VALUE
      CloseHandle_(*Dev\hDev)
    EndIf
    
    *Dev\hDev = CreateFile_(@*Dev\DevPath, #GENERIC_READ|#GENERIC_WRITE, #FILE_SHARE_READ|#FILE_SHARE_WRITE,
                            0, #OPEN_EXISTING, #FILE_FLAG_OVERLAPPED, 0)
    
    If *Dev\hDev<>#INVALID_HANDLE_VALUE
      Res = #True
    EndIf
    
  EndIf
  
  ProcedureReturn Res
EndProcedure

Procedure DeviceInfo(*Info.HID_DeviceInfo)
  If *Info
    ClearStructure(*Info, HID_DeviceInfo)
    Open_HID_Device(0, 0, 0, 0, @FunctInfo(), *Info)
    ProcedureReturn Bool(*Info\CountDevice>0)
  EndIf
EndProcedure

Procedure ReadDevice(*Dev.DevInfo, *Buffer, Len, TimeOut=-1) ; Чтение данных из HID устройства
  Protected ReadBytes.l=0
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice=0 Or *Buffer=0 Or Len<=0
    ProcedureReturn 0
  EndIf
  
  ResetEvent_(*Dev\Ov\hEvent)
  FillMemory(@*Dev\Ov, OffsetOf(OVERLAPPED\hEvent), 0)
  
  If ReadFile_(hDevice, *Buffer, Len, @ReadBytes, @*Dev\Ov) = 0 And GetLastError_() = #ERROR_IO_PENDING
    
    If TimeOut>=0
      
      If WaitForSingleObject_(*Dev\Ov\hEvent, TimeOut)<>#WAIT_OBJECT_0
        CancelIo_(hDevice)
        ProcedureReturn -1
      EndIf
      
    EndIf
    
    If GetOverlappedResult_(hDevice, @*Dev\Ov, @ReadBytes, #True) = 0
      CancelIo_(hDevice)
      ReadBytes = 0
    EndIf
  Else
    ;Debug "Read  "+GetLastError_()
    CancelIo_(hDevice)
    ReadBytes = 0
  EndIf
  
  
  ProcedureReturn ReadBytes
EndProcedure

Procedure WriteDevice(*Dev.DevInfo, *Buffer, Len, TimeOut=-1) ; Запись данных в HID устройство
  Protected Written.l=0, Ov.OVERLAPPED
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice=0 Or *Buffer=0 Or Len<=0
    ProcedureReturn 0
  EndIf
  
  
  ResetEvent_(*Dev\Ov\hEvent)
  FillMemory(@*Dev\Ov, OffsetOf(OVERLAPPED\hEvent), 0)
  
  If WriteFile_(hDevice, *Buffer, Len, @Written, @*Dev\Ov) = 0 And GetLastError_() = #ERROR_IO_PENDING
    
    If TimeOut>=0
      
      If WaitForSingleObject_(*Dev\Ov\hEvent, TimeOut)<>#WAIT_OBJECT_0
        CancelIo_(hDevice)
        ProcedureReturn -1
      EndIf
      
    EndIf
    
    If GetOverlappedResult_(hDevice, @*Dev\Ov, @Written, #True) = 0
      CancelIo_(hDevice)
      Written = 0
    EndIf
  Else
    ;Debug "Write  "+GetLastError_()
    CancelIo_(hDevice)
    Written = 0
  EndIf
  
;   If WriteFile_(hDevice, *Buffer, Len, 0, @Ov) = 0
;     If GetLastError_() = #ERROR_IO_PENDING
;       If GetOverlappedResult_(hDevice, @Ov, @Written, #True) = 0
;         Written = 0
;       EndIf
;     EndIf
;   EndIf
  
  ProcedureReturn Written
EndProcedure


Procedure GetFeature(*Dev.DevInfo, *Buffer, Len)
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice And *Buffer And Len>0
    ProcedureReturn HidD_GetFeature(hDevice, *Buffer, Len)
  Else
    ProcedureReturn 0
  EndIf
EndProcedure

Procedure SetFeature(*Dev.DevInfo, *Buffer, Len)
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice And *Buffer And Len>0
    ProcedureReturn HidD_SetFeature(hDevice, *Buffer, Len)
  Else
    ProcedureReturn 0
  EndIf
EndProcedure

 ; Чтение входного репорта. ВНИМАНИЕ функция появилась только в WinXP
Procedure GetInputReport(hDevice, *Buffer, Len)
;   Shared OS_Version
;   If hDevice And *buffer And Len>0 And
;      OS_Version>=#PB_OS_Windows_XP
;     ProcedureReturn HidD_GetInputReport(hDevice, *Buffer, Len)
;   Else
    ProcedureReturn 0
;  EndIf
EndProcedure

 ; Запись в устройство выходного репорта. ВНИМАНИЕ функция появилась только в WinXP
Procedure SetOutputReport(hDevice, *Buffer, Len)
;   Shared OS_Version
;   If hDevice And *buffer And Len>0 And
;      OS_Version>=#PB_OS_Windows_XP
;     ProcedureReturn HidD_SetOutputReport(hDevice, *Buffer, Len)
;   Else
    ProcedureReturn 0
;   EndIf
EndProcedure

Procedure GetCaps(*Dev.DevInfo, *Capabilities.HID_CAPS) ; Узнаём размеры буферов и др. информацию об устройстве
  Protected result=0, PreparsedData
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice
    
    If HidD_GetPreparsedData(hDevice, @PreparsedData) 
      HidP_GetCaps(PreparsedData, *Capabilities) 
      HidD_FreePreparsedData(PreparsedData)
      result=1
    EndIf  
    
  EndIf
  ProcedureReturn result
EndProcedure

 ; Узнаём по хендлу устройства, его PID, VID и номер версии.
Procedure GetAttributes(*Dev.DevInfo, *DeviceInfo.HID_Attributes)
  Protected Info.HIDD_ATTRIBUTES, Result=#False
  Protected hDevice = DeviceID(*Dev)
  
  If hDevice And *DeviceInfo
    Info\Size = SizeOf(HIDD_ATTRIBUTES)
    If HidD_GetAttributes(hDevice, @Info)
      CopyMemory(@Info+OffsetOf(HIDD_ATTRIBUTES\VendorID), *DeviceInfo, SizeOf(HID_Attributes))
      Result=#True
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure GetNumInputBuffers(*Dev.DevInfo)
  Protected NumInputBuffers=0
  Protected hDevice = DeviceID(*Dev)
  If hDevice
    HidD_GetNumInputBuffers(hDevice, @NumInputBuffers)
  EndIf
  ProcedureReturn NumInputBuffers
EndProcedure

Procedure.s GetManufacturerString(*Dev.DevInfo) ; Получаем идентификатор изготовителя
  Protected Result.s="", *mem
  Protected hDevice = DeviceID(*Dev)
  If hDevice
    *mem=AllocateMemory(256)
    If *mem
      If HidD_GetManufacturerString(hDevice, *mem, 252)
        Result=PeekS(*mem,-1,#PB_Unicode)
      EndIf
      FreeMemory(*mem)
    EndIf
  EndIf
  ProcedureReturn Result
EndProcedure

Procedure.s GetProductString(*Dev.DevInfo) ; Получаем идентификатор продукта
  Protected Result.s="", *mem
  Protected hDevice = DeviceID(*Dev)
  If hDevice
    *mem=AllocateMemory(256)
    If *mem
      If HidD_GetProductString(hDevice, *mem, 252)
        Result=PeekS(*mem,-1,#PB_Unicode)
      EndIf
      FreeMemory(*mem)
    EndIf
  EndIf
  ProcedureReturn Result
EndProcedure

Procedure.s GetSerialNumberString(*Dev.DevInfo) ; Получаем серийный номер продукта
  Protected Result.s="", *mem
  Protected hDevice = DeviceID(*Dev)
  If hDevice
    *mem=AllocateMemory(256)
    If *mem
      If HidD_GetSerialNumberString(hDevice, *mem, 252)
        Result=PeekS(*mem,-1,#PB_Unicode)
      EndIf
      FreeMemory(*mem)
    EndIf
  EndIf
  ProcedureReturn Result
EndProcedure

Procedure.s GetIndexedString(*Dev.DevInfo, Index) ; Чтение строки по её индексу из устройства
  Protected Result.s="", *mem
  Protected hDevice = DeviceID(*Dev)
  If hDevice
    *mem=AllocateMemory(256)
    If *mem
      If HidD_GetIndexedString(hDevice, Index, *mem, 252)
        Result=PeekS(*mem,-1,#PB_Unicode)
      EndIf
      FreeMemory(*mem)
    EndIf
  EndIf
  ProcedureReturn Result
EndProcedure

EndModule
; IDE Options = PureBasic 5.70 LTS (Windows - x86)
; CursorPosition = 218
; FirstLine = 187
; Folding = -----
; EnableXP
; CompileSourceDirectory