﻿; =============================================================================
; File     : PureAutoIt.pbi (2016.07.02)
; Purpose  : Complete autoit functions implement in purebasic
; Author   : Ward
; =============================================================================

DeclareModule PureAutoIt
  #PureAutoIt_EnableRegExp = 1
  #PureAutoIt_Version$ = "2016.07.02"

  ; ===========================================================================
  ;  Enumerations
  ; ===========================================================================

    Enumeration
      #SW_ENABLE = 64
      #SW_DISABLE = 65
    EndEnumeration

    EnumerationBinary
      #WINDOW_EXISTS
      #WINDIW_VISIBLE
      #WINDIW_ENABLED
      #WINDIW_ACTIVE
      #WINDIW_MINIMIZED
      #WINDIW_MAXIMIZED
    EndEnumeration

    Enumeration
      #STR_CHRSPLIT     = 0
      #STR_ENTIRESPLIT  = 1
      #STR_NOCOUNT      = 2
      #STR_STRIPLEADING  = 1
      #STR_STRIPTRAILING = 2
      #STR_STRIPSPACES   = 4
      #STR_STRIPALL      = 8
      #STR_NOCASESENSE   = 0
      #STR_CASESENSE     = 1
      #STR_NOCASESENSEBASIC = 0
    EndEnumeration

    Enumeration
      #ENCODING_UNKNOWN                             ; Unknown or binary
      #ENCODING_BINARY          = #ENCODING_UNKNOWN ; Unknown or binary
      #ENCODING_ANSI                                ; 0-255
      #ENCODING_ASCII           = #PB_Ascii         ; 0-127
      #ENCODING_UTF8_BOM        = #PB_UTF8          ; UTF8 with BOM
      #ENCODING_UTF8_NOBOM                          ; UTF8 without BOM
      #ENCODING_UTF16_LE_BOM    = #PB_Unicode       ; UTF16 LE with BOM
      #ENCODING_UTF16_LE_NOBOM                      ; UTF16 LE without BOM
      #ENCODING_UTF16_BE_BOM    = #PB_UTF16BE       ; UTF16-BE with BOM
      #ENCODING_UTF16_BE_NOBOM                      ; UTF16-BE without BOM
    EndEnumeration

    Enumeration
      #FO_READ        = 0
      #FO_APPEND      = 1
      #FO_OVERWRITE   = 2
      #FO_CREATEPATH  = 8
      #FO_BINARY      = 16
      #FO_UNICODE     = 32
      #FO_UTF16_LE    = 32
      #FO_UTF16_BE    = 64 ; not supported
      #FO_UTF8        = 128
      #FO_UTF8_NOBOM  = 256
      #FO_UTF8_FULL   = 16384

      #FO_UNICODE_NOBOM = 8192 ; not allowed in autoit
      #FO_UTF16_LE_NOBOM = 8192
    EndEnumeration

    Enumeration ; for DriveGetType
      #DT_DRIVETYPE = 1
      #DT_SSDSTATUS = 2
      #DT_BUSTYPE   = 3
    EndEnumeration

    Enumeration ; for FileCopy
      #FC_NOOVERWRITE = 0
      #FC_OVERWRITE   = 1
      #FC_CREATEPATH  = 8
    EndEnumeration

    Enumeration ; for FileSetPos
      #FILE_BEGIN   = 0
      #FILE_CURRENT = 1
      #FILE_END     = 2
    EndEnumeration

    Enumeration ; for FileOpenDialog, FileSaveDialog
      #FD_FILEMUSTEXIST   = 1
      #FD_PATHMUSTEXIST   = 2
      #FD_MULTISELECT     = 4
      #FD_PROMPTCREATENEW = 8
      #FD_PROMPTOVERWRITE = 16
    EndEnumeration

    Enumeration ; for FileGetTime, FileSetTime
      #FT_MODIFIED = 0
      #FT_CREATED  = 1
      #FT_ACCESSED = 2
    EndEnumeration

    Enumeration ; ProgressOn, SplashTextOn, SplashImageOn
      #DLG_NOTITLE = 1
      #DLG_NOTONTOP = 2
      #DLG_TEXTLEFT = 4
      #DLG_TEXTRIGHT = 8
      #DLG_MOVEABLE = 16
      #DLG_TEXTVCENTER = 32
    EndEnumeration

  ; ===========================================================================
  ;  Structures
  ; ===========================================================================

    Structure WinInfo
      title.s
      hwnd.i
    EndStructure

    Structure ProcessInfo
      name.s
      pid.i
    EndStructure

    Structure StringRegExpMatch
      Array group.s(0)
    EndStructure

  ; ===========================================================================
  ;  Macros
  ; ===========================================================================

    Declare.l _Error(value.l = -559038721)
    Declare.l _Extended(value.l = -559038721)
    Declare.i _ExitCode()
    Declare.s _ScriptDir()
    Declare.s _WorkingDir()
    Declare.s _TempDir(csidl.l = -1)
    Declare.s _ScriptName()
    Declare.l _DesktopDepth()
    Declare.l _DesktopRefresh()
    Declare.s _KBLayout()
    Declare.s _OSLang()
    Declare.s _MUILang()
    Declare.s _OSArch()
    Declare.s _CPUArch()
    Declare.s _OSType()
    Declare.s _OSVersion()
    Declare.l _OSBuild()
    Declare.s _OSServicePack()
    Declare.s _ComputerName()
    Declare.s _UserName()
    Declare.s _IPAddress(n.l = 0)
    Declare.s _CmdLine(n.l)
    Declare.s _CmdLineRaw()

    Macro _AutoItPID:       GetCurrentProcessId_:  EndMacro
    Macro _ScriptFullPath:  ProgramFilename:       EndMacro
    Macro _AutoItExe:       PureAutoIt::_ScriptFullPath:       EndMacro
    Macro _ProgramPID:      PureAutoIt::_AutoItPID:            EndMacro
    Macro _ProgramFullPath: PureAutoIt::_ScriptFullPath:       EndMacro
    Macro _ProgramExe:      PureAutoIt::_ScriptFullPath:       EndMacro
    Macro _ProgramDir:      PureAutoIt::_ScriptDir:            EndMacro
    Macro _ProgramName:     PureAutoIt::_ScriptName:           EndMacro

    Macro _AppDataCommonDir  (): PureAutoIt::_TempDir(#CSIDL_COMMON_APPDATA):          EndMacro
    Macro _AppDataDir        (): PureAutoIt::_TempDir(#CSIDL_APPDATA):                 EndMacro
    Macro _CommonFilesDir    (): PureAutoIt::_TempDir(#CSIDL_PROGRAM_FILES_COMMON):    EndMacro
    Macro _DesktopCommonDir  (): PureAutoIt::_TempDir(#CSIDL_COMMON_DESKTOPDIRECTORY): EndMacro
    Macro _DesktopDir        (): PureAutoIt::_TempDir(#CSIDL_DESKTOP):                 EndMacro
    Macro _DocumentsCommonDir(): PureAutoIt::_TempDir(#CSIDL_COMMON_DOCUMENTS):        EndMacro
    Macro _FavoritesCommonDir(): PureAutoIt::_TempDir(#CSIDL_COMMON_FAVORITES):        EndMacro
    Macro _FavoritesDir      (): PureAutoIt::_TempDir(#CSIDL_FAVORITES):               EndMacro
    Macro _LocalAppDataDir   (): PureAutoIt::_TempDir(#CSIDL_LOCAL_APPDATA):           EndMacro
    Macro _MyDocumentsDir    (): PureAutoIt::_TempDir(#CSIDL_PERSONAL):                EndMacro
    Macro _ProgramFilesDir   (): PureAutoIt::_TempDir(#CSIDL_PROGRAM_FILES):           EndMacro
    Macro _ProgramsCommonDir (): PureAutoIt::_TempDir(#CSIDL_COMMON_PROGRAMS):         EndMacro
    Macro _ProgramsDir       (): PureAutoIt::_TempDir(#CSIDL_PROGRAMS):                EndMacro
    Macro _StartMenuCommonDir(): PureAutoIt::_TempDir(#CSIDL_COMMON_STARTMENU):        EndMacro
    Macro _StartMenuDir      (): PureAutoIt::_TempDir(#CSIDL_STARTMENU):               EndMacro
    Macro _StartupCommonDir  (): PureAutoIt::_TempDir(#CSIDL_COMMON_STARTUP):          EndMacro
    Macro _StartupDir        (): PureAutoIt::_TempDir(#CSIDL_STARTUP):                 EndMacro
    Macro _SystemDir         (): PureAutoIt::_TempDir(#CSIDL_SYSTEM):                  EndMacro
    Macro _UserProfileDir    (): PureAutoIt::_TempDir(#CSIDL_PROFILE):                 EndMacro
    Macro _WindowsDir        (): PureAutoIt::_TempDir(#CSIDL_WINDOWS):                 EndMacro

    Macro _AutoItVersion()
      #PureAutoIt_Version$
    EndMacro

    Macro _PureAutoItVersion()
      #PureAutoIt_Version$
    EndMacro

    CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
      Macro _AutoItX64():     1: EndMacro
      Macro _PureAutoItX64(): 1: EndMacro
    CompilerElse
      Macro _AutoItX64():     0: EndMacro
      Macro _PureAutoItX64(): 0: EndMacro
    CompilerEndIf

    Macro _HomeDrive()
      Left(PureAutoIt::_UserProfileDir(), 2)
    EndMacro

    Macro _HomePath()
      Mid(PureAutoIt::_UserProfileDir(), 3)
    EndMacro

    Macro _ComSpec(): GetEnvironmentVariable("ComSpec"): EndMacro

    Macro _IPAddress1(): PureAutoIt::_IPAddress(1): EndMacro
    Macro _IPAddress2(): PureAutoIt::_IPAddress(2): EndMacro
    Macro _IPAddress3(): PureAutoIt::_IPAddress(3): EndMacro
    Macro _IPAddress4(): PureAutoIt::_IPAddress(4): EndMacro

    Macro _DesktopHeight(): GetSystemMetrics_(#SM_CYSCREEN): EndMacro
    Macro _DesktopWidth(): GetSystemMetrics_(#SM_CXSCREEN): EndMacro

    ; Time And Date
    Macro _MSEC(): RSet(Str(ElapsedMilliseconds() % 1000), 3, "0"): EndMacro
    Macro _SEC(): RSet(Str(Second(Date())), 2, "0"): EndMacro
    Macro _MIN(): RSet(Str(Minute(Date())), 2, "0"): EndMacro
    Macro _HOUR(): RSet(Str(Hour(Date())), 2, "0"): EndMacro
    Macro _MDAY(): RSet(Str(Day(Date())), 2, "0"): EndMacro
    Macro _MON(): RSet(Str(Month(Date())), 2, "0"): EndMacro
    Macro _YEAR(): Str(Year(Date())): EndMacro
    Macro _WDAY(): Str(DayOfWeek(Date()) + 1): EndMacro
    Macro _YDAY(): RSet(Str(DayOfYear(Date())), 3, "0"): EndMacro

  ; ===========================================================================
  ;  Environment functions
  ; ===========================================================================

    Declare.s ClipGet()
    Declare.l ClipPut(text.s)
    Declare.s EnvGet(name.s)
    Declare.l EnvSet(name.s, value.s = "")
    Declare.l EnvUpdate()
    Declare.l MemGetStats(Array status(1))

  ; ===========================================================================
  ;  File, Directory and Disk functions
  ; ===========================================================================

    Macro FileInstall(source, dest, overwrite = 0)
      DataSection
        FILEINSTALL_BEGIN#MacroExpandedCount:
          IncludeBinary source
        FILEINSTALL_END#MacroExpandedCount:
      EndDataSection

      If Not PathFileExists_(dest) Or overwrite
        __file.i = CreateFile(#PB_Any, dest)
        If __file
          WriteData(__file, ?FILEINSTALL_BEGIN#MacroExpandedCount, ?FILEINSTALL_END#MacroExpandedCount - ?FILEINSTALL_BEGIN#MacroExpandedCount)
          CloseFile(__file)
        EndIf
      EndIf
    EndMacro

    Declare.l DirCopy(source.s, dest.s, overwirte.l = 0)
    Declare.l DirCreate(path.s)
    Declare.q DirGetSize(path.s, mode.l = 0, *file_count = 0, *dir_count = 0)
    Declare.l DirMove(source.s, dest.s, overwirte.l = 0)
    Declare.l DirRemove(path.s, recurse.l = 0)
    Declare.s DriveGetDrive(type.s = "all", separator.s = ",")
    Declare.s DriveGetFileSystem(path.s)
    Declare.s DriveGetLabel(path.s)
    Declare.s DriveGetSerial(path.s)
    Declare.s DriveGetType(path.s, operation.l = #DT_DRIVETYPE)
    Declare.s DriveMapAdd(device.s, remote.s, flags = 0, user.s = "", password.s = "")
    Declare.l DriveMapDel(drive.s)
    Declare.s DriveMapGet(device.s)
    Declare.l DriveSetLabel(path.s, label.s)
    Declare.d DriveSpaceFree(path.s)
    Declare.d DriveSpaceTotal(path.s)
    Declare.s DriveStatus(path.s)
    Declare.l FileChangeDir(dir.s)
    Declare.l FileClose(file.s)
    Declare.l FileCopy(source.s, dest.s, flag = #FC_NOOVERWRITE)
    Declare.l FileCreateNTFSLink(source.s, link.s, overwrite.l = 0)
    Declare.l FileCreateShortcut(file.s, link.s, workdir.s = "", args.s = "", desc.s = "", icon.s = "", hotkey.s = "", icon_index.l = 0, state.l = 0)
    Declare.l FileDelete(filename.s, force.l = 0)
    Declare.l FileExists(path.s)
    Declare.l FileFindClose(handle.i)
    Declare.i FileFindFirstFile(pattern.s)
    Declare.s FileFindNextFile(handle.i)
    Declare.l FileFlush(file.s)
    Declare.s FileGetAttrib(filename.s)
    Declare.l FileGetEncoding(file.s, mode = 1)
    Declare.s FileGetLongName(path.s, flag = 0)
    Declare.q FileGetPos(file.s)
    Declare.l FileGetShortcut(link.s, Array result.s(1))
    Declare.s FileGetShortName(path.s, flag = 0)
    Declare.q FileGetSize(filename.s)
    Declare.s FileGetTime(filename.s, option.l = #FT_MODIFIED, format.l = 0)
    Declare.s FileGetVersion(filename.s, name.s = "")
    Declare.l FileMove(source.s, dest.s, flag = #FC_NOOVERWRITE)
    Declare.s FileOpen(filename.s, mode.l = 0)
    Declare.s FileOpenDialog(title.s = "", init_dir.s = "", filter.s = "", option.l = 0, default_name.s = "", hwnd.i = 0)
    Declare.s FileRead(file.s, count.q = -1)
    Declare.i FileReadBinary(file.s, *buffer = 0, count.q = -1)
    Declare.s FileReadLine(file.s, line.l = 0)
    Declare.l FileReadToArray(file.s, Array lines.s(1))
    Declare.l FileReadToList(file.s, List lines.s())
    Declare.l FileRecycle(source.s)
    Declare.l FileRecycleEmpty(path.s = "")
    Declare.s FileSaveDialog(title.s = "", init_dir.s = "", filter.s = "", option.l = 0, default_name.s = "", hwnd.i = 0)
    Declare.s FileSelectFolder(text.s = "", root_dir.s = "", flag.l = 0, init_dir.s = "", hwnd.i = 0)
    Declare.l FileSetAttrib(source.s, set.s, recurse = 0)
    Declare.l FileSetPos(file.s, offset.q, origin.l = #FILE_BEGIN)
    Declare.l FileSetTime(source.s, time.s = "", option.l = #FT_MODIFIED, recurse.l = 0)
    Declare.l FileWrite(file.s, text.s, flush.l = #False)
    Declare.l FileWriteBinary(file.s, *buffer, count.q = -1, flush.l = #False)
    Declare.l FileWriteLine(file.s, text.s, flush.l = #False)
    Declare.l IniDelete(filename.s, section.s, key.s = "")
    Declare.s IniRead(filename.s, section.s, key.s, value.s = "")
    Declare.l IniReadSection(Map dict.s(), filename.s, section.s)
    Declare.s IniReadSectionKeys(filename.s, section.s, separator.s = ",")
    Declare.s IniReadSectionNames(filename.s, separator.s = ",")
    Declare.l IniRenameSection(filename.s, section.s, new_section.s, overwrite.l = 0)
    Declare.l IniWrite(filename.s, section.s, key.s, value.s)
    Declare.l IniWriteSection(filename.s, section.s, string.s)
    Declare.l IniWriteSectionMap(Map dict.s(), filename.s, section.s, keys.s = "", separator.s = ",")

  ; ===========================================================================
  ;  Graphic and Sound functions
  ; ===========================================================================

    Declare.l Beep(frequency = 500, duration = 1000)
    Declare.l PixelGetColor(x.l, y.l, hwnd.i = 0, alt.l = 0)
    Declare.l PixelChecksum(left.l, top.l, right.l, bottom.l, hwnd.i = 0)
    Declare.s PixelSearch(left.l, top.l, right.l, bottom.l, color.l, variation.l = 0, hwnd.i = 0)
    Declare.l SoundPlay(file.s = "", wait.l = 0)
    Declare.l SoundSetWaveVolume(percent.f)

  ; ===========================================================================
  ;  Keyboard functions
  ; ===========================================================================

    Declare.i HotKeySet(keys.s, *callback = 0)
    Declare Send(keys.s, flag.l = 0)
    Declare.i SendKeepActive(title.s = "", text.s = "")

    Macro _HotKeyPressed()
      PeekS(PureAutoIt::HotKeySet("@HotKeyPressed"))
    EndMacro

  ; ===========================================================================
  ;  Math functions
  ; ===========================================================================

    Macro Ceiling(exp): Round(exp, #PB_Round_Up): EndMacro
    Macro Floor(exp): Round(exp, #PB_Round_Down): EndMacro

  ; ===========================================================================
  ;  Message Boxes and Dialogs functions
  ; ===========================================================================
    Declare.s InputBox(title.s = "", prompt.s = "", value.s = "", password.s = "", width.l = -1, height.l = -1, left.l = #PB_Ignore, top.l = #PB_Ignore, timeout.l = 0, parent.i = 0, custom_button.s = "")
    Declare.s InputBoxEx(custom_button.s = "", title.s = "", prompt.s = "", value.s = "", password.s = "", width.l = -1, height.l = -1, left.l = #PB_Ignore, top.l = #PB_Ignore, timeout.l = 0, parent.i = 0)
    Declare.l MsgBox(flag = 0, title.s = "", text.s = "", timeout.l = 0, hwnd.i = 0)
    Declare.l MsgBoxEx(custom_button.s, flag = 0, title.s = "", text.s = "", timeout.l = 0, hwnd.i = 0)
    Declare ProgressOn(title.s = #Null$, maintext.s = #Null$, subtext.s = #Null$, x.l = -1, y.l = -1, opt.l = 0)
    Declare ProgressSet(percent.l = -1, subtext.s = #Null$, maintext.s = #Null$)
    Declare ProgressOff()
    Declare SplashImageOn(title.s, file.s, width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = 0)
    Declare SplashTextOn(title.s = "", text.s = "", width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = 0, fontname.s = "", fontsz.l = 12, fontwt.l = #FW_DONTCARE)
    Declare SplashOff()
    Declare.l ToolTip(text.s = "", x.l = #PB_Ignore, y.l = #PB_Ignore, title.s = "", icon.l = 0, option.l = 0)

  ; ===========================================================================
  ;  Misc. functions
  ; ===========================================================================

    Declare.l AdlibRegister(*callback, time.l = 250)
    Declare.l AdlibUnRegister(*callback = 0)
    Declare.l Opt(option.s, value.l = -559038721)
    Declare.l CDTray(drive.s, status.s)
    Declare.l OnAutoItExitRegister(*callback)
    Declare.l OnAutoItExitUnRegister(*callback)

    Macro AutoItSetOption: PureAutoIt::Opt: EndMacro
    Macro SetError(code): PureAutoIt::_Error(code): EndMacro
    Macro SetExtended(code): PureAutoIt::_Extended(code): EndMacro

  ; ===========================================================================
  ;  Mouse functions
  ; ===========================================================================

    Declare.l MouseClick(button.s, x.l = #PB_Ignore, y.l = #PB_Ignore, clicks.l = 1, speed.l = 10)
    Declare.l MouseClickDrag(button.s, x1.l, y1.l, x2.l, y2.l, speed.l = 10)
    Declare.l MouseDown(button.s)
    Declare.l MouseGetCursor()
    Declare.l MouseGetPos(*x, *y = 0)
    Declare.l MouseMove(x.l = #PB_Ignore, y.l = #PB_Ignore, speed.l = 10)
    Declare.l MouseUp(button.s)
    Declare.l MouseWheel_(direction.s, clicks.l = 1)

  ; ===========================================================================
  ;  Network functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Obj/COM functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Process functions
  ; ===========================================================================

    Declare.l ProcessClose(process.s)
    Declare.i ProcessExists(process.s)
    Declare.s ProcessGetStats(process.s = "", type.l = 0)
    Declare.l ProcessSetPriority(process.s, priority.l)
    Declare.l ProcessList(List procs.ProcessInfo(), name.s = "")
    Declare.i ProcessWait(process.s, timeout.l = 0)
    Declare.l ProcessWaitClose(process.s, timeout.l = 0)
    Declare.i Run(program.s, workingdir.s = "", show_flag.l = #SW_NORMAL)
    Declare.i RunWait(program.s, workingdir.s = "", show_flag.l = #SW_NORMAL)
    Declare.i ShellExecute(filename.s, parameters.s = "", workingdir.s = "", verb.s = "", showflag.l = #SW_NORMAL)
    Declare.i ShellExecuteWait(filename.s, parameters.s = "", workingdir.s = "", verb.s = "", showflag.l = #SW_NORMAL)

  ; ===========================================================================
  ;  Registry functions
  ; ===========================================================================

    Declare.l RegDelete(keyname.s, valuename.s = Chr(1))
    Declare.s RegEnumKey(keyname.s, instance.l)
    Declare.s RegEnumVal(keyname.s, instance.l)
    Declare.s RegRead(keyname.s, valuename.s)
    Declare.l RegWrite(keyname.s, valuename.s = Chr(1), type.l = 0, value.s = "")

  ; ===========================================================================
  ;  String functions
  ; ===========================================================================

    ; All string functions don't change _Error And _Extended (except StringRegExp related)

    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
      CompilerIf #PB_Compiler_Unicode
        #PureAutoItObjectFile = "PureAutoItW.obj"
      CompilerElse
        #PureAutoItObjectFile = "PureAutoItA.obj"
      CompilerEndIf
    CompilerElse
      CompilerIf #PB_Compiler_Unicode
        #PureAutoItObjectFile = "PureAutoItW64.obj"
      CompilerElse
        #PureAutoItObjectFile = "PureAutoItA64.obj"
      CompilerEndIf
    CompilerEndIf

    ImportC #PureAutoItObjectFile
      StringIsAlNum(str.s)
      StringIsAlpha(str.s)
      StringIsASCII(str.s)
      StringIsDigit(str.s)
      StringIsFloat(str.s)
      StringIsInt(str.s)
      StringIsLower(str.s)
      StringIsSpace(str.s)
      StringIsUpper(str.s)
      StringIsXDigit(str.s)
      StringCompare(str1.s, str2.s, casesense.l = #False)
      StringInStr(str.s, substr.s, casesense = 0, occurrence = 1, start = 1, count = 0)
    EndImport

    Macro StringLeft: Left: EndMacro
    Macro StringLen: Len: EndMacro
    Macro StringLower: LCase: EndMacro
    Macro StringMid: Mid: EndMacro
    Macro StringReverse: ReverseString: EndMacro
    Macro StringRight: Right: EndMacro
    Macro StringUpper: UCase: EndMacro

    Macro StringTrimLeft(str, n)
      Mid(str, (n) + 1)
    EndMacro

    Macro StringTrimRight(str, n)
      Left(str, Len(str) - (n))
    EndMacro

    Macro StringAddCR(str)
      ReplaceString(str, #LF$, #CRLF$)
    EndMacro

    Macro StringStripCR(str)
      ReplaceString(str, #CR$, "")
    EndMacro

    Declare.s StringReplace(str.s, search.s, replace.s, occur.l = 0, casesense.l = #False)
    Declare.s StringReplaceFrom(str.s, from.l, replace.s)
    Declare.l StringSplit(Array arr.s(1), string.s, delimiters.s, flag = 0)
    Declare.s StringStripWS(str.s, flag.l)
    Declare.s StringFormat(format.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0)
    Declare.s StringRepeat(string.s, count.l)
    Declare.s StringInsert(string.s, insert.s, position.l = 0)
    Declare.l StringToASCIIArray(Array char.u(1), string.s, start.l = 0, last.l = -1, encoding.l = 0)
    Declare.s StringFromASCIIArray(Array char.u(1), start = 0, last = -1, encoding = 0)
    Declare.s HexToString(hex.s, format.l = #PB_Ascii)
    Declare.s StringToHex(string.s, format.l = #PB_Ascii)
    Declare.s MemoryToHex(*src, length)
    Declare.i HexToMemory(hex.s, *buffer = 0)
    Declare.l MemoryStringFormat(*ptr, size.l)

    CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
      Declare.l StringRegExp0(string.s, pattern.s)
      Declare.l StringRegExp1(Array match.s(1), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExp2(Array match.s(1), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExp3(Array match.s(1), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExp4(Array match.StringRegExpMatch(1), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExpList1(List match.s(), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExpList2(List match.s(), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExpList3(List match.s(), string.s, pattern.s, offset.l = 1)
      Declare.l StringRegExpList4(List match.StringRegExpMatch(), string.s, pattern.s, offset.l = 1)
      Declare.s StringRegExpReplace(string.s, pattern.s, replace.s, count = 0)
    CompilerEndIf: CompilerEndIf

  ; ===========================================================================
  ;  Timer and Delay functions
  ; ===========================================================================

    Declare Sleep(delay.l)
    Declare.i TimerInit()
    Declare.l TimerDiff(init.i)

  ; ===========================================================================
  ;  Tray functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Variables and Conversions
  ; ===========================================================================

  ; ===========================================================================
  ;  Window functions
  ; ===========================================================================

    Declare.i WinGetHandle(title.s, text.s = "")
    Declare.i WinActivate(title.s, text.s = "")
    Declare.i WinActive(title.s, text.s = "")
    Declare.i WinClose(title.s, text.s = "")
    Declare.i WinExists(title.s, text.s = "")
    Declare.i WinFlash(title.s, text.s = "", flashes.l = 4, delay.l = 500, wait.l = #True)
    Declare.s WinGetCaretPos()
    Declare.s WinGetClassList(title.s, text.s = "")
    Declare.s WinGetClientSize(title.s, text.s = "")
    Declare.s WinGetPos(title.s, text.s = "")
    Declare.l WinGetProcess(title.s, text.s = "")
    Declare.l WinGetState(title.s, text.s = "")
    Declare.s WinGetTitle(title.s, text.s = "")
    Declare.s WinGetText(title.s, text.s = "")
    Declare.l WinKill(title.s, text.s = "")
    Declare.l WinList(List windows.WinInfo(), title.s = "", text.s = "")
    Declare.l WinMenuSelectItem(title.s, text.s, item0.s, item1.s = "", item2.s ="", item3.s ="", item4.s ="", item5.s ="", item6.s ="", item7.s ="")
    Declare WinMinimizeAll()
    Declare WinMinimizeAllUndo()
    Declare.i WinMove(title.s, text.s = "", x.l = #PB_Ignore, y.l = #PB_Ignore, width.l = #PB_Ignore, height.l = #PB_Ignore, speed.l = 0)
    Declare.i WinSetOnTop(title.s, text.s = "", flag.l = #True)
    Declare.i WinSetState(title.s, text.s, flag.l = #SW_RESTORE)
    Declare.i WinSetTitle(title.s, text.s, string.s)
    Declare.i WinSetTrans(title.s, text.s = "", transparency.l = 128)
    Declare.i WinWait(title.s, text.s = "", timeout.l = 0)
    Declare.i WinWaitActive(title.s, text.s = "", timeout.l = 0)
    Declare.i WinWaitClose(title.s, text.s = "", timeout.l = 0)
    Declare.i WinWaitNotActive(title.s, text.s = "", timeout.l = 0)

  ; ===========================================================================
  ;  Controls functions
  ; ===========================================================================

    Declare.i ControlGetHandle(title.s, text.s, control_id.s)
    Declare.i ControlClick(title.s, text.s, control_id.s, button.s = "", clicks.l = 1, x = #PB_Ignore, y = #PB_Ignore)
    Declare.i ControlSetText(title.s, text.s, control_id.s, string.s)
    Declare.s ControlGetText(title.s, text.s, control_id.s)
    Declare.l ControlDisable(title.s, text.s, control_id.s)
    Declare.l ControlEnable(title.s, text.s, control_id.s)
    Declare.l ControlFocus(title.s, text.s, control_id.s)
    Declare.s ControlGetFocus(title.s, text.s = "")
    Declare.s ControlGetPos(title.s, text.s, control_id.s)
    Declare.l ControlHide(title.s, text.s, control_id.s)
    Declare.l ControlShow(title.s, text.s, control_id.s)
    Declare.l ControlMove(title.s, text.s, control_id.s, x.l = #PB_Ignore, y.l = #PB_Ignore, width.l = #PB_Ignore, height.l = #PB_Ignore)
    Declare.s ControlCommand(title.s, text.s, control_id.s, command.s, option.s = "")
    Declare.i ControlSend(title.s, text.s, control_id.s, keys.s, dbcs = #False)
    Declare.s StatusbarGetText(title.s, text.s = "", part.l = 1)

EndDeclareModule

Module PureAutoIt

  ; ===========================================================================
  ;  Macros, structures, constants
  ; ===========================================================================

    Macro IIf(expr, truepart, falsepart=__expr_null__.b)
      If expr
        truepart
      Else
        falsepart
      EndIf
    EndMacro

    Macro Do(expr1=__expr_null__.b, expr2=__expr_null__.b, expr3=__expr_null__.b, expr4=__expr_null__.b, expr5=__expr_null__.b, expr6=__expr_null__.b, expr7=__expr_null__.b, expr8=__expr_null__.b, expr9=__expr_null__.b, expr10=__expr_null__.b)
      expr1: expr2: expr3: expr4: expr5: expr6: expr7: expr8: expr9: expr10
    EndMacro

    Macro LengthOf(str)
      (SizeOf(str) / SizeOf(Character))
    EndMacro

    Macro Benchmark(expr, times = 100000)
      __begin = ElapsedMilliseconds()
      For __i = 1 To times
        expr
      Next
      Debug "Benchmark: " + Str(ElapsedMilliseconds() - __begin) + "ms"
    EndMacro

    Macro EndianSwitch(value)
      EnableASM
        mov eax, value
        bswap eax
        mov value, eax
      DisableASM
    EndMacro

    Macro IntegerABS(value)
      EnableASM
        mov eax, value
        cdq
        XOr eax, edx
        sub eax, edx
        mov value, eax
      DisableASM
    EndMacro

    Macro ErrorBreak(err, level = 1)
      __error = err
      Break level
    EndMacro

    Macro ReturnBreak(ret, level = 1)
      result = ret
      Break level
    EndMacro

    Macro ErrorReturn(err, ret)
      __error = err
      ProcedureReturn ret
    EndMacro

    Macro ReturnErrorBreak(ret, err = 0, level = 1)
      result = ret
      __error = err
      Break level
    EndMacro

    Macro Error(err)
      __error = err
    EndMacro

    Macro Extended(ext)
      __extended = ext
    EndMacro

    Macro FunctionInit(err = 0, ext = 0)
      __error = err
      __extended = ext
    EndMacro

    Enumeration
      #GetListInclude = 1
      #GetListExclude = 2
      #ContinueEnum = #True
      #StopEnum = #False
      #TextBufferLength = 65535
      #TextBufferSize = (#TextBufferLength + 1) * SizeOf(Character)
      #DEADBEFF = -559038721
    EndEnumeration

    Enumeration
      #MatchStart     = 1
      #MatchSubstring = 2
      #MatchExact     = 3
    EndEnumeration

    Enumeration
      #RelativeToWindow
      #AbsoluteScreen
      #RelativeToClientArea
    EndEnumeration

    Structure KeyInfo
      count.q
      modifier.u
      vk.u
      toggle.a
      extend.a
    EndStructure

    Structure GUIThread
      win.i
      parent_thread.i
      semaphore.i
      init_done.l
    EndStructure

    Structure PureAutoItGUI
      input_win.i
      button_ok.i
      button_cancel.i
      gadget_string.i
      gadget_text.i
      progress.GUIThread
      gadget_maintext.i
      gadget_subtext.i
      gadget_progress.i
      splash.GUIThread
      splash_font.i
      splash_bmp.i
      gadget_image.i
      gadget_splashtext.i
    EndStructure

    Macro AttachThreadInput(hwnd)
      thread_attached.l = #False
      main_thread.i = GetCurrentThreadId_()
      target_thread.i = GetWindowThreadProcessId_(hwnd, 0)
      If target_thread And target_thread <> main_thread
        thread_attached = AttachThreadInput_(main_thread, target_thread, #True)
      EndIf
    EndMacro

    Macro DetachThreadInput()
      If thread_attached
        AttachThreadInput_(main_thread, target_thread, #False)
      EndIf
    EndMacro

  ; ===========================================================================
  ;  Threaded (global) scope variable
  ; ===========================================================================

    CompilerIf #PB_Compiler_Thread
      Macro GlobalThreaded
        Threaded
      EndMacro

      ; make sure some procedure init only once unser multithreaded programs
      Global __global_mutex.i = CreateMutex()
    CompilerElse
      Macro GlobalThreaded
        Global
      EndMacro
    CompilerEndIf

    GlobalThreaded __error.l = 0      ; @error macro
    GlobalThreaded __extended.l = 0   ; @extended macro
    GlobalThreaded __keep_active_hwnd.i = 0   ; SendKeepActive, Send
    GlobalThreaded __custom_button.s          ; MsgBoxEx, MsgBoxEx_CBTProc
    GlobalThreaded __last_hwnd.i = 0          ; WinGetHandle
    GlobalThreaded __tip_hwnd.i               ; ToolTip
    GlobalThreaded NewMap __options.l()       ; Opt
    GlobalThreaded gui.PureAutoItGUI          ; InputBox, ProgressOn, ProgressSet, ProgressOff, SplashTextOn, SplashImageOn, SplashOff

    ; for OnAutoItExitRegister, OnAutoItExitUnRegister
    Declare AtExitHandler()
    Global NewList atexit.i()
    Global exit_code.i, *exit_handler = @AtExitHandler()

  ; ===========================================================================
  ;  Prototype And declarations
  ; ===========================================================================
    Declare SendKeyParse(keys.s, List key_list.KeyInfo())
    Declare CoordAdjust(mode, x.l, y.l, *x, *y, flag = 1, hwnd.i = 0)
    Declare.l DirWalk(source.s, recurse = 0, *callback = 0, *info = 0)

    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
      Import "Kernel32.lib"
        GetProcAddress_(handle.i, name.p-ascii) As "_GetProcAddress@8"
        GetSystemDefaultUILanguage_() As "_GetSystemDefaultUILanguage@0"
        GetUserDefaultUILanguage_() As "_GetUserDefaultUILanguage@0"
        GetNativeSystemInfo_(*psi) As "_GetNativeSystemInfo@4"
        CreateHardLink_(link.p-unicode, path.p-unicode, reserved.l = 0) As "_CreateHardLinkW@12"
        GetProcessId_(process.i) As "_GetProcessId@4"
      EndImport
    CompilerElse
      Import "Kernel32.lib"
        GetProcAddress_(handle.i, name.p-ascii) As "GetProcAddress"
        GetSystemDefaultUILanguage_() As "GetSystemDefaultUILanguage"
        GetUserDefaultUILanguage_() As "GetUserDefaultUILanguage"
        GetNativeSystemInfo_(*psi) As "GetNativeSystemInfo"
        CreateHardLink_(link.p-unicode, path.p-unicode, reserved.l = 0) As "CreateHardLinkW"
        GetProcessId_(process.i) As "GetProcessId"
      EndImport
    CompilerEndIf

    ImportC #PureAutoItObjectFile
      StringSplit_InPlace(str.s, delimiters.s)
      StringStripWS_InPlace(str.s, flag.l)
      StringRepeat_InPlace(str.s, length, *buffer, count.l)
      MemoryToHex_InPlace(*ptr, size.l, *buffer)
      StringReplace_InPlace(str.s, search.s, occurrence.l, casesense.l, *count, *search_len)
      HexToMemory_InPlace(str.s, *ptr)
      XXH_fast32(*input, len.l, seed.l = 0)
    EndImport

    Prototype.l IsWow64Process(handle.i, *iswow64)
    Prototype.l GetNtVersionNumbers(*major, *minor, *build)
    Prototype.l GetProcessMemoryInfo(proc.i, *pmc, size.i)
    Prototype.l GetProcessIoCounters(proc.i, *ic)
    Prototype.i SHEmptyRecycleBin(hwnd.i, path.p-unicode, flag.l)
    Prototype.i SHParseDisplayName(name.p-unicode, *bc, *pidl, sfgaoin, *sfgaoout)
    Prototype.l MessageBoxTimeout(hwnd.i, text.p-unicode, title.p-unicode, type.l, lang.u, timeout.l)

  ; ===========================================================================
  ;  Macros
  ; ===========================================================================

    Procedure.l _Error(value.l = #DEADBEFF)
      IIf(value <> #DEADBEFF, __error = value)
      ProcedureReturn __error
    EndProcedure

    Procedure.l _Extended(value.l = #DEADBEFF)
      IIf(value <> #DEADBEFF, __extended = value)
      ProcedureReturn __extended
    EndProcedure

    Procedure.i _ExitCode()
      ProcedureReturn exit_code
    EndProcedure

    Procedure.s _ScriptDir()
      result.s = GetPathPart(ProgramFilename())
      IIf(Right(result, 2) <> ":\", ProcedureReturn StringTrimRight(result, 1), ProcedureReturn result)
    EndProcedure

    Procedure.s _WorkingDir()
      result.s = GetCurrentDirectory()
      IIf(Right(result, 2) <> ":\", ProcedureReturn StringTrimRight(result, 1), ProcedureReturn result)
    EndProcedure

    Procedure.s _TempDir(csidl.l = -1)
      buffer.s{#MAX_PATH + 1}
      If csidl = -1
        GetTempPath_(#MAX_PATH, @buffer)
      Else
        SHGetSpecialFolderPath_(0, @buffer, csidl, 0)
      EndIf
      ProcedureReturn buffer
    EndProcedure

    Procedure.s _ScriptName()
      ProcedureReturn GetFilePart(ProgramFilename())
    EndProcedure

    Procedure.l _DesktopDepth()
      dm.DEVMODE\dmSize = SizeOf(DEVMODE)
      EnumDisplaySettings_(0, #ENUM_CURRENT_SETTINGS, @dm.DEVMODE)
      ProcedureReturn dm\dmBitsPerPel
    EndProcedure

    Procedure.l _DesktopRefresh()
      dm.DEVMODE\dmSize = SizeOf(DEVMODE)
      EnumDisplaySettings_(0, #ENUM_CURRENT_SETTINGS, @dm.DEVMODE)
      ProcedureReturn dm\dmDisplayFrequency
    EndProcedure

    Procedure.s _KBLayout()
      ProcedureReturn RSet(Hex(GetKeyboardLayout_(0) & $FFFF, #PB_Long), 8, "0")
    EndProcedure

    Procedure.s _OSLang()
      ProcedureReturn RSet(Hex(GetSystemDefaultUILanguage_() & $FFFF, #PB_Long), 4, "0")
    EndProcedure

    Procedure.s _MUILang()
      ProcedureReturn RSet(Hex(GetUserDefaultUILanguage_() & $FFFF, #PB_Long), 4, "0")
    EndProcedure

    Procedure.s _OSArch()
      CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
        ProcedureReturn "X64"
      CompilerElse
        IsWow64Process_.IsWow64Process = GetProcAddress_(GetModuleHandle_("kernel32"), "IsWow64Process")
        IIf(IsWow64Process_ And IsWow64Process_(GetCurrentProcess_(), @iswow64) And iswow64, ProcedureReturn "X64", ProcedureReturn "X86")
      CompilerEndIf
    EndProcedure

    Procedure.s _CPUArch()
      GetNativeSystemInfo_(@si.SYSTEM_INFO)
      IIf(si\wProcessorArchitecture = 9 Or si\wProcessorArchitecture = 6, ProcedureReturn "X64")
      ProcedureReturn "X86"
    EndProcedure

    Procedure.s _OSType()
      ProcedureReturn "WIN32_NT"
    EndProcedure

    Procedure.s _OSVersion()
      vi.OSVERSIONINFOEX\dwOSVersionInfoSize = SizeOf(vi)
      GetVersionEx_(@vi)
      major.l = vi\dwMajorVersion
      minor.l = vi\dwMinorVersion
      IIf(vi\wProductType = #VER_NT_WORKSTATION, is_server.l = #False, is_server = #True)

      GetNtVersionNumbers_.GetNtVersionNumbers = GetProcAddress_(GetModuleHandle_("ntdll.dll"), "RtlGetNtVersionNumbers")
      IIf(GetNtVersionNumbers_, GetNtVersionNumbers_(@major, @minor, @build.l))

      version.s = RSet(Str(major), 2, "0") + RSet(Str(minor), 2, "0")
      ; https://msdn.microsoft.com/zh-tw/library/windows/desktop/ms724834(v=vs.85).aspx
      Select version
        Case "0500": ProcedureReturn "WIN_2000"
        Case "0501": ProcedureReturn "WIN_XP"
        Case "0502": ProcedureReturn "WIN_2003"
        Case "0600": IIf(is_server, ProcedureReturn "WIN_2008", ProcedureReturn "WIN_VISTA")
        Case "0601": IIf(is_server, ProcedureReturn "WIN_2008R2", ProcedureReturn "WIN_7")
        Case "0602": IIf(is_server, ProcedureReturn "WIN_2012", ProcedureReturn "WIN_8")
        Case "0603": IIf(is_server, ProcedureReturn "WIN_2012R2", ProcedureReturn "WIN_81")
        Case "1000": IIf(is_server, ProcedureReturn "WIN_2016", ProcedureReturn "WIN_10")
      EndSelect
    EndProcedure

    Procedure.l _OSBuild()
      vi.OSVERSIONINFOEX\dwOSVersionInfoSize = SizeOf(vi)
      GetVersionEx_(@vi)
      build.l = vi\dwBuildNumber

      GetNtVersionNumbers_.GetNtVersionNumbers = GetProcAddress_(GetModuleHandle_("ntdll.dll"), "RtlGetNtVersionNumbers")
      IIf(GetNtVersionNumbers_, GetNtVersionNumbers_(@major.l, @minor.l, @build))
      ProcedureReturn build & $FFFF
    EndProcedure

    Procedure.s _OSServicePack()
      vi.OSVERSIONINFOEX\dwOSVersionInfoSize = SizeOf(vi)
      GetVersionEx_(@vi)
      ProcedureReturn PeekS(@vi\szCSDVersion)
    EndProcedure

    Procedure.s _ComputerName()
      buffer.s{#MAX_PATH + 1}
      size.l = #MAX_PATH
      GetComputerName_(@buffer, @size)
      ProcedureReturn buffer
    EndProcedure

    Procedure.s _UserName()
      buffer.s{#MAX_PATH}
      size.l = #MAX_PATH
      GetUserName_(@buffer, @size)
      ProcedureReturn buffer
    EndProcedure

    Procedure.s _IPAddress(n.l = 0)
      InitNetwork()
      ExamineIPAddresses()
      For i = 1 To n - 1
        NextIPAddress()
      Next
      ProcedureReturn IPString(NextIPAddress())
    EndProcedure

    Procedure.s _CmdLine(n.l)
      IIf(n <= 0, ProcedureReturn Str(CountProgramParameters()))
      ProcedureReturn ProgramParameter(n + 1)
    EndProcedure

    Procedure.s _CmdLineRaw()
      ProcedureReturn PeekS(GetCommandLine_())
    EndProcedure

  ; ===========================================================================
  ;  Environment functions
  ; ===========================================================================

    Procedure.s ClipGet()
      FunctionInit()
      result.s = GetClipboardText()
      IIf(result = "", Error(1))
      ProcedureReturn result
    EndProcedure

    Procedure.l ClipPut(text.s)
      FunctionInit()
      SetClipboardText(text)
      ProcedureReturn 1
    EndProcedure

    Procedure.s EnvGet(name.s)
      FunctionInit()
      ProcedureReturn GetEnvironmentVariable(name)
    EndProcedure

    Procedure.l EnvSet(name.s, value.s = "")
      FunctionInit()
      SetEnvironmentVariable(name, value)
      ProcedureReturn 1
    EndProcedure

    Procedure.l EnvUpdate()
      FunctionInit()
      result.l = SendMessageTimeout_(#HWND_BROADCAST, #WM_SETTINGCHANGE, 0, @"Environment", #SMTO_BLOCK, 15000, @outcome.l)
      IIf(result = 0, Error(1))
      ProcedureReturn result
    EndProcedure

    Procedure.l MemGetStats(Array status(1))
      FunctionInit()
      ReDim status(6)
      m.MEMORYSTATUSEX\dwLength = SizeOf(MEMORYSTATUSEX)
      GlobalMemoryStatusEx_(@m)
      status(0) = m\dwMemoryLoad
      status(1) = m\ullTotalPhys >> 10
      status(2) = m\ullAvailPhys >> 10
      status(3) = m\ullTotalPageFile >> 10
      status(4) = m\ullAvailPageFile >> 10
      status(5) = m\ullTotalVirtual >> 10
      status(6) = m\ullAvailVirtual >> 10
      ProcedureReturn 6
    EndProcedure

  ; ===========================================================================
  ;  File, Directory and Disk functions
  ; ===========================================================================

    Procedure SetPrivilege(privilege.s = "SeDebugPrivilege")
      Define token.i = 0, token_priv.TOKEN_PRIVILEGES
      Repeat
        token_priv\PrivilegeCount = 1
        token_priv\Privileges\Attributes = #SE_PRIVILEGE_ENABLED
        IIf(OpenProcessToken_(GetCurrentProcess_(), #TOKEN_ADJUST_PRIVILEGES, @token) = 0, Break)
        IIf(LookupPrivilegeValue_(0, privilege, @token_priv\Privileges\Luid) = 0, Break)
        AdjustTokenPrivileges_(token, 0, @token_priv, SizeOf(TOKEN_PRIVILEGES), 0, 0)
      Until #True
      IIf(token, CloseHandle_(token))
    EndProcedure

    Procedure.l IsMemory(*ptr)
      PB_Memory_Heap.i = 0
      EnableASM
      CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
        !extrn _PB_Memory_Heap
        mov eax, [_PB_Memory_Heap]
        mov PB_Memory_Heap, eax
      CompilerElse
        !extrn PB_Memory_Heap
        mov rax, [PB_Memory_Heap]
        mov PB_Memory_Heap, rax
      CompilerEndIf
      DisableASM
      ProcedureReturn HeapValidate_(PB_Memory_Heap, 0, *ptr)
    EndProcedure

    Procedure.l IsFolder(path.s, *attr = 0)
      attr.l = GetFileAttributes_(path)
      IIf(*attr, PokeL(*attr, attr))
      IIf(attr <> -1 And attr & #FILE_ATTRIBUTE_DIRECTORY <> 0, ProcedureReturn #True)
      ProcedureReturn #False
    EndProcedure

    Procedure.s FileGetFullName(path.s)
      buffer.s{#MAX_PATH + 1}
      GetFullPathName_(path, #MAX_PATH, @buffer, 0)
      PathRemoveBackslash_(@buffer)
      ProcedureReturn buffer
    EndProcedure

    Procedure.l FileOperation(action.l, src.s, dest.s = "", flags = #FOF_SILENT | #FOF_NOCONFIRMATION | #FOF_NOERRORUI | #FOF_NOCONFIRMMKDIR)
      new_src.s{#MAX_PATH + 2} = src ; make source\dest double nulled strings
      new_dest.s{#MAX_PATH + 2} = dest
      fo.SHFILEOPSTRUCT\hwnd = 0
      fo\wFunc = action
      fo\pFrom = @new_src
      fo\pTo = @new_dest
      fo\fFlags = flags
      IIf(SHFileOperation_(@fo) = 0, ProcedureReturn 1, ProcedureReturn 0)
    EndProcedure

    Procedure.l DirCreate(path.s)
      FunctionInit()
      dir.s = ""
      For i = 1 To CountString(path, "\") + 1
        dir.s + StringField(path, i, "\") + "\"
        IIf(Not PathFileExists_(dir) And Not CreateDirectory(dir), ProcedureReturn 0)
      Next
      ProcedureReturn 1
    EndProcedure

    Procedure.l DirCopy(source.s, dest.s, overwirte.l = 0)
      FunctionInit()
      IIf(source = "" Or dest = "", ProcedureReturn 0)
      source = FileGetFullName(source)
      dest = FileGetFullName(dest)
      IIf(Not IsFolder(source), ProcedureReturn 0)

      If IsFolder(dest)
        IIf(Not overwirte, ProcedureReturn 0)
      Else
        IIf(Not DirCreate(dest), ProcedureReturn 0)
      EndIf
      ProcedureReturn FileOperation(#FO_COPY, source + "\*.*", dest)
    EndProcedure

    Procedure.l DirRemove(path.s, recurse.l = 0)
      FunctionInit()
      IIf(path = "", ProcedureReturn 0)
      path = FileGetFullName(path)

      IIf(Not recurse, ProcedureReturn RemoveDirectory_(path))
      ProcedureReturn FileOperation(#FO_DELETE, path)
    EndProcedure

    Procedure.l DirMove(source.s, dest.s, overwirte.l = 0)
      FunctionInit()
      IIf(source = "" Or dest = "", ProcedureReturn 0)
      source = FileGetFullName(source)
      dest = FileGetFullName(dest)
      IIf(Not IsFolder(source), ProcedureReturn 0)

      If IsFolder(dest, @attr.l)
        IIf(Not overwirte, ProcedureReturn 0)
      Else
        IIf(attr <> -1, ProcedureReturn 0)
      EndIf

      IIf(FileOperation(#FO_MOVE, source, dest), ProcedureReturn #True)

      ; some OSes fail? copy and delete
      IIf(DirCopy(source, dest, 1) = 0, ProcedureReturn 0)
      ProcedureReturn DirRemove(source, 1)
    EndProcedure

    Structure DirGetSizeInfo
      total_size.q
      file_count.i
      dir_count.i
    EndStructure

    Procedure.l DirGetSizeCallback(path.s, *fd.WIN32_FIND_DATA, *info.DirGetSizeInfo)
      If *fd\dwFileAttributes & #FILE_ATTRIBUTE_DIRECTORY
        *info\dir_count + 1
      Else
        size.q = *fd\nFileSizeHigh << 32 + *fd\nFileSizeLow & $FFFFFFFF
        *info\file_count + 1
        *info\total_size + size
      EndIf
    EndProcedure

    Procedure.q DirGetSize(path.s, mode.l = 0, *file_count = 0, *dir_count = 0)
      recurse.l = #True
      IIf(mode & 2, recurse = #False)

      info.DirGetSizeInfo
      DirWalk(path + "\*.*", recurse, @DirGetSizeCallback(), @info)
      IIf(*file_count, PokeL(*file_count, info\file_count))
      IIf(*dir_count, PokeL(*dir_count, info\dir_count))
      ProcedureReturn info\total_size
    EndProcedure

    Procedure.s DriveGetDrive(type.s = "all", separator.s = ",")
      FunctionInit()
      Define result.s = "", typenum.l, drives.l = GetLogicalDrives_()

      Select LCase(type)
        Case "all":        typenum = -1
        Case "cdrom":      typenum = #DRIVE_CDROM
        Case "removable":  typenum = #DRIVE_REMOVABLE
        Case "fixed":      typenum = #DRIVE_FIXED
        Case "network":    typenum = #DRIVE_REMOTE
        Case "ramdisk":    typenum = #DRIVE_RAMDISK
        Case "unknown":    typenum = #DRIVE_UNKNOWN
        Default: ErrorReturn(1, "")
      EndSelect

      For i = 0 To 25
        If 1 << i & drives
          drive.s = Chr('a' + i) + ":"
          If typenum = -1 Or typenum = GetDriveType_(drive + "\")
            result + drive + separator
          EndIf
        EndIf
      Next

      IIf(Right(result, Len(separator)) = separator, result = StringTrimRight(result, Len(separator)))
      ProcedureReturn result
    EndProcedure

    Procedure.s DriveGetFileSystem(path.s)
      FunctionInit()
      result.s{#MAX_PATH + 1}
      IIf(GetVolumeInformation_(path, 0, 0, 0, 0, 0, @result, #MAX_PATH) = 0, ErrorReturn(1, ""))
      ProcedureReturn result
    EndProcedure

    Procedure.s DriveGetLabel(path.s)
      FunctionInit()
      result.s{#MAX_PATH + 1}
      IIf(GetVolumeInformation_(path, @result, #MAX_PATH, 0, 0, 0, 0, 0) = 0, ErrorReturn(1, ""))
      ProcedureReturn result
    EndProcedure

    Procedure.s DriveGetSerial(path.s)
      FunctionInit()
      IIf(GetVolumeInformation_(path, 0, 0, @serial.l, 0, 0, 0, 0) = 0, ErrorReturn(1, ""))
      ProcedureReturn Str(serial)
    EndProcedure

    #IOCTL_STORAGE_QUERY_PROPERTY = $2D1400
    #IOCTL_STORAGE_GET_MEDIA_SERIAL_NUMBER = $2D0C10
    #StorageDeviceSeekPenaltyProperty = 7
    #StorageDeviceTrimProperty = 8

    Structure STORAGE_PROPERTY_QUERY
      PropertyId.l
      QueryType.l
      AdditionalParameters.l
    EndStructure

    Structure DEVICE_TRIM_DESCRIPTOR
      Version.l
      Size.l
      TrimEnabled.l
    EndStructure

    Structure DEVICE_SEEK_PENALTY_DESCRIPTOR
      Version.l
      Size.l
      IncursSeekPenalty.l
    EndStructure

    Structure STORAGE_DEVICE_DESCRIPTOR
      Version.l
      Size.l
      DeviceType.b
      DeviceTypeModifier.b
      RemovableMedia.b
      CommandQueueing.b
      VendorIdOffset.l
      ProductIdOffset.l
      ProductRevisionOffset.l
      SerialNumberOffset.l
      BusType.w
      RawPropertiesLength.l
      RawDeviceProperties.b
      Reserved.b[1024]
    EndStructure

    Enumeration
      #BusTypeUnknown            = $0
      #BusTypeScsi               = $1
      #BusTypeAtapi              = $2
      #BusTypeAta                = $3
      #BusType1394               = $4
      #BusTypeSsa                = $5
      #BusTypeFibre              = $6
      #BusTypeUsb                = $7
      #BusTypeRAID               = $8
      #BusTypeiScsi              = $9
      #BusTypeSas                = $A
      #BusTypeSata               = $B
      #BusTypeSd                 = $C
      #BusTypeMmc                = $D
      #BusTypeVirtual            = $E
      #BusTypeFileBackedVirtual  = $F
      #BusTypeMax                = $10
      #BusTypeMaxReserved        = $7
    EndEnumeration

    Procedure.s DriveGetType(path.s, operation.l = #DT_DRIVETYPE)
      FunctionInit()
      Define result.s = "", drive.s = Left(path, 2), spq.STORAGE_PROPERTY_QUERY

      If operation = #DT_SSDSTATUS Or operation = #DT_BUSTYPE
        IIf(StringIsDigit(drive), device.s = "\\.\PhysicalDrive" + drive, device = "\\.\" + drive)
        handle.i = CreateFile_(device, 0, #FILE_SHARE_READ | #FILE_SHARE_WRITE, 0, #OPEN_EXISTING, 0, 0)
        IIf(handle = #INVALID_HANDLE_VALUE, ErrorReturn(1, ""))

        If operation = #DT_SSDSTATUS
          If result = ""
            spq\PropertyId = #StorageDeviceTrimProperty
            spq\QueryType = 0
            If DeviceIoControl_(handle, #IOCTL_STORAGE_QUERY_PROPERTY, @spq, SizeOf(spq), @dtd.DEVICE_TRIM_DESCRIPTOR, SizeOf(dtd), @bytes, 0) And dtd\TrimEnabled
              result = "SSD"
            EndIf
          EndIf

          If result = ""
            spq\PropertyId = #StorageDeviceSeekPenaltyProperty
            spq\QueryType = 0
            If DeviceIoControl_(handle, #IOCTL_STORAGE_QUERY_PROPERTY, @spq, SizeOf(spq), @dspd.DEVICE_SEEK_PENALTY_DESCRIPTOR, SizeOf(dspd), @bytes, 0) And dspd\IncursSeekPenalty = 0
              result = "SSD"
            EndIf
          EndIf
        Else ; operation = #DT_BUSTYPE
          If DeviceIoControl_(handle, #IOCTL_STORAGE_QUERY_PROPERTY, @spq, SizeOf(spq), @sdd.STORAGE_DEVICE_DESCRIPTOR, SizeOf(sdd), @bytes, 0)
            Select sdd\Bustype
              Case #BusTypeScsi:               result = "SCSI"
              Case #BusTypeAtapi:              result = "ATAPI"
              Case #BusTypeAta:                result = "ATA"
              Case #BusType1394:               result = "1394"
              Case #BusTypeSsa:                result = "SSA"
              Case #BusTypeFibre:              result = "Fibre"
              Case #BusTypeUsb:                result = "USB"
              Case #BusTypeRAID:               result = "RAID"
              Case #BusTypeiScsi:              result = "iSCSI"
              Case #BusTypeSas:                result = "SAS"
              Case #BusTypeSata:               result = "SATA"
              Case #BusTypeSd:                 result = "SD"
              Case #BusTypeMmc:                result = "MMC"
              Case #BusTypeVirtual:            result = "Virtual"
              Case #BusTypeFileBackedVirtual:  result = "FileBackedVirtual"
              Default:                         result = "Unknown"
            EndSelect
          EndIf
        EndIf
        CloseHandle_(handle)
        ProcedureReturn result

      Else ; operation = #DT_DRIVETYPE
        Select GetDriveType_(drive + "\")
          Case #DRIVE_CDROM:     result = "CDROM"
          Case #DRIVE_REMOVABLE: result = "Removable"
          Case #DRIVE_FIXED:     result = "Fixed"
          Case #DRIVE_REMOTE:    result = "Network"
          Case #DRIVE_RAMDISK:   result = "RAMDisk"
          Case #DRIVE_UNKNOWN:   result = "Unknown"
          Default: ErrorReturn(1, "")
        EndSelect
        ProcedureReturn result
      EndIf
    EndProcedure

    Procedure.s DriveMapAdd(device.s, remote.s, flags = 0, user.s = "", password.s = "")
      FunctionInit()
      #CONNECT_REDIRECT = $80
      Define nr.NETRESOURCE, *user = 0, *password = 0, buffer.s{64}, size.l = 64

      If Left(device, 1) = "*"
        nr\dwType = #RESOURCETYPE_DISK
        nr\lpLocalName = 0
      Else
        nr\dwType = #RESOURCETYPE_ANY
        nr\lpLocalName = @device
      EndIf
      nr\lpRemoteName = @remote
      nr\lpProvider = 0
      IIf(user <> "", *user = @user)
      IIf(password <> "", *password = @password)

      Extended(WNetUseConnection_(0, @nr, *password, *user, flags | #CONNECT_REDIRECT, @buffer, @size, @outcome.l))
      Select _Extended()
        Case #ERROR_SUCCESS:                    ProcedureReturn buffer
        Case #ERROR_ACCESS_DENIED:              Error(2)
        Case #ERROR_ALREADY_ASSIGNED:           Error(3)
        Case #ERROR_DEVICE_ALREADY_REMEMBERED:  Error(3)
        Case #ERROR_BAD_DEVICE:                 Error(4)
        Case #ERROR_BAD_NET_NAME:               Error(5)
        Case #ERROR_INVALID_PASSWORD:           Error(6)
        Default:                                Error(1)
      EndSelect
      ProcedureReturn ""
    EndProcedure

    Procedure.l DriveMapDel(drive.s)
      FunctionInit()
      IIf(WNetCancelConnection2_(drive, 0, 1) = 0, ProcedureReturn 1, ProcedureReturn 0)
    EndProcedure

    Procedure.s DriveMapGet(device.s)
      FunctionInit()
      IIf(WNetGetConnection_(device, 0, @length.l) <> #ERROR_MORE_DATA, ErrorReturn(1, ""))
      *buffer = AllocateMemory(length * SizeOf(Character) + 4)
      IIf(*buffer = 0, ErrorReturn(1, ""))

      WNetGetConnection_(device, *buffer, @length.l)
      result.s = PeekS(*buffer)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.l DriveSetLabel(path.s, label.s)
      FunctionInit()
      ProcedureReturn SetVolumeLabel_(path, label)
    EndProcedure

    Procedure.d DriveSpaceFree(path.s)
      FunctionInit()
      IIf(GetDiskFreeSpaceEx_(path, @avail.q, @total.q, @free.q), ProcedureReturn total / 1048576)
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.d DriveSpaceTotal(path.s)
      FunctionInit()
      IIf(GetDiskFreeSpaceEx_(path, @avail.q, @total.q, @free.q), ProcedureReturn free / 1048576)
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.s DriveStatus(path.s)
      FunctionInit()
      IIf(GetDiskFreeSpace_(path, @spc.l, @bps.l, @fc.l, @tc.l), ProcedureReturn "READY")
      Select GetLastError_()
        Case #ERROR_PATH_NOT_FOUND: ProcedureReturn "INVALID"
        Case #ERROR_NOT_READY:      ProcedureReturn "NOTREADY"
      EndSelect
      ProcedureReturn "UNKNOWN"
    EndProcedure

    Procedure.s ExpandFilenameWildcardPart(source.s, dest.s)
      result.s = ReplaceString(dest, "*", source, 0, 1, 1)
      ProcedureReturn ReplaceString(result, "*", "")
    EndProcedure

    Procedure.s ExpandFilenameWildcard(source.s, dest.s)
      IIf(FindString(dest, "*") = 0, ProcedureReturn dest)
      dest_ext.s = ExpandFilenameWildcardPart(GetExtensionPart(source), GetExtensionPart(dest))
      dest_file.s = ExpandFilenameWildcardPart(GetFilePart(source, #PB_FileSystem_NoExtension), GetFilePart(dest, #PB_FileSystem_NoExtension))
      IIf(dest_ext <> "", dest_file + "." + dest_ext)
      ProcedureReturn GetPathPart(dest) + dest_file
    EndProcedure

    Procedure.l FileCopyMove(source.s, dest.s, overwrite.l = 0, move.l = 0)
      IIf(source = "" Or dest = "", ProcedureReturn 0)

      source = FileGetFullName(source)
      dest = FileGetFullName(dest)
      IIf(IsFolder(source), source + "\*.*")
      IIf(IsFolder(dest), dest + "\*.*")

      search.i = FindFirstFile_(source, @fd.WIN32_FIND_DATA)
      IIf(search = #INVALID_HANDLE_VALUE, ProcedureReturn 0)

      Define result.l = 0, source_path.s = GetPathPart(source)
      IIf(overwrite, fail_if_exists.i = #False, fail_if_exists = #True)
      Repeat
        IIf(fd\dwFileAttributes & #FILE_ATTRIBUTE_DIRECTORY, Continue)
        source_expand.s = source_path + PeekS(@fd\cFileName)
        dest_expand.s = ExpandFilenameWildcard(source_expand, dest)

        If move
          If MoveFile_(source_expand, dest_expand)
            result + 1
          ElseIf overwrite And DeleteFile_(dest_expand) And MoveFile_(source_expand, dest_expand)
            result + 1
          EndIf
        Else
          IIf(CopyFile_(source_expand, dest_expand, fail_if_exists), result + 1)
        EndIf
      Until FindNextFile_(search, @fd) = 0
      FindClose_(search)
      ProcedureReturn result
    EndProcedure

    Procedure.l FileCopy(source.s, dest.s, flag = #FC_NOOVERWRITE)
      FunctionInit()
      IIf(source = "" Or dest = "", ProcedureReturn 0)
      IIf(flag & #FC_CREATEPATH, DirCreate(GetPathPart(dest)))
      ProcedureReturn FileCopyMove(source, dest, flag & #FC_OVERWRITE, 0)
    EndProcedure

    Procedure.l FileMove(source.s, dest.s, flag = #FC_NOOVERWRITE)
      FunctionInit()
      IIf(source = "" Or dest = "", ProcedureReturn 0)
      IIf(flag & #FC_CREATEPATH, DirCreate(GetPathPart(dest)))
      ProcedureReturn FileCopyMove(source, dest, flag & #FC_OVERWRITE, 1)
    EndProcedure

    Procedure.l FileDelete(filename.s, force.l = 0)
      FunctionInit()
      IIf(filename = "", ProcedureReturn 0)
      IIf(force, flag.l = #PB_FileSystem_Force, flag = 0)
      IIf(Right(filename, 1) <> "\" And IsFolder(filename), filename + "\")

      Define path.s = GetPathPart(filename), file.s = GetFilePart(filename)
      Define result.l = 0, search.i = ExamineDirectory(#PB_Any, path, file)
      IIf(search = 0, ProcedureReturn 0)

      While NextDirectoryEntry(search)
        If DirectoryEntryType(search) = #PB_DirectoryEntry_File
          IIf(DeleteFile(path + DirectoryEntryName(search), flag), result + 1)
        EndIf
      Wend
      FinishDirectory(search)
      ProcedureReturn result
    EndProcedure

    Procedure.l FileRecycle(source.s)
      FunctionInit()
      #FOF_WANTNUKEWARNING = $4000
      IIf(source = "", ProcedureReturn 0)
      source = FileGetFullName(source)
      ProcedureReturn FileOperation(#FO_DELETE, source, "", #FOF_SILENT | #FOF_ALLOWUNDO | #FOF_NOCONFIRMATION | #FOF_WANTNUKEWARNING)
    EndProcedure

    Procedure.l FileRecycleEmpty(path.s = "")
      FunctionInit()
      #SHERB_NOCONFIRMATION = $000001
      #SHERB_NOPROGRESSUI   = $000002
      #SHERB_NOSOUND        = $000004
      Define result.l = 0, lib.i = 0
      Repeat
        lib = OpenLibrary(#PB_Any, "shell32.dll")
        IIf(lib = 0, Break)

        SHEmptyRecycleBin_.SHEmptyRecycleBin = GetFunction(lib, "SHEmptyRecycleBinW")
        IIf(SHEmptyRecycleBin_ = 0, Break)

        IIf(SHEmptyRecycleBin_(0, path, #SHERB_NOCONFIRMATION | #SHERB_NOPROGRESSUI | #SHERB_NOSOUND) = #S_OK, result = #True)
      Until #True
      IIf(lib, CloseLibrary(lib))
      ProcedureReturn result
    EndProcedure

    Procedure.s FileGetAttrib(filename.s)
      FunctionInit()
      attr.l = GetFileAttributes_(filename)
      IIf(attr = -1, ErrorReturn(1, ""))
      result.s = ""
      IIf(attr & #FILE_ATTRIBUTE_READONLY,   result + "R")
      IIf(attr & #FILE_ATTRIBUTE_ARCHIVE,    result + "A")
      IIf(attr & #FILE_ATTRIBUTE_SYSTEM,     result + "S")
      IIf(attr & #FILE_ATTRIBUTE_HIDDEN,     result + "H")
      IIf(attr & #FILE_ATTRIBUTE_NORMAL,     result + "N")
      IIf(attr & #FILE_ATTRIBUTE_DIRECTORY,  result + "D")
      IIf(attr & #FILE_ATTRIBUTE_OFFLINE,    result + "O")
      IIf(attr & #FILE_ATTRIBUTE_COMPRESSED, result + "C")
      IIf(attr & #FILE_ATTRIBUTE_TEMPORARY,  result + "T")
      ProcedureReturn result
    EndProcedure

    Procedure.l DirWalk(source.s, recurse = 0, *callback = 0, *info = 0)
      source = FileGetFullName(source)
      Define path.s = GetPathPart(source), pattern.s = GetFilePart(source)
      Define search.i = FindFirstFile_(source, @fd.WIN32_FIND_DATA), count.l = 0
      If search <> #INVALID_HANDLE_VALUE
        Repeat
          name.s = PeekS(@fd\cFileName)
          IIf(name = "." Or name = "..", Continue)

          IIf(*callback, CallFunctionFast(*callback, @path, @fd, *info))
          count + 1
        Until FindNextFile_(search, @fd) = 0
        FindClose_(search)
      EndIf

      If recurse
        Define search.i = FindFirstFile_(path + "*.*", @fd.WIN32_FIND_DATA)
        If search <> #INVALID_HANDLE_VALUE
          Repeat
            name.s = PeekS(@fd\cFileName)
            IIf(name = "." Or name = "..", Continue)

            If fd\dwFileAttributes & #FILE_ATTRIBUTE_DIRECTORY
              count + DirWalk(path + name + "\" + pattern, recurse, *callback, *info)
            EndIf
          Until FindNextFile_(search, @fd) = 0
          FindClose_(search)
        EndIf
      EndIf
      ProcedureReturn count
    EndProcedure

    Structure FileSetAttribInfo
      set.s
      error_count.l
    EndStructure

    Procedure.l FileSetAttribCallback(path.s, *fd.WIN32_FIND_DATA, *info.FileSetAttribInfo)
      Define attr.l = *fd\dwFileAttributes, name.s = PeekS(@*fd\cFileName), set.s = *info\set, mode.l = -1
      For i = 0 To Len(set) - 1
        Define bit.l = 0, ch.c = PeekC(@set + i * SizeOf(Character))
        Select ch
          Case '-': mode = 0
          Case '+': mode = 1
          Case '^': mode = 2
          Case 'R': bit = #FILE_ATTRIBUTE_READONLY
          Case 'A': bit = #FILE_ATTRIBUTE_ARCHIVE
          Case 'S': bit = #FILE_ATTRIBUTE_SYSTEM
          Case 'H': bit = #FILE_ATTRIBUTE_HIDDEN
          Case 'N': bit = #FILE_ATTRIBUTE_NORMAL
          Case 'O': bit = #FILE_ATTRIBUTE_OFFLINE
          Case 'T': bit = #FILE_ATTRIBUTE_TEMPORARY
        EndSelect

        If bit
          Select mode
            Case 0: attr & (~bit)
            Case 1: attr | bit
            Case 2: attr ! bit
          EndSelect
        EndIf
      Next

      IIf(SetFileAttributes_(path + name, attr) = 0, *info\error_count + 1)
    EndProcedure

    Procedure.l FileSetAttrib(source.s, set.s, recurse = 0)
      FunctionInit()
      info.FileSetAttribInfo
      info\set = set
      DirWalk(source, recurse, @FileSetAttribCallback(), @info)
      IIf(info\error_count <> 0, ProcedureReturn 0, ProcedureReturn 1)
    EndProcedure

    Procedure.s FileGetVersion(filename.s, name.s = "")
      FunctionInit()
      Define result.s = "", *buffer = 0
      Repeat
        size.l = GetFileVersionInfoSize_(filename, @unused.i)
        IIf(size = 0, Break)

        *buffer = AllocateMemory(size)
        IIf(*buffer = 0, Break)
        IIf(GetFileVersionInfo_(filename, 0, size, *buffer) = 0, Break)

        name = LCase(name)
        If name = ""
          IIf(VerQueryValue_(*buffer, "\", @*ffi.VS_FIXEDFILEINFO, @bytes) = 0, Break)
          result = Str(*ffi\dwFileVersionMS >> 16) + "." + Str(*ffi\dwFileVersionMS & $FFFF) + "."
          result + Str(*ffi\dwFileVersionLS >> 16) + "." + Str(*ffi\dwFileVersionLS & $FFFF)
          ReturnBreak(result)

        Else
          IIf(VerQueryValue_(*buffer, "\VarFileInfo\Translation", @*lang, @bytes) = 0, Break)
          IIf(bytes < 4, Break)

          codepage.s = RSet(Hex(PeekU(*lang)), 4, "0") + RSet(Hex(PeekU(*lang + 2)), 4, "0")
          If name = "defaultlangcodepage"
            ReturnBreak(codepage)

          Else
            If StringIsXDigit(StringField(name, 1, "\"))
              query.s = "\StringFileInfo\" + name
            Else
              query.s = "\StringFileInfo\" + codepage + "\" + name
            EndIf

            IIf(VerQueryValue_(*buffer, query, @*ptr, @bytes) = 0, Break)
            ReturnBreak(PeekS(*ptr))
          EndIf
        EndIf

      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      If result = ""
        Error(1)
        IIf(name = "", ProcedureReturn "0.0.0.0")
        IIf(name <> "", ProcedureReturn "")
      EndIf
      ProcedureReturn result
    EndProcedure

    Structure SYMBOLIC_LINK_REPARSE_BUFFER
      SubstituteNameOffset.w
      SubstituteNameLength.w
      PrintNameOffset.w
      PrintNameLength.w
      Flags.l
      PathBuffer.u[1]
    EndStructure

    Structure MOUNT_POINT_REPARSE_BUFFER
      SubstituteNameOffset.w
      SubstituteNameLength.w
      PrintNameOffset.w
      PrintNameLength.w
      PathBuffer.u[1]
    EndStructure

    Structure REPARSE_DATA_BUFFER
      ReparseTag.l
      ReparseDataLength.w
      Reserved.w
      StructureUnion
        SymbolicLinkReparseBuffer.SYMBOLIC_LINK_REPARSE_BUFFER
        MountPointReparseBuffer.MOUNT_POINT_REPARSE_BUFFER
        GenericReparseBuffer.u[1]
      EndStructureUnion
    EndStructure

    Structure REPARSE_MOUNTPOINT_DATA_BUFFER
      ReparseTag.l
      ReparseDataLength.l
      Reserved.w
      ReparseTargetLength.w
      ReparseTargetMaximumLength.w
      Reserved1.w
      ReparseTarget.u[1]
    EndStructure

    #FILE_FLAG_OPEN_REPARSE_POINT = $200000
    #DIR_ATTR = #FILE_ATTRIBUTE_DIRECTORY | #FILE_ATTRIBUTE_REPARSE_POINT
    #MAXIMUM_REPARSE_DATA_BUFFER_SIZE = 16 * 1024
    #FSCTL_GET_REPARSE_POINT = $900A8
    #FSCTL_SET_REPARSE_POINT = $900A4
    #IO_REPARSE_TAG_MOUNT_POINT = $A0000003

    Procedure.i OpenDirectory(path.s, readwrite.l = 0)
      IIf(readwrite, SetPrivilege("SeRestorePrivilege"), SetPrivilege("SeBackupPrivilege"))
      IIf(readwrite, access.l = #GENERIC_READ | #GENERIC_WRITE, access.l = #GENERIC_READ)
      ProcedureReturn CreateFile_(path, access, 0, 0, #OPEN_EXISTING, #FILE_FLAG_OPEN_REPARSE_POINT | #FILE_FLAG_BACKUP_SEMANTICS, 0)
    EndProcedure

    Procedure.l IsJunction(path.s)
      attr.l = GetFileAttributes_(path)
      IIf(attr = -1 Or attr & #DIR_ATTR <> #DIR_ATTR, ProcedureReturn #False)

      handle.i = OpenDirectory(path, 0)
      IIf(handle = -1, ProcedureReturn #False)

      buffer.s{#MAXIMUM_REPARSE_DATA_BUFFER_SIZE / SizeOf(Character)}
      succeed.l = DeviceIoControl_(handle, #FSCTL_GET_REPARSE_POINT, 0, 0, @buffer, SizeOf(buffer), @ret, 0)
      CloseHandle_(handle)

      *p.REPARSE_DATA_BUFFER = @buffer
      IIf(succeed And *p\ReparseTag = #IO_REPARSE_TAG_MOUNT_POINT, ProcedureReturn #True, ProcedureReturn #False)
    EndProcedure

    Procedure.l CreateJunction(path.s, target.s)
      IIf(FileSize(path) = -1, CreateDirectory_(path, 0))
      handle.i = OpenDirectory(path, 1)
      IIf(handle = -1, ProcedureReturn #False)

      Define result.l = 0, *buffer.REPARSE_MOUNTPOINT_DATA_BUFFER = 0
      Repeat
        *buffer = AllocateMemory(SizeOf(REPARSE_MOUNTPOINT_DATA_BUFFER) + (32768 + Len(target)) * SizeOf(Character))
        IIf(*buffer = 0, Break)
        IIf(GetFullPathName_(target, 32768, *buffer, 0) = 0, Break)

        target = "\??\" + PeekS(*buffer) + "\"
        length.l = Len(target)
        FillMemory(*buffer, MemorySize(*buffer))

        *buffer\ReparseTag = #IO_REPARSE_TAG_MOUNT_POINT
        *buffer\ReparseTargetMaximumLength = (length + 1) * 2
        *buffer\ReparseTargetLength = length * 2
        *buffer\ReparseDataLength =  length * 2 + 12
        PokeS(@*buffer\ReparseTarget, target, -1, #PB_Unicode)

        buffer_size = *buffer\ReparseDataLength + 8
        result = DeviceIoControl_(handle, #FSCTL_SET_REPARSE_POINT, *buffer, buffer_size, 0, 0, @buffer_size, 0)
      Until #True
      CloseHandle_(handle)
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Procedure.l FileCreateNTFSLink(source.s, link.s, overwrite.l = 0)
      FunctionInit()
      Select FileSize(source)
      Case -1 ; not found
        ProcedureReturn #False

      Case -2 ; is dir
        IIf(FileSize(link) <> -1 And overwrite = 0, ProcedureReturn #False)
        ProcedureReturn CreateJunction(link, source)

      Default ; is file
        IIf(FileSize(link) >= 0 And overwrite, DeleteFile(link, #PB_FileSystem_Force))
        ProcedureReturn CreateHardLink_(link, source)
      EndSelect
    EndProcedure

    DataSection
      CLSID_ShellLink:
        ; 00021401-0000-0000-C000-000000000046
        Data.l $00021401
        Data.w $0000,$0000
        Data.b $C0,$00,$00,$00,$00,$00,$00,$46

      IID_IShellLinkA:
        ; 000214EE-0000-0000-C000-000000000046
        Data.l $000214EE
        Data.w $0000,$0000
        Data.b $C0,$00,$00,$00,$00,$00,$00,$46

      IID_IShellLinkW:
        ; 000214F9-0000-0000-C000-000000000046
        Data.l $000214F9
        Data.w $0000,$0000
        Data.b $C0,$00,$00,$00,$00,$00,$00,$46

      IID_IPersistFile:
        ; 0000010b-0000-0000-C000-000000000046
        Data.l $0000010b
        Data.w $0000,$0000
        Data.b $C0,$00,$00,$00,$00,$00,$00,$46
    EndDataSection

    Procedure.l FileCreateShortcut(file.s, link.s, workdir.s = "", args.s = "", desc.s = "", icon.s = "", hotkey.s = "", icon_index.l = 0, state.l = 0)
      FunctionInit()
      CoInitialize_(0)
      Define result.l = 0, buffer.s{#MAX_PATH + 1}, shell_link.IShellLinkW = 0, link_file.IPersistFile = 0
      Repeat
        IIf(CoCreateInstance_(?CLSID_ShellLink, 0, 1, ?IID_IShellLinkW, @shell_link) <> #S_OK, Break)
        IIf(GetFullPathName_(file, #MAX_PATH, @buffer, 0) = 0, Break)
        IIf(shell_link\SetPath(buffer) <> #S_OK, Break)

        IIf(args <> "" And shell_link\SetArguments(args) <> #S_OK, Break)
        IIf(desc <> "" And shell_link\SetDescription(desc) <> #S_OK, Break)
        IIf(workdir <> "" And shell_link\SetWorkingDirectory(workdir) <> #S_OK, Break)
        IIf(icon <> "" And shell_link\SetIconLocation(icon, icon_index) <> #S_OK, Break)
        IIf(state <> 0 And shell_link\SetShowCmd(state) <> #S_OK, Break)

        If hotkey <> ""
          NewList key_list.KeyInfo()
          SendKeyParse(hotkey.s, key_list())

          IIf(SelectElement(key_list(), 0) = 0, Break)
          IIf(key_list()\toggle <> 0, Break)
          IIf(key_list()\modifier & #MOD_WIN, Break)

          modifier.u = 0
          IIf(key_list()\modifier & #MOD_ALT, modifier | #HOTKEYF_ALT)
          IIf(key_list()\modifier & #MOD_CONTROL, modifier | #HOTKEYF_CONTROL)
          IIf(key_list()\modifier & #MOD_SHIFT, modifier | #HOTKEYF_SHIFT)

          IIf(shell_link\SetHotkey(modifier << 8 | key_list()\vk) <> #S_OK, Break)
        EndIf

        IIf(shell_link\QueryInterface(?IID_IPersistFile, @link_file) <> #S_OK, Break)
        IIf(LCase(Right(link, 4)) <> ".lnk", link + ".lnk")
        IIf(link_file\Save(link, #True) <> #S_OK, Break)
        result = #True
      Until #True
      IIf(link_file, link_file\Release())
      IIf(shell_link, shell_link\Release())
      CoUninitialize_()
      ProcedureReturn result
    EndProcedure

    Procedure.l FileGetShortcut(link.s, Array result.s(1))
      FunctionInit()
      CoInitialize_(0)
      Define result.l = 0, *buffer = 0, shell_link.IShellLinkW = 0, link_file.IPersistFile = 0
      Repeat
        *buffer = AllocateMemory(32768 * SizeOf(Character))
        IIf(*buffer = 0, Break)

        IIf(CoCreateInstance_(?CLSID_ShellLink, 0, 1, ?IID_IShellLinkW, @shell_link) <> #S_OK, Break)
        IIf(shell_link\QueryInterface(?IID_IPersistFile, @link_file) <> #S_OK, Break)
        IIf(link_file\Load(link, #STGM_READ) <> #S_OK, Break)
        IIf(shell_link\Resolve(0, 1) <> #S_OK, Break)

        ReDim result(7)
        IIf(shell_link\GetPath(*buffer, 32767, 0, 0) = #S_OK, result(0) = PeekS(*buffer))
        IIf(shell_link\GetWorkingDirectory(*buffer, 32767) = #S_OK, result(1) = PeekS(*buffer))
        IIf(shell_link\GetArguments(*buffer, 32767) = #S_OK, result(2) = PeekS(*buffer))
        IIf(shell_link\GetDescription(*buffer, 32767) = #S_OK, result(3) = PeekS(*buffer))
        IIf(shell_link\GetShowCmd(@state.l) = #S_OK, result(6) = Str(state))
        If shell_link\GetIconLocation(*buffer, 32767, @icon_index.l) = #S_OK
          result(4) = PeekS(*buffer)
          result(5) = Str(icon_index)
        EndIf
        result = #True
      Until #True
      IIf(link_file, link_file\Release())
      IIf(shell_link, shell_link\Release())
      IIf(*buffer, FreeMemory(*buffer))
      CoUninitialize_()
      IIf(result = 0, Error(1))
      ProcedureReturn result
    EndProcedure

    Structure FileFindInfo
      fd.WIN32_FIND_DATA
      search.i
      used.l
    EndStructure

    Procedure.i FileFindFirstFile(pattern.s)
      FunctionInit()
      *ffi.FileFindInfo = AllocateMemory(SizeOf(FileFindInfo))
      IIf(*ffi = 0, ErrorReturn(2, -1))
      Repeat
        *ffi\search = FindFirstFile_(pattern, *ffi)
        IIf(*ffi\search = #INVALID_HANDLE_VALUE, Break)

        Repeat
          name.s = PeekS(@*ffi\fd\cFileName)
          IIf(name <> "." And name <> "..", Break)
          IIf(FindNextFile_(*ffi\search, *ffi) = 0, Break 2)
        ForEver

        *ffi\used = 0
        ProcedureReturn *ffi
      Until #True
      IIf(*ffi\search <> #INVALID_HANDLE_VALUE, FindClose_(*ffi\search))
      FreeMemory(*ffi)
      ErrorReturn(1, -1)
    EndProcedure

    Procedure.s FileFindNextFile(handle.i)
      FunctionInit()
      *ffi.FileFindInfo = handle
      IIf(Not IsMemory(*ffi), ErrorReturn(2, ""))
      If *ffi\used = 0
        *ffi\used = 1
        ProcedureReturn PeekS(@*ffi\fd\cFileName)
      ElseIf FindNextFile_(*ffi\search, *ffi)
        ProcedureReturn PeekS(@*ffi\fd\cFileName)
      Else
        ErrorReturn(1, "")
      EndIf
    EndProcedure

    Procedure.l FileFindClose(handle.i)
      FunctionInit()
      *ffi.FileFindInfo = handle
      IIf(Not IsMemory(*ffi), ErrorReturn(2, 0))
      result.l = FindClose_(*ffi\search)
      FreeMemory(*ffi)
      IIf(result = 0, Error(1))
      ProcedureReturn result
    EndProcedure

    Procedure.s FileGetLongName(path.s, flag = 0)
      FunctionInit()
      buffer.s{#MAX_PATH + 1}
      IIf(flag, GetFullPathName_(path, #MAX_PATH, @buffer, 0), buffer = path)
      IIf(GetLongPathName_(buffer, @buffer, #MAX_PATH), ProcedureReturn buffer)
      ErrorReturn(1, path)
    EndProcedure

    Procedure.s FileGetShortName(path.s, flag = 0)
      FunctionInit()
      buffer.s{#MAX_PATH + 1}
      IIf(flag, GetFullPathName_(path, #MAX_PATH, @buffer, 0), buffer = path)
      IIf(GetShortPathName_(buffer, @buffer, #MAX_PATH), ProcedureReturn buffer)
      ErrorReturn(1, path)
    EndProcedure

    Procedure.l FileExists(path.s)
      FunctionInit()
      ProcedureReturn PathFileExists_(path)
    EndProcedure

    Procedure.l FileChangeDir(dir.s)
      FunctionInit()
      ProcedureReturn SetCurrentDirectory(dir)
    EndProcedure

    Procedure.l FileFormat(file.s, count.l = 64 * 1024)
      Define result.l = 0, *buffer, handle.i = ReadFile(#PB_Any, file)
      Repeat
        IIf(handle = 0, ReturnBreak(0))
        IIf(count < 0, count = Lof(handle))
        IIf(count < 64, count = 64)
        *buffer = AllocateMemory(count)
        IIf(*buffer = 0, ReturnBreak(0))

        size.l = ReadData(handle, *buffer, count)
        ReturnBreak(MemoryStringFormat(*buffer, size))
      Until #True
      IIf(handle, CloseFile(handle))
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Procedure.i FileOpenRaw(filename.s, mode.l = 0)
      Define skip_bom.l = 0, write_bom = 0, flag.l = 0, format.l = 0
      If mode & #FO_BINARY = 0
        IIf(mode & #FO_UTF8_FULL, count.l = -1, count.l = 64 * 1024)
        IIf(mode & #FO_OVERWRITE = 0, format = FileFormat(filename, count))

        Select format
        Case #ENCODING_UTF16_LE_BOM
          flag = #PB_Unicode
          skip_bom = 2
        Case #ENCODING_UTF16_LE_NOBOM
          flag = #PB_Unicode

        Case #ENCODING_UTF8_BOM
          flag = #PB_UTF8
          skip_bom = 3
        Case #ENCODING_UTF8_NOBOM
          flag = #PB_UTF8

        Default
          If mode & #FO_UTF8
            flag = #PB_UTF8
            write_bom = #PB_UTF8
          ElseIf mode & #FO_UTF8_NOBOM
            flag = #PB_UTF8
          ElseIf mode & #FO_UNICODE
            flag = #PB_Unicode
            write_bom = #PB_Unicode
          ElseIf mode & #FO_UNICODE_NOBOM
            flag = #PB_Unicode
          Else
            flag = #PB_Ascii
          EndIf
        EndSelect
      EndIf

      If (mode & #FO_CREATEPATH) And (mode & #FO_APPEND Or mode & #FO_OVERWRITE)
        DirCreate(GetPathPart(filename))
      EndIf

      If mode & #FO_APPEND
        handle.i = OpenFile(#PB_Any, filename, flag | #PB_File_Append)
        If handle
          IIf(Loc(handle) = 0 And write_bom, WriteStringFormat(handle, write_bom))
        EndIf

      ElseIf mode & #FO_OVERWRITE
        handle.i = CreateFile(#PB_Any, filename, flag)
        IIf(handle And write_bom, WriteStringFormat(handle, write_bom))

      Else
        handle.i = ReadFile(#PB_Any, filename, flag)
        IIf(handle And skip_bom, FileSeek(handle, skip_bom))
      EndIf
      ProcedureReturn handle
    EndProcedure

    Procedure.i FileToHandle(file.s)
      handle.i = Val(Mid(file, 3))
      If Left(file, 2) = "**" And IsFile(handle)
        ProcedureReturn handle
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i FileOpenHandle(file.s, mode.l, *is_open)
      handle.i = FileToHandle(file)
      If handle
        PokeL(*is_open, #False)
      Else
        handle = FileOpenRaw(file, mode)
        PokeL(*is_open, #True)
      EndIf
      ProcedureReturn handle
    EndProcedure

    Procedure.s FileOpen(filename.s, mode.l = 0)
      FunctionInit()
      handle.i = FileOpenRaw(filename, mode)
      IIf(handle, ProcedureReturn "**" + Str(handle), ErrorReturn(1, ""))
    EndProcedure

    Procedure.l FileClose(file.s)
      FunctionInit()
      handle.i = FileToHandle(file)
      If handle = 0 ; FileFindClose?
        search.i = Val(file)
        If IsMemory(search) And MemorySize(search) = SizeOf(FileFindInfo)
          ProcedureReturn FileFindClose(search)
        EndIf
        ProcedureReturn 0
      EndIf

      CloseFile(handle)
      ProcedureReturn 1
    EndProcedure

    Procedure.l FileFlush(file.s)
      FunctionInit()
      handle.i = FileToHandle(file)
      If handle
        ProcedureReturn FlushFileBuffers(handle)
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.s FileRead(file.s, count.q = -1)
      FunctionInit()
      IIf(count = 0, ProcedureReturn "")
      handle.i = FileOpenHandle(file, #FO_READ, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, ""))

      IIf(count < 0, count = Lof(handle) - Loc(handle))
      result.s = ReadString(handle, #PB_File_IgnoreEOL, count)
      length.l = Len(result)
      Extended(length)

      IIf(length = 0 And Eof(handle), Error(-1))
      IIf(is_open, CloseFile(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.s FileReadLine(file.s, line.l = 0)
      FunctionInit()
      handle.i = FileOpenHandle(file, #FO_READ, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, ""))

      If line >= 1
        FileSeek(handle, 0)
        While line > 1
          ReadString(handle)
          line - 1
        Wend
      EndIf
      result.s = ReadString(handle)

      IIf(result = "" And Eof(handle), Error(-1))
      IIf(is_open, CloseFile(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.l FileReadToArray(file.s, Array lines.s(1))
      FunctionInit()
      handle.i = FileOpenHandle(file, #FO_READ, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, 0))

      Define size.l = 10, index.l = 0
      ReDim lines(size)
      While Not Eof(handle)
        lines(index) = ReadString(handle)
        index + 1
        If index >= size
          size * 2
          ReDim lines(size)
        EndIf
      Wend
      ReDim lines(index)
      IIf(is_open, CloseFile(handle))
      IIf(index = 0, Error(2))
      ProcedureReturn index
    EndProcedure

    Procedure.l FileReadToList(file.s, List lines.s())
      FunctionInit()
      ClearList(lines())
      handle.i = FileOpenHandle(file, #FO_READ, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, 0))

      While Not Eof(handle)
        AddElement(lines())
        lines() = ReadString(handle)
      Wend

      IIf(is_open, CloseFile(handle))
      index.l = ListSize(lines())
      IIf(index = 0, Error(2))
      ProcedureReturn index
    EndProcedure

    Procedure.l FileWrite(file.s, text.s, flush.l = #False)
      FunctionInit()
      handle.i = FileOpenHandle(file, #FO_APPEND, @is_open.l)
      IIf(handle = 0, ProcedureReturn 0)

      result.l = WriteString(handle, text)
      IIf(flush, result = FlushFileBuffers(handle))

      IIf(is_open, CloseFile(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.l FileWriteLine(file.s, text.s, flush.l = #False)
      IIf(Right(text, 2) <> #CRLF$, text + #CRLF$)
      ProcedureReturn FileWrite(file, text, flush)
    EndProcedure

    Procedure.i FileReadBinary(file.s, *buffer = 0, count.q = -1)
      FunctionInit()
      Define result.i = 0, handle.i = FileOpenHandle(file, #FO_READ, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, 0))

      IIf(count < 0, count = Lof(handle) - Loc(handle))
      If count = 0
        ; empty file or eof
        Error(-1)
      ElseIf *buffer ; store data in buffer, return bytes (also _Extended())
        result = ReadData(handle, *buffer, count)
        Extended(result)
      Else ; return new memory block, bytes store in _Extended()
        result = AllocateMemory(count)
        IIf(result, Extended(ReadData(handle, result, count)))
      EndIf

      IIf(is_open, CloseFile(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.l FileWriteBinary(file.s, *buffer, count.q = -1, flush.l = #False)
      FunctionInit()
      IIf(count = 0, ProcedureReturn 0)
      Define result.l = 0, handle.i = FileOpenHandle(file, #FO_APPEND, @is_open.l)
      IIf(handle = 0, ProcedureReturn 0)

      IIf(count < 0 And IsMemory(*buffer), count = MemorySize(*buffer))
      IIf(count > 0, result = WriteData(handle, *buffer, count))
      IIf(flush, result = FlushFileBuffers(handle))

      IIf(is_open, CloseFile(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.q FileGetPos(file.s)
      FunctionInit()
      handle.i = FileToHandle(file)
      IIf(handle, ProcedureReturn Loc(handle))
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.l FileSetPos(file.s, offset.q, origin.l = #FILE_BEGIN)
      FunctionInit()
      handle.i = FileToHandle(file)
      If handle
        Select origin
        Case #FILE_END
          FileSeek(handle, offset + Lof(handle), #PB_Absolute)
        Case #FILE_CURRENT
          FileSeek(handle, offset, #PB_Relative)
        Default ; #FILE_BEGIN
          FileSeek(handle, offset, #PB_Absolute)
        EndSelect
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.l FileGetEncoding(file.s, mode = 1)
      FunctionInit()
      handle.i = FileOpenHandle(file, 1, @is_open.l)
      IIf(handle = 0, ErrorReturn(1, -1))

      cur.q = Loc(handle)
      FileSeek(handle, 0)

      Define result.l = 0, *buffer
      Repeat
        IIf(mode = 1, count = Lof(handle), count = 64 * 1024)
        IIf(count = 0, ReturnBreak(0))

        *buffer = AllocateMemory(count)
        IIf(*buffer = 0, ReturnBreak(-1))

        size.l = ReadData(handle, *buffer, count)
        Select MemoryStringFormat(*buffer, size)
        Case #ENCODING_UTF16_LE_BOM, #ENCODING_UTF16_LE_NOBOM
          ReturnBreak(32)
        Case #ENCODING_UTF16_BE_BOM, #ENCODING_UTF16_BE_NOBOM
          ReturnBreak(64)
        Case #ENCODING_UTF8_BOM
          ReturnBreak(128)
        Case #ENCODING_UTF8_NOBOM
          ReturnBreak(256)
        Default
          ReturnBreak(0)
        EndSelect
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      IIf(is_open, CloseFile(handle), FileSeek(handle, cur))
      ProcedureReturn result
    EndProcedure

    Procedure.q FileGetSize(filename.s)
      FunctionInit()
      result.q = FileSize(filename)
      IIf(result = -2, result = 0)
      IIf(result < 0, ErrorReturn(1, 0))
      ProcedureReturn result
    EndProcedure

    Procedure.s FileGetTime(filename.s, option.l = #FT_MODIFIED, format.l = 0)
      FunctionInit()
      Define result.s = "", handle.i
      Repeat
        handle.i = CreateFile_(filename, #GENERIC_READ, 0, 0, #OPEN_EXISTING, 0, 0)
        IIf(handle = #INVALID_HANDLE_VALUE, ReturnErrorBreak("", 1))
        IIf(GetFileTime_(handle, @create.FILETIME, @access.FILETIME, @modify.FILETIME) = 0, ReturnErrorBreak("", 1))

        Select option
          Case #FT_CREATED: *filetime = @create
          Case #FT_ACCESSED: *filetime = @access
          Default: *filetime = @modify
        EndSelect

        IIf(FileTimeToLocalFileTime_(*filetime, @localtime.FILETIME) = 0, ReturnErrorBreak("", 1))
        IIf(FileTimeToSystemTime_(@localtime, @time.SYSTEMTIME) = 0, ReturnErrorBreak("", 1))

        IIf(format, pattern.s = "%04d%02d%02d%02d%02d%02d", pattern.s = "%04d/%02d/%02d %02d:%02d:%02d")
        ReturnBreak(StringFormat(pattern, time\wYear, time\wMonth, time\wDay, time\wHour, time\wMinute, time\wSecond))
      Until #True
      IIf(handle <> #INVALID_HANDLE_VALUE, CloseHandle_(handle))
      ProcedureReturn result
    EndProcedure

    Structure FileSetTimeInfo
      create.i
      access.i
      modify.i
      error_count.l
    EndStructure

    Procedure.l FileSetTimeCallback(path.s, *fd.WIN32_FIND_DATA, *info.FileSetTimeInfo)
      handle.i = CreateFile_(path + PeekS(@*fd\cFileName), #FILE_WRITE_ATTRIBUTES, 0, 0, #OPEN_EXISTING, 0, 0)
      If handle <> #INVALID_HANDLE_VALUE
        IIf(SetFileTime_(handle, *info\create, *info\access, *info\modify) = 0, *info\error_count + 1)
        CloseHandle_(handle)
      Else
        *info\error_count + 1
      EndIf
    EndProcedure

    Procedure.l FileSetTime(source.s, time.s = "", option.l = #FT_MODIFIED, recurse.l = 0)
      FunctionInit()
      time = RemoveString(RemoveString(RemoveString(time, " "), "/"), ":")
      If time = ""
        GetSystemTimeAsFileTime_(@filetime.FILETIME)
      Else
        st.SYSTEMTIME
        st\wYear = Val(Left(time, 4))
        st\wMonth = Val(Mid(time, 5, 2))
        st\wDay = Val(Mid(time, 7, 2))
        st\wHour = Val(Mid(time, 9, 2))
        st\wMinute = Val(Mid(time, 11, 2))
        st\wSecond = Val(Mid(time, 13, 2))
        IIf(SystemTimeToFileTime_(@st, @localtime.FILETIME) = 0, ProcedureReturn 0)
        IIf(LocalFileTimeToFileTime_(@localtime, @filetime.FILETIME) = 0, ProcedureReturn 0)
      EndIf

      info.FileSetTimeInfo
      Select option
        Case #FT_CREATED: info\create = @filetime
        Case #FT_ACCESSED: info\access = @filetime
        Default: info\modify = @filetime
      EndSelect

      DirWalk(source, recurse, @FileSetTimeCallback(), @info)
      IIf(info\error_count <> 0, ProcedureReturn 0, ProcedureReturn 1)
    EndProcedure

    Procedure.i ConvertPathToPIDL(path.s)
      Define pidl.i = 0, lib.i = LoadLibrary_("shell32.dll")
      SHParseDisplayName_.SHParseDisplayName = GetProcAddress_(lib, "SHParseDisplayName")
      IIf(SHParseDisplayName_, SHParseDisplayName_(path, 0, @pidl, 0, 0))
      IIf(lib, FreeLibrary_(lib))
      ProcedureReturn pidl
    EndProcedure

    Procedure.s ConvertPIDLToPath(pidl)
      result.s{#MAX_PATH + 1}
      IIf(pidl, SHGetPathFromIDList_(pidl, @result))
      ProcedureReturn result
    EndProcedure

    Procedure.i FileDialog_Parse(filter.s)
      IIf(filter = "", ProcedureReturn 0)

      *buffer = AllocateMemory(Len(filter) * 2 * SizeOf(Character))
      IIf(*buffer = 0, ProcedureReturn 0)

      *ptr = *buffer
      For i = 1 To CountString(filter, "|") + 1
        part.s = Trim(StringField(filter, i, "|"))
        start.l = StringInStr(part, "(", 0, -1)
        last.l = StringInStr(part, ")", 0, -1)
        IIf(start = 0 Or last = 0, Continue)

        filter_part.s = Trim(Mid(part, start + 1, last - start - 1))
        If filter_part <> ""
          *ptr + PokeS(*ptr, part) + SizeOf(Character)
          *ptr + PokeS(*ptr, filter_part) + SizeOf(Character)
        EndIf
      Next
      ProcedureReturn *buffer
    EndProcedure

    Procedure.s FileDialog(is_save, title.s, init_dir.s, filter.s, option.l, default_name.s, hwnd.i)
      FunctionInit()
      Define result.s = "", *buffer = 0, max.l = 256 * (#MAX_PATH + 1) + 1
      Repeat
        *filter = FileDialog_Parse(filter)
        *buffer = AllocateMemory(max * SizeOf(Character))
        IIf(*buffer = 0, ReturnErrorBreak("", 1))
        PokeS(*buffer, default_name)

        If Left(init_dir, 3) = "::{" And Right(init_dir, 1) = "}"
          pidl = ConvertPathToPIDL(init_dir)
          If pidl
            init_dir = ConvertPIDLToPath(pidl)
            CoTaskMemFree_(pidl)
          EndIf
        EndIf

        lp.OPENFILENAME\lStructSize = SizeOf(OPENFILENAME)
        lp\hwndOwner = hwnd
        lp\lpstrFilter = *filter
        lp\lpstrTitle = @title
        lp\lpstrInitialDir = @init_dir
        lp\lpstrFile = *buffer
        lp\nMaxFile = max
        lp\Flags = #OFN_EXPLORER | #OFN_HIDEREADONLY
        IIf(option & #FD_FILEMUSTEXIST, lp\Flags | #OFN_FILEMUSTEXIST)
        IIf(option & #FD_PATHMUSTEXIST, lp\Flags | #OFN_PATHMUSTEXIST)
        IIf(option & #FD_MULTISELECT, lp\Flags | #OFN_ALLOWMULTISELECT)
        IIf(option & #FD_PROMPTCREATENEW, lp\Flags | #OFN_CREATEPROMPT)
        IIf(option & #FD_PROMPTOVERWRITE, lp\Flags | #OFN_OVERWRITEPROMPT)

        If is_save
          IIf(GetSaveFileName_(@lp) = 0, ReturnErrorBreak("", 1))
        Else
          IIf(GetOpenFileName_(@lp) = 0, ReturnErrorBreak("", 1))
        EndIf

        *ptr = *buffer
        While PeekC(*ptr) <> 0
          entry.s = PeekS(*ptr)
          *ptr + (Len(entry) + 1) * SizeOf(Character)
          result + entry + "|"
        Wend
        IIf(Right(result, 1) = "|", result = StringTrimRight(result, 1))

      Until #True
      IIf(*filter, FreeMemory(*filter))
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Procedure.s FileOpenDialog(title.s = "", init_dir.s = "", filter.s = "", option.l = 0, default_name.s = "", hwnd.i = 0)
      ProcedureReturn FileDialog(#False, title, init_dir, filter, option, default_name, hwnd)
    EndProcedure

    Procedure.s FileSaveDialog(title.s = "", init_dir.s = "", filter.s = "", option.l = 0, default_name.s = "", hwnd.i = 0)
      ProcedureReturn FileDialog(#True, title, init_dir, filter, option, default_name, hwnd)
    EndProcedure

    Procedure.l FileSelectFolder_Callback(hwnd.i, msg.l, lparam.i, lpdata.i)
      IIf(lpdata And msg = #BFFM_INITIALIZED, SendMessage_(hwnd, #BFFM_SETSELECTION, 0, lpdata))
      ProcedureReturn 0
    EndProcedure

    Procedure.s FileSelectFolder(text.s = "", root_dir.s = "", flag.l = 0, init_dir.s = "", hwnd.i = 0)
      FunctionInit()
      Define root_pidl.i = 0, init_pidl.i = 0
      IIf(root_dir <> "", root_pidl = ConvertPathToPIDL(root_dir))
      IIf(init_dir <> "", init_pidl = ConvertPathToPIDL(init_dir))

      bi.BROWSEINFO
      bi\hwndOwner = hwnd
      bi\lpszTitle = @text
      bi\pidlRoot = root_pidl
      bi\lParam  = init_pidl
      IIf(init_pidl, bi\lpfn = @FileSelectFolder_Callback())

      ; 1 = Show Create Folder Button (requires IE6.0 or later)
      ; 2 = Use New Dialog Style (requires IE5.0 or later)
      ; 4 = Show Edit Control (to type the path)

      bi\ulFlags = #BIF_RETURNONLYFSDIRS
      IIf(flag & 4, bi\ulFlags | #BIF_EDITBOX)
      IIf(flag & 2, bi\ulFlags | #BIF_NEWDIALOGSTYLE | #BIF_NONEWFOLDERBUTTON)
      IIf(flag & 1, bi\ulFlags | #BIF_NEWDIALOGSTYLE & ~#BIF_NONEWFOLDERBUTTON)

      Define result.s = "", pidl.i = SHBrowseForFolder_(@bi)
      IIf(pidl, result = ConvertPIDLToPath(pidl), Error(1))

      IIf(root_pidl, CoTaskMemFree_(root_pidl))
      IIf(init_pidl, CoTaskMemFree_(init_pidl))
      IIf(pidl, CoTaskMemFree_(pidl))
      ProcedureReturn result
    EndProcedure

    Procedure.s IniRead(filename.s, section.s, key.s, value.s = "")
      FunctionInit()
      filename = FileGetFullName(filename)
      buffer.s{65535}
      GetPrivateProfileString_(section, key, value, @buffer, LengthOf(buffer), filename)
      ProcedureReturn buffer
    EndProcedure

    Procedure.l IniWrite(filename.s, section.s, key.s, value.s)
      FunctionInit()
      filename = FileGetFullName(filename)
      ProcedureReturn WritePrivateProfileString_(section, key, value, filename)
    EndProcedure

    Procedure.l IniDelete(filename.s, section.s, key.s = "")
      FunctionInit()
      filename = FileGetFullName(filename)
      IIf(key = "", *key = 0, *key = @key)
      ProcedureReturn WritePrivateProfileString_(section, *key, 0, filename)
    EndProcedure

    Procedure.s IniReadSectionNames(filename.s, separator.s = ",")
      FunctionInit()
      filename = FileGetFullName(filename)
      buffer.s{65535}
      IIf(GetPrivateProfileSectionNames_(@buffer, LengthOf(buffer), filename) = 0, ErrorReturn(1, ""))

      Define result.s = "", *ptr = @buffer
      While PeekC(*ptr) <> 0
        entry.s = PeekS(*ptr)
        *ptr + (Len(entry) + 1) * SizeOf(Character)
        result + entry + separator
      Wend
      IIf(Right(result, Len(separator)) = separator, result = StringTrimRight(result, Len(separator)))
      ProcedureReturn result
    EndProcedure

    Procedure.s IniReadSectionKeys(filename.s, section.s, separator.s = ",")
      FunctionInit()
      filename = FileGetFullName(filename)
      buffer.s{65535}
      IIf(GetPrivateProfileSection_(section, @buffer, LengthOf(buffer), filename) = 0, ErrorReturn(1, ""))

      Define result.s = "", *ptr = @buffer
      While PeekC(*ptr) <> 0
        entry.s = PeekS(*ptr)
        *ptr + (Len(entry) + 1) * SizeOf(Character)
        result + StringField(entry, 1, "=") + separator
      Wend
      IIf(Right(result, Len(separator)) = separator, result = StringTrimRight(result, Len(separator)))
      ProcedureReturn result
    EndProcedure

    Procedure.l IniReadSection(Map dict.s(), filename.s, section.s)
      FunctionInit()
      ClearMap(dict())
      filename = FileGetFullName(filename)
      buffer.s{65535}
      IIf(GetPrivateProfileSection_(section, @buffer, LengthOf(buffer), filename) = 0, ErrorReturn(1, 0))

      Define *ptr = @buffer
      While PeekC(*ptr) <> 0
        entry.s = PeekS(*ptr)
        *ptr + (Len(entry) + 1) * SizeOf(Character)

        key.s = StringField(entry, 1, "=")
        dict(key) = Mid(entry, Len(key) + 2)
      Wend
      ProcedureReturn MapSize(dict())
    EndProcedure

    Procedure.l IniWriteSection(filename.s, section.s, string.s)
      FunctionInit()
      filename = FileGetFullName(filename)
      Define buffer.s{65535}, *ptr = @buffer

      For i = 1 To CountString(string, #LF$) + 1
        part.s = Trim(StringField(string, i, #LF$))
        *ptr + PokeS(*ptr, part) + SizeOf(Character)
      Next
      result.l = WritePrivateProfileSection_(section, @buffer, filename)
      IIf(result = 0, Error(1))
      ProcedureReturn result
    EndProcedure

    Procedure.l IniWriteSectionMap(Map dict.s(), filename.s, section.s, keys.s = "", separator.s = ",")
      FunctionInit()
      filename = FileGetFullName(filename)
      Define buffer.s{65535}, *ptr = @buffer
      If keys <> ""
        For i = 1 To CountString(keys, separator) + 1
          key.s = StringField(keys, i, separator)
          part.s = key + "=" + dict(key)
          *ptr + PokeS(*ptr, part) + SizeOf(Character)
        Next
      Else
        ForEach dict()
          part.s = MapKey(dict()) + "=" + dict()
          *ptr + PokeS(*ptr, part) + SizeOf(Character)
        Next
      EndIf
      result.l = WritePrivateProfileSection_(section, @buffer, filename)
      IIf(result = 0, Error(1))
      ProcedureReturn result
    EndProcedure

    Procedure.l IniRenameSection(filename.s, section.s, new_section.s, overwrite.l = 0)
      FunctionInit()
      separator.s = Chr(1) + Chr($FF) + Chr(1) + Chr($FF) + Chr(1)
      NewMap dict.s()

      keys.s = IniReadSectionKeys(filename, section, separator)
      IIf(keys = "", ErrorReturn(1, 0))

      If IniReadSectionKeys(filename, new_section) <> ""
        IIf(overwrite, IniDelete(filename, new_section), ErrorReturn(1, 0))
      EndIf

      IniReadSection(dict(), filename, section)
      IniDelete(filename, section)
      ProcedureReturn IniWriteSectionMap(dict(), filename, new_section, keys, separator)
    EndProcedure

  ; ===========================================================================
  ;  Graphic and Sound functions
  ; ===========================================================================

    Procedure.l Beep(frequency = 500, duration = 1000)
      FunctionInit()
      ProcedureReturn Beep_(frequency, duration)
    EndProcedure

    Procedure.l PixelGetColor(x.l, y.l, hwnd.i = 0, alt.l = 0)
      FunctionInit()
      IIf(alt, hdc.i = CreateDC_("DISPLAY", 0, 0, 0), hdc.i = GetDC_(0))
      IIf(hdc = 0, ProcedureReturn -1)

      CoordAdjust(Opt("PixelCoordMode"), x, y, @x, @y, 1, hwnd)

      color.l = GetPixel_(hdc, x, y)
      IIf(alt, DeleteDC_(hdc), ReleaseDC_(0, hdc))

      EndianSwitch(color)
      ProcedureReturn color >> 8 & $FFFFFF
    EndProcedure

    Structure PixelAreaInfo
      ptr.i
      size.l
      hdc.i
      sdc.i
      bmp.i
      orig.i
      width.l
      height.l
      left.l
      top.l
      x_reverse.l
      y_reverse.l
    EndStructure

    Procedure.i PixelGetArea(*pai.PixelAreaInfo, left.l, top.l, right.l, bottom.l, hwnd.i)
      Do(*pai\x_reverse = #False, *pai\y_reverse = #False)
      If left > right: Swap left, right: *pai\x_reverse.l = #True: EndIf
      If top > bottom: Swap top, bottom: *pai\y_reverse.l = #True: EndIf
      CoordAdjust(Opt("PixelCoordMode"), left, top, @left, @top, 1, hwnd)
      CoordAdjust(Opt("PixelCoordMode"), right + 1, bottom + 1, @right, @bottom, 1, hwnd)

      *pai\left = left
      *pai\top = top
      *pai\width = right - left
      *pai\height = bottom - top
      *pai\size = *pai\width * *pai\height * 4
      IIf(*pai\width < 0 Or *pai\height < 0, ProcedureReturn 0)

      Repeat
        *pai\hdc = GetDC_(0)
        IIf(*pai\hdc = 0, ProcedureReturn 0)

        *pai\sdc = CreateCompatibleDC_(*pai\hdc)
        IIf(*pai\sdc = 0, ProcedureReturn 0)

        bitmapinfo.BITMAPINFOHEADER
        bitmapinfo\biSize = SizeOf(BITMAPINFOHEADER)
        bitmapinfo\biWidth = *pai\width
        bitmapinfo\biHeight = *pai\height
        bitmapinfo\biPlanes = 1
        bitmapinfo\biBitCount = 32
        *pai\bmp = CreateDIBSection_(0, @bitmapinfo, 0, @*pai\ptr, 0, 0)
        IIf(*pai\bmp = 0, ProcedureReturn 0)

        *pai\orig = SelectObject_(*pai\sdc, *pai\bmp)
        BitBlt_(*pai\sdc, 0, 0, *pai\width, *pai\height, *pai\hdc, left, top, #SRCCOPY)
      Until #True
      ProcedureReturn 1
    EndProcedure

    Procedure.i PixelFreeArea(*pai.PixelAreaInfo)
      IIf(*pai\sdc <> 0, SelectObject_(*pai\sdc, *pai\orig) : DeleteDC_(*pai\sdc))
      IIf(*pai\bmp <> 0, DeleteObject_(*pai\bmp))
      IIf(*pai\hdc <> 0, ReleaseDC_(0, *pai\hdc))
    EndProcedure

    Procedure.l PixelChecksum(left.l, top.l, right.l, bottom.l, hwnd.i = 0)
      FunctionInit()
      result.l = 0
      If PixelGetArea(@pai.PixelAreaInfo, left, top, right, bottom, hwnd)
        result = XXH_fast32(pai\ptr, pai\size)
      EndIf
      PixelFreeArea(@pai)
      ProcedureReturn result
    EndProcedure

    Procedure.s PixelSearch(left.l, top.l, right.l, bottom.l, color.l, variation.l = 0, hwnd.i = 0)
      FunctionInit()
      Define result_x.l = -1, result_y.l = -1
      IIf(variation > 255 Or variation < 0, ErrorReturn(1, ""))

      If PixelGetArea(@pai.PixelAreaInfo, left, top, right, bottom, hwnd)
        color & $FFFFFF
        Define r.l = color >> 16, g.l = color >> 8 & $FF, b.l = color & $FF

        If pai\x_reverse
          Define x_start.l = pai\width - 1, x_end.l = -1, x_step.l = -1
        Else
          Define x_start.l = 0, x_end.l = pai\width, x_step.l = 1
        EndIf

        If pai\y_reverse
          Define y_start.l = 0, y_end.l = pai\height, y_step.l = 1
        Else
          Define y_start.l = pai\height - 1, y_end.l = -1, y_step.l = -1
        EndIf

        found.l = #False
        j = y_start
        While j <> y_end
          i = x_start
          While i <> x_end
            pixel.l = PeekL(pai\ptr + (j * pai\width + i) * 4) & $FFFFFF
            If variation = 0
              IIf(pixel = color, found = #True)
            Else
              Define diff_r.l = (pixel >> 16) - r, diff_g.l = (pixel >> 8 & $FF) - g, diff_b.l = (pixel & $FF) - b
              Do(IntegerABS(diff_r), IntegerABS(diff_g), IntegerABS(diff_b))
              IIf(diff_r <= variation And diff_g <= variation And diff_b <= variation, found = #True)
            EndIf

            If found
              result_x = pai\left + i
              result_y = pai\top + (pai\height - j - 1)
              Break 2
            EndIf

            i + x_step
          Wend
          j + y_step
        Wend

      EndIf
      PixelFreeArea(@pai)
      CoordAdjust(Opt("PixelCoordMode"), result_x, result_y, @result_x, @result_y, -1, hwnd)
      IIf(result_x < 0 Or result_y < 0, ErrorReturn(1, ""), ProcedureReturn Str(result_x) + "," + Str(result_y))
    EndProcedure

    Procedure.l SoundPlay(file.s = "", wait.l = 0)
      FunctionInit()
      If file <> ""
        mciSendString_(~"OPEN \"" + file + ~"\" Type MPEGVIDEO ALIAS PUREAUTOIT_SOUND", 0, 0, 0)
        mciSendString_("play PUREAUTOIT_SOUND from 0", 0, 0, 0)
        If wait
          buffer.s{#MAX_PATH + 1}
          Repeat
            Delay(10)
            mciSendString_("status PUREAUTOIT_SOUND mode", @buffer, #MAX_PATH, 0)
          Until buffer = "stopped"
          file = "" ; to close it
        EndIf
      EndIf

      If file = ""
        mciSendString_("stop PUREAUTOIT_SOUND", 0, 0, 0)
        mciSendString_("close PUREAUTOIT_SOUND", 0, 0, 0)
      EndIf
      ProcedureReturn 1 ; Always returns 1 regardless of success
    EndProcedure

    Procedure.l SoundSetWaveVolume(percent.f)
      FunctionInit()
      IIf(percent > 100 Or percent < 0, ErrorReturn(1, 0))
      volume.l = percent * $FFFF  / 100
      IIf(waveOutSetVolume_(0, volume) = #MMSYSERR_NOERROR, ProcedureReturn 1, ErrorReturn(1, 0))
    EndProcedure

  ; ===========================================================================
  ;  Keyboard functions
  ; ===========================================================================

    EnumerationBinary
      #MOD_LALT      = #MOD_ALT
      #MOD_LCONTROL  = #MOD_CONTROL
      #MOD_LSHIFT    = #MOD_SHIFT
      #MOD_LWIN      = #MOD_WIN
      #MOD_RALT
      #MOD_RCONTROL
      #MOD_RSHIFT
      #MOD_RWIN
    EndEnumeration

    Enumeration
      #TOGGLE           = 0
      #TOGGLE_ON        = 1
      #TOGGLE_OFF       = 2
      #TOGGLE_DOWN      = 3
      #TOGGLE_UP        = 4
      #TOGGLE_SHIFT     = 5
      #TOGGLE_EXTEND    = 6
      #TOGGLE_ASC       = 7
      #TOGGLE_UNICODE   = 8
      #VK_TOGGLE_ON     = $100
      #VK_TOGGLE_OFF    = $200
      #VK_TOGGLE_DOWN   = $300
      #VK_TOGGLE_UP     = $400
      #VK_TOGGLE_SHIFT  = $500
      #VK_TOGGLE_EXTEND = $600

      #KEYEVENTF_UNICODE = 4
    EndEnumeration

    Procedure keybd_event_send(vk.a, flag.l, extra.i = 0)
      input.INPUT
      input\type = #INPUT_KEYBOARD
      input\ki\wVk = vk
      input\ki\wScan = MapVirtualKey_(vk, #MAPVK_VK_TO_VSC)
      input\ki\dwFlags = flag
      input\ki\dwExtraInfo = extra
      SendInput_(1, @input, SizeOf(INPUT))
    EndProcedure

    Procedure keybd_event_send_unicode(unicode.u, flag.l, extra.i = 0)
      input.INPUT
      input\type = #INPUT_KEYBOARD
      input\ki\wVk = 0
      input\ki\wScan = unicode
      input\ki\dwFlags = flag | #KEYEVENTF_UNICODE
      input\ki\dwExtraInfo = extra
      SendInput_(1, @input, SizeOf(INPUT))
    EndProcedure

    ; Only for VK_CAPITAL, VK_SCROLL, and VK_NUMLOCK, all don't need EXTENDEDKEY
    Procedure.l ToggleKeyState(vk.a, toggle.l = #TOGGLE)
      IIf(GetKeyState_(vk) & 1, old_state = #TOGGLE_ON, old_state = #TOGGLE_OFF)

      Select toggle
      Case #TOGGLE_ON
        IIf(old_state = #TOGGLE_ON, ProcedureReturn #TOGGLE_ON)
      Case #TOGGLE_OFF
        IIf(old_state = #TOGGLE_OFF, ProcedureReturn #TOGGLE_OFF)
      EndSelect

      keybd_event_send(vk, 0)
      Delay(Opt("SendKeyDownDelay"))
      keybd_event_send(vk, #KEYEVENTF_KEYUP)
      Delay(Opt("SendKeyDelay"))

      ProcedureReturn old_state
    EndProcedure

    ; Only left shift, left ctrl, and left alt don't need EXTENDEDKEY
    Procedure SendModifier(modifier.u, type.l = 0)
      Macro SendModifierNoExt(vk)
        keybd_event_send(vk, type)
        Delay(Opt("SendKeyDelay"))
      EndMacro

      Macro SendModifierExt(vk)
        keybd_event_send(vk, type | #KEYEVENTF_EXTENDEDKEY)
        Delay(Opt("SendKeyDelay"))
      EndMacro

      IIf(modifier & #MOD_LSHIFT, SendModifierNoExt(#VK_LSHIFT))
      IIf(modifier & #MOD_RSHIFT, SendModifierExt(#VK_RSHIFT))
      IIf(modifier & #MOD_LCONTROL, SendModifierNoExt(#VK_LCONTROL))
      IIf(modifier & #MOD_RCONTROL, SendModifierExt(#VK_RCONTROL))
      IIf(modifier & #MOD_LALT, SendModifierNoExt(#VK_LMENU))
      IIf(modifier & #MOD_RALT, SendModifierExt(#VK_RMENU))
      IIf(modifier & #MOD_LWIN, SendModifierExt(#VK_LWIN))
      IIf(modifier & #MOD_RWIN, SendModifierExt(#VK_RWIN))
    EndProcedure

    Procedure SendKey(vk.a, modifier.u, count.l = 1, extend = 0, target_hwnd.i = 0)
      IIf(count <= 0, count = 1)

      SendModifier(modifier, 0)
      For i = 1 To count
        keybd_event_send(vk, extend)
        Delay(Opt("SendKeyDownDelay"))
        keybd_event_send(vk, #KEYEVENTF_KEYUP | extend)
        Delay(Opt("SendKeyDelay"))
      Next
      SendModifier(modifier, #KEYEVENTF_KEYUP)
    EndProcedure

    Procedure SendUnicode(unicode.u, count.l = 1, target_hwnd.i = 0, dbcs.l = #False)
      IIf(count <= 0, count = 1)

      For i = 1 To count
        If target_hwnd
          If dbcs
            Dim dbcs.a(2)
            PokeS(@dbcs(), Chr(unicode), -1, #PB_Ascii)
            PostMessage_(target_hwnd, #WM_CHAR, dbcs(0), 0)
            IIf(dbcs(1), PostMessage_(target_hwnd, #WM_CHAR, dbcs(1), 0))
            Delay(Opt("SendKeyDelay"))
          Else
            PostMessage_(target_hwnd, #WM_CHAR, unicode, 0)
            Delay(Opt("SendKeyDelay"))
          EndIf
        Else
          keybd_event_send_unicode(unicode, 0)
          Delay(Opt("SendKeyDownDelay"))
          keybd_event_send_unicode(unicode, #KEYEVENTF_KEYUP)
          Delay(Opt("SendKeyDelay"))
        EndIf
      Next
    EndProcedure

    Procedure SendAsc(number.q)
      code.s = StrU(number)

      keybd_event_send(#VK_MENU, 0)
      For i = 1 To Len(code)
        vk.b = Val(Mid(code, i, 1)) + #VK_NUMPAD0
        keybd_event_send(vk, 0)
        keybd_event_send(vk, #KEYEVENTF_KEYUP)
      Next
      keybd_event_send(#VK_MENU, #KEYEVENTF_KEYUP)
    EndProcedure

    Macro InitKeyTable()
      If Not key_table("")
        For i = 'A' To 'Z'
          key_table(Chr(i)) = i
        Next
        For i = '0' To '9'
          key_table(Chr(i)) = i
        Next

        key_table(";") = #VK_OEM_1
        key_table(":") = #VK_OEM_1 | #VK_TOGGLE_SHIFT
        key_table("/") = #VK_OEM_2
        key_table("?") = #VK_OEM_2 | #VK_TOGGLE_SHIFT
        key_table("`") = #VK_OEM_3
        key_table("~") = #VK_OEM_3 | #VK_TOGGLE_SHIFT
        key_table("[") = #VK_OEM_4
        key_table("{") = #VK_OEM_4 | #VK_TOGGLE_SHIFT
        key_table("\") = #VK_OEM_5
        key_table("|") = #VK_OEM_5 | #VK_TOGGLE_SHIFT
        key_table("]") = #VK_OEM_6
        key_table("}") = #VK_OEM_6 | #VK_TOGGLE_SHIFT
        key_table("'") = #VK_OEM_7
        key_table(~"\"") = #VK_OEM_7 | #VK_TOGGLE_SHIFT
        key_table("=") = #VK_OEM_PLUS
        key_table("+") = #VK_OEM_PLUS | #VK_TOGGLE_SHIFT
        key_table("-") = #VK_OEM_MINUS
        key_table("_") = #VK_OEM_MINUS | #VK_TOGGLE_SHIFT
        key_table(",") = #VK_OEM_COMMA
        key_table("<") = #VK_OEM_COMMA | #VK_TOGGLE_SHIFT
        key_table(".") = #VK_OEM_PERIOD
        key_table(">") = #VK_OEM_PERIOD | #VK_TOGGLE_SHIFT
        key_table(")") = '0' | #VK_TOGGLE_SHIFT
        key_table("!") = '1' | #VK_TOGGLE_SHIFT
        key_table("@") = '2' | #VK_TOGGLE_SHIFT
        key_table("#") = '3' | #VK_TOGGLE_SHIFT
        key_table("$") = '4' | #VK_TOGGLE_SHIFT
        key_table("%") = '5' | #VK_TOGGLE_SHIFT
        key_table("^") = '6' | #VK_TOGGLE_SHIFT
        key_table("&") = '7' | #VK_TOGGLE_SHIFT
        key_table("*") = '8' | #VK_TOGGLE_SHIFT
        key_table("(") = '9' | #VK_TOGGLE_SHIFT
        key_table("SPACE")         = #VK_SPACE
        key_table("ENTER")         = #VK_RETURN
        key_table("ALT")           = #VK_MENU
        key_table("BACKSPACE")     = #VK_BACK
        key_table("BS")            = #VK_BACK
        key_table("DELETE")        = #VK_DELETE     | #VK_TOGGLE_EXTEND
        key_table("DEL")           = #VK_DELETE     | #VK_TOGGLE_EXTEND
        key_table("UP")            = #VK_UP         | #VK_TOGGLE_EXTEND
        key_table("DOWN")          = #VK_DOWN       | #VK_TOGGLE_EXTEND
        key_table("LEFT")          = #VK_LEFT       | #VK_TOGGLE_EXTEND
        key_table("RIGHT")         = #VK_RIGHT      | #VK_TOGGLE_EXTEND
        key_table("HOME")          = #VK_HOME       | #VK_TOGGLE_EXTEND
        key_table("END")           = #VK_END        | #VK_TOGGLE_EXTEND
        key_table("ESCAPE")        = #VK_ESCAPE
        key_table("ESC")           = #VK_ESCAPE
        key_table("INSERT")        = #VK_INSERT     | #VK_TOGGLE_EXTEND
        key_table("INS")           = #VK_INSERT     | #VK_TOGGLE_EXTEND
        key_table("PGUP")          = #VK_PRIOR      | #VK_TOGGLE_EXTEND
        key_table("PAGEUP")        = #VK_PRIOR      | #VK_TOGGLE_EXTEND
        key_table("PGDN")          = #VK_NEXT       | #VK_TOGGLE_EXTEND
        key_table("PAGEDOWN")      = #VK_NEXT       | #VK_TOGGLE_EXTEND
        key_table("F1")            = #VK_F1
        key_table("F2")            = #VK_F2
        key_table("F3")            = #VK_F3
        key_table("F4")            = #VK_F4
        key_table("F5")            = #VK_F5
        key_table("F6")            = #VK_F6
        key_table("F7")            = #VK_F7
        key_table("F8")            = #VK_F8
        key_table("F9")            = #VK_F9
        key_table("F10")           = #VK_F10
        key_table("F11")           = #VK_F11
        key_table("F12")           = #VK_F12
        key_table("TAB")           = #VK_TAB
        key_table("PRINTSCREEN")   = #VK_SNAPSHOT
        key_table("LWIN")          = #VK_LMENU      | #VK_TOGGLE_EXTEND
        key_table("RWIN")          = #VK_RMENU      | #VK_TOGGLE_EXTEND
        key_table("NUMLOCK")       = #VK_NUMLOCK
        key_table("CAPSLOCK")      = #VK_CAPITAL
        key_table("SCROLLLOCK")    = #VK_SCROLL
        key_table("BREAK")         = #VK_CANCEL
        key_table("PAUSE")         = #VK_PAUSE
        key_table("NUMPAD0")       = #VK_NUMPAD0
        key_table("NUMPAD1")       = #VK_NUMPAD1
        key_table("NUMPAD2")       = #VK_NUMPAD2
        key_table("NUMPAD3")       = #VK_NUMPAD3
        key_table("NUMPAD4")       = #VK_NUMPAD4
        key_table("NUMPAD5")       = #VK_NUMPAD5
        key_table("NUMPAD6")       = #VK_NUMPAD6
        key_table("NUMPAD7")       = #VK_NUMPAD7
        key_table("NUMPAD8")       = #VK_NUMPAD8
        key_table("NUMPAD9")       = #VK_NUMPAD9
        key_table("NUMPADMULT")    = #VK_MULTIPLY
        key_table("NUMPADADD")     = #VK_ADD
        key_table("NUMPADSUB")     = #VK_SUBTRACT
        key_table("NUMPADDIV")     = #VK_DIVIDE     | #VK_TOGGLE_EXTEND
        key_table("NUMPADDOT")     = #VK_DECIMAL
        key_table("NUMPADENTER")   = #VK_RETURN     | #VK_TOGGLE_EXTEND
        key_table("APPSKEY")       = #VK_APPS       | #VK_TOGGLE_EXTEND
        key_table("LALT")          = #VK_LMENU
        key_table("RALT")          = #VK_RMENU      | #VK_TOGGLE_EXTEND
        key_table("LCTRL")         = #VK_LCONTROL
        key_table("RCTRL")         = #VK_RCONTROL   | #VK_TOGGLE_EXTEND
        key_table("LSHIFT")        = #VK_LSHIFT
        key_table("RSHIFT")        = #VK_RSHIFT     | #VK_TOGGLE_EXTEND
        key_table("SLEEP")         = #VK_SLEEP      | #VK_TOGGLE_EXTEND

        key_table("BROWSER_BACK")      = #VK_BROWSER_BACK        | #VK_TOGGLE_EXTEND
        key_table("BROWSER_FORWARD")   = #VK_BROWSER_FORWARD     | #VK_TOGGLE_EXTEND
        key_table("BROWSER_REFRESH")   = #VK_BROWSER_REFRESH     | #VK_TOGGLE_EXTEND
        key_table("BROWSER_STOP")      = #VK_BROWSER_STOP        | #VK_TOGGLE_EXTEND
        key_table("BROWSER_SEARCH")    = #VK_BROWSER_SEARCH      | #VK_TOGGLE_EXTEND
        key_table("BROWSER_FAVORITES") = #VK_BROWSER_FAVORITES   | #VK_TOGGLE_EXTEND
        key_table("BROWSER_HOME")      = #VK_BROWSER_HOME        | #VK_TOGGLE_EXTEND
        key_table("VOLUME_MUTE")       = #VK_VOLUME_MUTE         | #VK_TOGGLE_EXTEND
        key_table("VOLUME_DOWN")       = #VK_VOLUME_DOWN         | #VK_TOGGLE_EXTEND
        key_table("VOLUME_UP")         = #VK_VOLUME_UP           | #VK_TOGGLE_EXTEND
        key_table("MEDIA_NEXT")        = #VK_MEDIA_NEXT_TRACK    | #VK_TOGGLE_EXTEND
        key_table("MEDIA_PREV")        = #VK_MEDIA_PREV_TRACK    | #VK_TOGGLE_EXTEND
        key_table("MEDIA_STOP")        = #VK_MEDIA_STOP          | #VK_TOGGLE_EXTEND
        key_table("MEDIA_PLAY_PAUSE")  = #VK_MEDIA_PLAY_PAUSE    | #VK_TOGGLE_EXTEND
        key_table("LAUNCH_MAIL")       = #VK_LAUNCH_MAIL         | #VK_TOGGLE_EXTEND
        key_table("LAUNCH_MEDIA")      = #VK_LAUNCH_MEDIA_SELECT | #VK_TOGGLE_EXTEND
        key_table("LAUNCH_APP1")       = #VK_LAUNCH_APP1         | #VK_TOGGLE_EXTEND
        key_table("LAUNCH_APP2")       = #VK_LAUNCH_APP2         | #VK_TOGGLE_EXTEND

        key_table("ALTDOWN")   = #MOD_LALT     | #VK_TOGGLE_DOWN
        key_table("SHIFTDOWN") = #MOD_LSHIFT   | #VK_TOGGLE_DOWN
        key_table("CTRLDOWN")  = #MOD_LCONTROL | #VK_TOGGLE_DOWN
        key_table("LWINDOWN")  = #MOD_LWIN     | #VK_TOGGLE_DOWN
        key_table("RWINDOWN")  = #MOD_RWIN     | #VK_TOGGLE_DOWN
        key_table("ALTUP")     = #MOD_LALT     | #VK_TOGGLE_UP
        key_table("SHIFTUP")   = #MOD_LSHIFT   | #VK_TOGGLE_UP
        key_table("CTRLUP")    = #MOD_LCONTROL | #VK_TOGGLE_UP
        key_table("LWINUP")    = #MOD_LWIN     | #VK_TOGGLE_UP
        key_table("RWINUP")    = #MOD_RWIN     | #VK_TOGGLE_UP

        key_table("NUMLOCKON")        = #VK_NUMLOCK | #VK_TOGGLE_ON
        key_table("NUMLOCKOFF")       = #VK_NUMLOCK | #VK_TOGGLE_OFF
        key_table("CAPSLOCKON")       = #VK_CAPITAL | #VK_TOGGLE_ON
        key_table("CAPSLOCKOFF")      = #VK_CAPITAL | #VK_TOGGLE_OFF
        key_table("SCROLLLOCKON")     = #VK_SCROLL  | #VK_TOGGLE_ON
        key_table("SCROLLLOCKOFF")    = #VK_SCROLL  | #VK_TOGGLE_OFF
        key_table("") = 1
      EndIf
    EndMacro

    Procedure.l SendKeyAdd(command.s, modifier.u, List key_list.KeyInfo())
      Static NewMap key_table.u()
      InitKeyTable()

      Define vk.a = 0, toggle.a = 0, number.q = 1, extend.a = 0

      If command <> " "
        command = StringStripWS(command, #STR_STRIPSPACES | #STR_STRIPLEADING | #STR_STRIPTRAILING)
        key.s = StringField(command, 1, " ")
      Else
        key.s = " "
      EndIf
      str_num.s = UCase(StringField(command, 2, " "))

      IIf(key = "", ProcedureReturn)

      Select str_num
      Case "ON", "OFF", "UP", "DOWN"
        key + str_num
      Default
        number = Val(str_num)
      EndSelect

      If FindMapElement(key_table(), UCase(key))
        vk = key_table() & $FF
        toggle = key_table() >> 8

        If toggle = #TOGGLE_SHIFT Or (Len(key) = 1 And StringIsUpper(key)) ; deal with A-Z
          modifier | #MOD_LSHIFT
          toggle = 0
        EndIf

        IIf(toggle = #TOGGLE_EXTEND, extend = #KEYEVENTF_EXTENDEDKEY)

        AddElement(key_list())
        key_list()\modifier = modifier
        key_list()\count = number
        key_list()\vk = vk
        key_list()\extend = extend
        key_list()\toggle = toggle

      ElseIf UCase(key) = "ASC"
        AddElement(key_list())
        key_list()\count = number
        key_list()\toggle = #TOGGLE_ASC

      ElseIf UCase(key) = "UNICODE" Or UCase(key) = "UNI"
        AddElement(key_list())
        key_list()\count = 1
        key_list()\vk = number
        key_list()\toggle = #TOGGLE_UNICODE

      Else ; send unicode
        AddElement(key_list())
        key_list()\count = number
        key_list()\vk = Asc(key)
        key_list()\toggle = #TOGGLE_UNICODE
      EndIf
    EndProcedure

    Procedure SendKeyParse(keys.s, List key_list.KeyInfo())
      modifier.u = 0

      For i = 1 To Len(keys)
        char.s = Mid(keys, i, 1)
        Select char
        Case "{"
          If Mid(keys, i, 3) = "{}}"
            SendKeyAdd("}", modifier, key_list())
            modifier = 0
            i + 2
          Else
            command.s = Mid(keys, i + 1, FindString(keys, "}", i) - i - 1)
            SendKeyAdd(command, modifier, key_list())
            modifier = 0
            i + Len(command) + 1
          EndIf

        Case "+"
          modifier | #MOD_LSHIFT
        Case "!"
          modifier | #MOD_LALT
        Case "^"
          modifier | #MOD_LCONTROL
        Case "#"
          modifier | #MOD_LWIN

        Default
          SendKeyAdd(char, modifier, key_list())
          modifier = 0
        EndSelect
      Next
    EndProcedure

    Procedure.i SendKeepActive(title.s = "", text.s = "")
      FunctionInit()
      If title = "" And text = ""
        __keep_active_hwnd = 0
      Else
        __keep_active_hwnd = WinGetHandle(title, text)
      EndIf
      ProcedureReturn __keep_active_hwnd
    EndProcedure

    Procedure Send(keys.s, flag.l = 0)
      FunctionInit()
      If flag
        For i = 1 To Len(keys)
          If __keep_active_hwnd
            IIf(IsIconic_(__keep_active_hwnd), ShowWindow_(__keep_active_hwnd, #SW_RESTORE))
            SetForegroundWindow_(__keep_active_hwnd)
            IIf(Opt("SendAttachMode"), AttachThreadInput(__keep_active_hwnd))
          Else
            IIf(Opt("SendAttachMode"), AttachThreadInput(GetForegroundWindow_()))
          EndIf
          SendUnicode(Asc(Mid(keys, i, 1)), 1)
          DetachThreadInput()
        Next
        ProcedureReturn
      EndIf

      NewList key_list.KeyInfo()
      SendKeyParse(keys.s, key_list())

      old_state = ToggleKeyState(#VK_CAPITAL, #TOGGLE_OFF)
      presist_modifier.u = 0

      ForEach key_list()
        If __keep_active_hwnd
          IIf(IsIconic_(__keep_active_hwnd), ShowWindow_(__keep_active_hwnd, #SW_RESTORE))
          SetForegroundWindow_(__keep_active_hwnd)
          IIf(Opt("SendAttachMode"), AttachThreadInput(__keep_active_hwnd))
        Else
          IIf(Opt("SendAttachMode"), AttachThreadInput(GetForegroundWindow_()))
        EndIf

        Select key_list()\toggle
        Case #TOGGLE_ON, #TOGGLE_OFF
          ToggleKeyState(key_list()\vk, key_list()\toggle)

        Case #TOGGLE_DOWN
          SendModifier(key_list()\vk, 0)
          presist_modifier | key_list()\vk

        Case #TOGGLE_UP
          SendModifier(key_list()\vk, #KEYEVENTF_KEYUP)
          presist_modifier & (~key_list()\vk)

        Case #TOGGLE_ASC
          SendAsc(key_list()\count)

        Case #TOGGLE_UNICODE
          SendUnicode(key_list()\vk, key_list()\count)

        Default
          SendKey(key_list()\vk, key_list()\modifier & (~presist_modifier), key_list()\count, key_list()\extend)
        EndSelect

        DetachThreadInput()
      Next

      IIf(Opt("SendCapslockMode") <> 0, ToggleKeyState(#VK_CAPITAL, old_state))
    EndProcedure

    Structure HotKeyThreadInfo
      thread_pbid.i
      thread_id.i
      semaphore.i
      mutex.i
      init.l
      last_result.l
      last_pressed.l
      List queue.i()
    EndStructure

    #WM_HOTKEY_SET = #WM_USER + $101

    Procedure HotkeyExecuteThread(*info.HotKeyThreadInfo)
      Repeat
        LockMutex(*info\mutex)
        If SelectElement(*info\queue(), 0)
          CallFunctionFast(*info\queue())
          DeleteElement(*info\queue())
        EndIf
        UnlockMutex(*info\mutex)
        Delay(10)
      ForEver
    EndProcedure

    Procedure HotKeyThread(*info.HotKeyThreadInfo)
      Static NewMap hotkey_map.l()

      *info\thread_id = GetCurrentThreadId_()
      ; force the system to create the message queue
      PeekMessage_(@msg.MSG, 0, #WM_USER, #WM_USER, #PM_NOREMOVE)
      ; to indicate that the thread is ready to receive posted messages
      SignalSemaphore(*info\semaphore)

      Repeat
        GetMessage_(@msg.MSG, 0, 0, 0)
        hotkey.l = msg\wparam
        callback.i = msg\lparam

        Select msg\message
        Case #WM_HOTKEY
          ; use quene, dont call it to avoid WM_HOTKEY message reentry (set hotkey in callback)
          ; wparam is hotkey id, find the callback addr in map and then add into queue
          LockMutex(*info\mutex)
          If FindMapElement(hotkey_map(), Str(hotkey))
            *info\last_pressed = hotkey
            AddElement(*info\queue())
            *info\queue() = hotkey_map()
          EndIf
          UnlockMutex(*info\mutex)

        Case #WM_HOTKEY_SET
          If callback
            vk.u = hotkey & $FF ; hotkey is combine of vk and modifier
            modifier.u = hotkey >> 16
            *info\last_result = RegisterHotKey_(0, hotkey, modifier, vk)
            IIf(*info\last_result, hotkey_map(Str(hotkey)) = callback) ; store hotkey id => callback addr in map
          Else
            *info\last_result = UnregisterHotKey_(0, hotkey)
            IIf(*info\last_result, DeleteMapElement(hotkey_map(), Str(hotkey)))
          EndIf
          SignalSemaphore(*info\semaphore)
        EndSelect

      ForEver
    EndProcedure

    Structure HotkeyName
      name.s
    EndStructure

    Procedure.i HotKeySet(keys.s, *callback = 0)
      FunctionInit()
      Static info.HotKeyThreadInfo
      Static NewMap hotkey_str.HotkeyName()

      ; only one HotKeyThread and one HotkeyExecuteThread will be started
      IIf(__global_mutex, LockMutex(__global_mutex))
      If Not info\init And Not IsThread(info\thread_pbid)
        info\mutex = CreateMutex()
        IIf(info\mutex = 0, ProcedureReturn 0)

        info\semaphore = CreateSemaphore()
        IIf(info\semaphore = 0, ProcedureReturn 0)

        IIf(CreateThread(@HotkeyExecuteThread(), @info) = 0, ProcedureReturn 0)

        info\thread_pbid = CreateThread(@HotKeyThread(), @info)
        IIf(info\thread_pbid = 0, ProcedureReturn 0)

        WaitSemaphore(info\semaphore)
        info\init = #True
      EndIf
      IIf(__global_mutex, UnlockMutex(__global_mutex))

      If keys = "@HotKeyPressed" And *callback = 0
        IIf(FindMapElement(hotkey_str(), Str(info\last_pressed)), ProcedureReturn @hotkey_str()\name, ProcedureReturn @"")
      EndIf

      NewList key_list.KeyInfo()
      SendKeyParse(keys.s, key_list())
      IIf(SelectElement(key_list(), 0) = 0, ProcedureReturn 0)
      IIf(key_list()\toggle <> 0, ProcedureReturn 0)

      ; key_list()\modifier only combine with #MOD_LALT, #MOD_LCONTROL, #MOD_LSHIFT, #MOD_LWIN
      ; these values are identical to #MOD_ALT, #MOD_CONTROL, #MOD_SHIFT, #MOD_WIN
      hotkey.l = key_list()\modifier << 16 | key_list()\vk
      PostThreadMessage_(info\thread_id, #WM_HOTKEY_SET, hotkey, *callback)
      WaitSemaphore(info\semaphore)
      If info\last_result
        If *callback
          AddMapElement(hotkey_str(), Str(hotkey))
          hotkey_str()\name = keys
        Else
          DeleteMapElement(hotkey_str(), Str(hotkey))
        EndIf
        ProcedureReturn 1
      Else
        ProcedureReturn 0
      EndIf
    EndProcedure

  ; ===========================================================================
  ;  Math functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Message Boxes and Dialogs functions
  ; ===========================================================================

    Procedure.l ToolTip(text.s = "", x.l = #PB_Ignore, y.l = #PB_Ignore, title.s = "", icon.l = 0, option.l = 0)
      FunctionInit()
      #TTF_TRACK = $20

      If text = ""
        IIf(__tip_hwnd, DestroyWindow_(__tip_hwnd))
        __tip_hwnd = 0
      Else
        ti.TOOLINFO\cbSize = SizeOf(ti)
        ti\uFlags = #TTF_TRACK
        ti\lpszText = @text

        If x = #PB_Ignore Or y = #PB_Ignore
          GetCursorPos_(@p.POINT)
          IIf(x = #PB_Ignore, x = p\x + 16)
          IIf(y = #PB_Ignore, y = p\y + 16)
        EndIf

        If __tip_hwnd = 0 Or Not IsWindow_(__tip_hwnd)
          __tip_hwnd = CreateWindowEx_(#WS_EX_TOPMOST, #TOOLTIPS_CLASS, 0, #TTS_NOPREFIX | #TTS_ALWAYSTIP, #CW_USEDEFAULT, #CW_USEDEFAULT, #CW_USEDEFAULT, #CW_USEDEFAULT, 0, 0, 0, 0)
          SendMessage_(__tip_hwnd, #TTM_ADDTOOL, 0, @ti)
          SendMessage_(__tip_hwnd, #TTM_SETMAXTIPWIDTH, 0, GetSystemMetrics_(#SM_CXSCREEN))
        EndIf

        style.l = GetWindowLong_(__tip_hwnd, #GWL_STYLE)
        IIf(option & 1, style | #TTS_BALLOON, style & ~#TTS_BALLOON)
        SetWindowLong_(__tip_hwnd, #GWL_STYLE, style)

        SendMessage_(__tip_hwnd, #TTM_TRACKPOSITION, 0, y << 16 + x)
        SendMessage_(__tip_hwnd, #TTM_TRACKACTIVATE, #True, @ti)
        SendMessage_(__tip_hwnd, #TTM_SETTITLE, icon, Left(title, 99))
        SendMessage_(__tip_hwnd, #TTM_UPDATETIPTEXT, 0, @ti)
      EndIf
    EndProcedure

    Procedure.l MsgBoxEx_CBTProc(code.l, wParam.l, lParam.l)
      If code = #HCBT_ACTIVATE
        For i = 1 To CountString(__custom_button, "|") + 1
          ControlSetText(Str(wParam), "", Str(i), StringField(__custom_button, i, "|"))
        Next
      EndIf
      ProcedureReturn CallNextHookEx_(0, code, wParam, lParam)
    EndProcedure

    Procedure.l MsgBox(flag = 0, title.s = "", text.s = "", timeout.l = 0, hwnd.i = 0)
      FunctionInit()
      timeout * 1000
      MessageBoxTimeout_.MessageBoxTimeout = GetProcAddress_(GetModuleHandle_("user32"), "MessageBoxTimeoutW")
      If MessageBoxTimeout_
        result.l = MessageBoxTimeout_(hwnd, text, title, flag, 0, timeout)
      Else
        result.l = MessageBox_(hwnd, text, title, flag)
      EndIf
      IIf(result = 32000, ProcedureReturn -1, ProcedureReturn result)
    EndProcedure

    Procedure.l MsgBoxEx(custom_button.s, flag = 0, title.s = "", text.s = "", timeout.l = 0, hwnd.i = 0)
      __custom_button = custom_button
      hook.i = SetWindowsHookEx_(#WH_CBT, @MsgBoxEx_CBTProc(), 0, GetCurrentThreadId_())
      result.l = MsgBox(flag, title, text, timeout, hwnd)
      UnhookWindowsHookEx_(hook)
      ProcedureReturn result
    EndProcedure

    Enumeration
      #MANDATORY_NONE     = 0
      #MANDATORY_ENTER    = 1
      #MANDATORY_NOCANCEL = 2

      #PROGRESS_WIN = 1401
      #GADGET_MAINTEXT
      #GADGET_SUBTEXT
      #GADGET_PROGRESS
    EndEnumeration

    Procedure InputBox_GadgetResize()
      height.l = WindowHeight(gui\input_win)
      width.l = WindowWidth(gui\input_win)
      ResizeGadget(gui\gadget_string, 10, height - 65, width - 20, 25)
      ResizeGadget(gui\gadget_text, 10, 10, width - 20, height - 80)

      If IsGadget(gui\button_cancel)
        ResizeGadget(gui\button_ok, (width - 88) / 4 - 20, height - 35, 88, 28)
        ResizeGadget(gui\button_cancel, (width - 88) / 4 * 3 + 20, height - 35, 88, 28)
      Else
        ResizeGadget(gui\button_ok, (width - 88) / 2 , height - 35, 88, 28)
      EndIf
    EndProcedure

    Procedure InputBox_ResizeHandler()
      IIf(EventWindow() = gui\input_win, InputBox_GadgetResize())
    EndProcedure

    Procedure InputBox_DefPushButton(set, nil)
      SendMessage_(set, #BM_SETSTYLE, GetWindowLong_(set, #GWL_STYLE) | #BS_DEFPUSHBUTTON, #True)
      SendMessage_(nil, #BM_SETSTYLE, GetWindowLong_(nil, #GWL_STYLE) & ~#BS_DEFPUSHBUTTON, #True)
    EndProcedure

    Structure InputBoxInfo
      title.s
      prompt.s
      value.s
      password.s
      width.l
      height.l
      left.l
      top.l
      timeout.l
      parent.i
      custom_button.s
      result.s
    EndStructure

    Procedure InputBoxThread(*info.InputBoxInfo)
      Define title.s, prompt.s, value.s, password.s, width.l, height.l, left.l, top.l, timeout.l, parent.i, custom_button.s
      Do(title = *info\title, prompt = *info\prompt, value = *info\value, password = *info\password)
      Do(width = *info\width, height = *info\height, left = *info\left, top = *info\top)
      Do(timeout = *info\timeout, parent = *info\parent, custom_button = *info\custom_button)

      FunctionInit()
      Define result.s = "", text_ok.s = "OK", text_cancel.s = "Cancel", passflag.l = 0, passchar.u = 0
      Define mandatory.l = #MANDATORY_NONE, numstr.s = "", limit.l = 0

      c.s = Left(password, 1)
      If StringIsLower(c)
        passflag = #PB_String_Password
      ElseIf c <> " " And c <> ""
        passchar = Asc(c)
      EndIf

      c.s = LCase(Mid(password, 2, 1))
      IIf(c = "m", mandatory = #MANDATORY_ENTER)
      IIf(c = "f", mandatory = #MANDATORY_NOCANCEL)

      For i = 1 To Len(password)
        c.s = Mid(password, i, 1)
        IIf(StringIsDigit(c), numstr + c)
      Next
      limit = Val(numstr)

      IIf(width <= 0, width = 285)
      IIf(height <= 0, height = 190)
      If custom_button <> ""
        text_ok = StringField(custom_button, 1, "|")
        text_cancel = StringField(custom_button, 2, "|")
      EndIf

      DisableDebugger
      gui\input_win = OpenWindow(#PB_Any, 0, 0, width, height, title, #PB_Window_SystemMenu | #PB_Window_ScreenCentered | #PB_Window_SizeGadget | #PB_Window_Invisible, parent)
      EnableDebugger

      If gui\input_win = 0
        *info\result = ""
        ErrorReturn(3, 0)
      EndIf

      gui\gadget_text = TextGadget(#PB_Any, 0, 0, 0, 0, prompt)

      gui\gadget_string = StringGadget(#PB_Any, 0, 0, 0, 0, value, passflag)
      IIf(passchar, SendMessage_(GadgetID(gui\gadget_string), #EM_SETPASSWORDCHAR, passchar, 0))
      IIf(limit, SendMessage_(GadgetID(gui\gadget_string), #EM_LIMITTEXT, limit, 0))

      gui\button_ok = ButtonGadget(#PB_Any, 0, 0, 0, 0, text_ok, #PB_Button_Default)
      hwnd_ok.i = GadgetID(gui\button_ok)
      hwnd_default.i = hwnd_ok

      If mandatory <> #MANDATORY_NOCANCEL
        gui\button_cancel = ButtonGadget(#PB_Any, 0, 0, 0, 0, text_cancel)
        hwnd_cancel.i = GadgetID(gui\button_cancel)
      EndIf

      SetActiveGadget(gui\gadget_string)
      AddKeyboardShortcut(gui\input_win, #PB_Shortcut_Return, #PB_Shortcut_Return)
      AddKeyboardShortcut(gui\input_win, #PB_Shortcut_Escape, #PB_Shortcut_Escape)
      IIf(timeout <> 0, AddWindowTimer(gui\input_win, 0, timeout * 1000))

      WindowBounds(gui\input_win, 190, 115, 1024, 768)
      BindEvent(#PB_Event_SizeWindow, @InputBox_ResizeHandler())
      InputBox_GadgetResize()

      ResizeWindow(gui\input_win, left, top, #PB_Ignore, #PB_Ignore)
      HideWindow(gui\input_win, #False)

      Repeat
        DisableDebugger
        event.i =  WaitWindowEvent()
        EnableDebugger
        Select event
          Case #PB_Event_CloseWindow: SendMessage_(hwnd_cancel, #BM_CLICK, 0, 0) ; don't use postevent because it only work in main thread
          Case #PB_Event_Timer: ErrorBreak(2)

          Case #PB_Event_Menu
            Select EventMenu()
              Case #PB_Shortcut_Return: SendMessage_(hwnd_default, #BM_CLICK, 0, 0)
              Case #PB_Shortcut_Escape: SendMessage_(hwnd_cancel, #BM_CLICK, 0, 0)
            EndSelect

          Case #PB_Event_Gadget
            Select EventGadget()
              Case gui\button_ok
                result = GetGadgetText(gui\gadget_string)
                If mandatory <> #MANDATORY_NONE And result = ""
                  SetActiveGadget(gui\gadget_string)
                  MessageBeep_(#MB_ICONWARNING)
                Else
                  Break
                EndIf

              Case gui\button_cancel
                If mandatory = #MANDATORY_NOCANCEL
                  SetActiveGadget(gui\gadget_string)
                  MessageBeep_(#MB_ICONWARNING)
                Else
                  ErrorBreak(1)
                EndIf
            EndSelect

          Default
            If mandatory <> #MANDATORY_NOCANCEL
              If GetFocus_() = hwnd_cancel
                If hwnd_default = hwnd_ok
                  InputBox_DefPushButton(hwnd_cancel, hwnd_ok)
                  hwnd_default = hwnd_cancel
                EndIf
              Else
                If hwnd_default = hwnd_cancel
                  InputBox_DefPushButton(hwnd_ok, hwnd_cancel)
                  hwnd_default = hwnd_ok
                EndIf
              EndIf
            EndIf
        EndSelect
      ForEver
      UnbindEvent(#PB_Event_SizeWindow, @InputBox_ResizeHandler())
      CloseWindow(gui\input_win)

      *info\result = result
    EndProcedure

    Procedure.s InputBox(title.s = "", prompt.s = "", value.s = "", password.s = "", width.l = -1, height.l = -1, left.l = #PB_Ignore, top.l = #PB_Ignore, timeout.l = 0, parent.i = 0, custom_button.s = "")
      info.InputBoxInfo
      Do(info\title = title, info\prompt = prompt, info\value = value, info\password = password)
      Do(info\width = width, info\height = height, info\left = left, info\top = top)
      Do(info\timeout = timeout, info\parent = parent, info\custom_button = custom_button)

      thread = CreateThread(@InputBoxThread(), @info)
      WaitThread(thread)
      ProcedureReturn info\result
    EndProcedure

    Procedure.s InputBoxEx(custom_button.s = "", title.s = "", prompt.s = "", value.s = "", password.s = "", width.l = -1, height.l = -1, left.l = #PB_Ignore, top.l = #PB_Ignore, timeout.l = 0, parent.i = 0)
      ProcedureReturn InputBox(title, prompt, value, password, width, height, left, top, timeout, parent, custom_button)
    EndProcedure

    Procedure GUIEventLoop(*gth.GUIThread)
      *gth\semaphore = CreateSemaphore()
      *gth\init_done = #True

      Repeat
        DisableDebugger
        WindowEvent()
        EnableDebugger

        If *gth\parent_thread And GetExitCodeThread_(*gth\parent_thread, @code.l)
          IIf(code <> #STILL_ACTIVE, Break)
        EndIf

        IIf(TrySemaphore(*gth\semaphore), Break)
        Delay(1)
      ForEver

      CloseHandle_(*gth\parent_thread)
      *gth\parent_thread = 0
      FreeSemaphore(*gth\semaphore)
      *gth\semaphore = 0
      CloseWindow(*gth\win)
      *gth\win = 0
    EndProcedure

    Procedure GUICreateThread(*gth.GUIThread, *thread_main, *info)
      *gth\init_done = 0
      th.i = CreateThread(*thread_main, *info)
      While *gth\init_done = 0 And IsThread(th)
        Delay(1)
      Wend
      process.i = GetCurrentProcess_()
      DuplicateHandle_(process, GetCurrentThread_(), process, @*gth\parent_thread, 0, #True, #DUPLICATE_SAME_ACCESS)
    EndProcedure

    Procedure GUIClose(*gth.GUIThread)
      If *gth\win And *gth\semaphore
        SignalSemaphore(*gth\semaphore)
        While IsWindow(*gth\win)
          Delay(1)
        Wend
      EndIf
    EndProcedure

    Procedure GUISetXY(*x, *y, width, height, visible.l = #False)
      Do(x.l = PeekL(*x), y.l = PeekL(*y))
      If x < 0 Or y < 0
        If visible
          IIf(x < 0, x = #PB_Ignore)
          IIf(y < 0, y = #PB_Ignore)
        Else ; center the window if not visible
          SystemParametersInfo_(#SPI_GETWORKAREA, 0, @rect.RECT, 0)
          IIf(x < 0, x = rect\left + (((rect\right - rect\left) - width) / 2))
          IIf(y < 0, y = rect\top + (((rect\bottom - rect\top) - height) / 2))
        EndIf
      EndIf
      Do(PokeL(*x, x), PokeL(*y, y))
    EndProcedure

    Structure ProgressInfo
      title.s
      maintext.s
      subtext.s
      x.l
      y.l
      opt.l
      pgui.i
    EndStructure

    Procedure ProgressState(*gui.PureAutoItGUI, percent.l = -1, title.s = #Null$, maintext.s = #Null$, subtext.s = #Null$, x.l = -1, y.l = -1, opt.l = -1)
      Define hwnd.i = WindowID(*gui\progress\win), visible.l = IsWindowVisible_(hwnd)
      If opt >= 0
        IIf(opt & #DLG_NOTONTOP = 0, sticky.l = 1, sticky = 0)
        IIf(opt & #DLG_MOVEABLE = 0, disable.l = 1, disable.l = 0)
        StickyWindow(*gui\progress\win, sticky)
        DisableWindow(*gui\progress\win, disable)

        style.l = GetWindowLong_(hwnd, #GWL_STYLE)
        IIf(opt & #DLG_NOTITLE, style & (~#WS_CAPTION), style | #WS_CAPTION)
        SetWindowLong_(hwnd, #GWL_STYLE, style)
        SetWindowLong_(hwnd, #GWL_EXSTYLE, GetWindowLong_(hwnd, #GWL_EXSTYLE) | #WS_EX_TOOLWINDOW)
      EndIf

      GUISetXY(@x, @y, 300, 100, visible)
      ResizeWindow(*gui\progress\win, x, y, 300, 100) ; always call ResizeWindow so that style change work

      IIf(title <> #Null$, SetWindowTitle(*gui\progress\win, title))
      IIf(percent >= 0, SetGadgetState(*gui\gadget_progress, percent))
      IIf(subtext <> #Null$, SetGadgetText(*gui\gadget_subtext, subtext))
      IIf(maintext <> #Null$, SetGadgetText(*gui\gadget_maintext, maintext))
    EndProcedure

    Procedure ProgressMain(*info.ProgressInfo)
      Define title.s, maintext.s, subtext.s, x.l, y.l, opt.l
      Do(title = *info\title, maintext = *info\maintext, subtext = *info\subtext)
      Do(x = *info\x, y = *info\y, opt = *info\opt)
      *gui.PureAutoItGUI = *info\pgui

      DisableDebugger
      *gui\progress\win = OpenWindow(#PB_Any, x, y, 0, 0, "", #PB_Window_Invisible)
      EnableDebugger
      IIf(*gui\progress\win = 0, ProcedureReturn)

      *gui\gadget_maintext = TextGadget(#PB_Any, 19, 4, 300, 24, "")
      *gui\gadget_subtext = TextGadget(#PB_Any, 20, 55, 300, 50, "")
      *gui\gadget_progress = ProgressBarGadget(#PB_Any, 20, 30, 260, 20, 0, 100, #PB_ProgressBar_Smooth)

      Define hwnd_maintext.i = GadgetID(*gui\gadget_maintext)
      Define font.i = SendMessage_(hwnd_maintext, #WM_GETFONT, 0, 0), main_font.i = 0
      If font And GetObject_(font, SizeOf(LOGFONT), @lf.LOGFONT)
        lf\lfWeight = #FW_HEAVY
        IIf(lf\lfHeight < 0, lf\lfHeight - 1, lf\lfHeight + 1)
        main_font = CreateFontIndirect_(@lf)
        IIf(main_font, SendMessage_(hwnd_maintext, #WM_SETFONT, main_font, #True))
      EndIf

      ProgressState(*gui, -1, title, maintext, subtext, x, y, opt)
      HideWindow(*gui\progress\win, #False, #PB_Window_NoActivate)

      GUIEventLoop(@*gui\progress)

      IIf(main_font, DeleteObject_(main_font))
    EndProcedure

    Procedure ProgressOn(title.s = #Null$, maintext.s = #Null$, subtext.s = #Null$, x.l = -1, y.l = -1, opt.l = 0)
      FunctionInit()
      If gui\progress\win
        ProgressState(@gui, -1, title, maintext, subtext, x, y, opt)
      Else
        IIf(title = #Null$, title = "Progress")
        IIf(maintext = #Null$, maintext = "")
        IIf(subtext = #Null$, subtext = "")

        info.ProgressInfo
        Do(info\title = title, info\maintext = maintext, info\subtext = subtext)
        Do(info\x = x, info\y = y, info\opt = opt, info\pgui = @gui)

        GUICreateThread(@gui\progress, @ProgressMain(), @info)
      EndIf
    EndProcedure

    Procedure ProgressSet(percent.l = -1, subtext.s = #Null$, maintext.s = #Null$)
      FunctionInit()
      If gui\progress\win
        ProgressState(@gui, percent, #Null$, maintext, subtext)
      EndIf
    EndProcedure

    Procedure ProgressOff()
      FunctionInit()
      GUIClose(@gui\progress)
    EndProcedure

    Import "gdiplus.lib"
      GdiplusStartup(*token, *input, *output)
      GdiplusShutdown(token.i)
      GdipCreateHBITMAPFromBitmap(*bitmap, *hbmReturn, background.l)
      GdipGetImageWidth(gdipbmp.i, *width)
      GdipGetImageHeight(gdipbmp.i, *height)
      GdipDisposeImage(gdipbmp.i)
      GdipCreateBitmapFromFile(filename.p-unicode, *bitmap)
      GdipCreateFromHDC(hdc.i, *graphics)
      GdipCreateBitmapFromGraphics(width.l, height.l, graphic.i, *bitmap)
      GdipDrawImageRectI(graphic.i, image.i, x.l, y.l, width.l, height.l)
      GdipGetImageGraphicsContext(image.i, *graphics)
      GdipDeleteGraphics(graphic.i)
    EndImport

    Structure GdiplusStartupInput
      GdiplusVersion.l
      *DebugEventCallback
      SuppressBackgroundThread.l
      SuppressExternalCodecs.l
    EndStructure

    Procedure.i SplashImageLoad(path.s, *width, *height)
      Define token.i = 0, si.GdiplusStartupInput\GdiplusVersion = 1, width.l = PeekL(*width), height.l = PeekL(*height)
      Define result.i = 0, gdipbmp1.i = 0, gdipbmp2.i = 0, graphic1.i = 0, graphic2.i = 0, hdc.i = GetDC_(0)
      Repeat
        IIf(hdc = 0, Break)
        IIf(GdiplusStartup(@token, @si, 0) <> #S_OK, Break)
        IIf(GdipCreateFromHDC(hdc, @graphic1) <> #S_OK, Break)
        IIf(GdipCreateBitmapFromFile(path, @gdipbmp1) <> #S_OK, Break)
        IIf(width < 0, GdipGetImageWidth(gdipbmp1, @width))
        IIf(height < 0, GdipGetImageHeight(gdipbmp1, @height))
        IIf(GdipCreateBitmapFromGraphics(width, height, graphic1, @gdipbmp2) <> #S_OK, Break)
        IIf(GdipGetImageGraphicsContext(gdipbmp2, @graphic2) <> #S_OK, Break)
        IIf(GdipDrawImageRectI(graphic2, gdipbmp1, 0, 0, width, height) <> #S_OK, Break)
        GdipCreateHBITMAPFromBitmap(gdipbmp2, @result, 0)
      Until #True
      IIf(gdipbmp1, GdipDisposeImage(gdipbmp1))
      IIf(gdipbmp2, GdipDisposeImage(gdipbmp2))
      IIf(graphic1, GdipDeleteGraphics(graphic1))
      IIf(graphic2, GdipDeleteGraphics(graphic2))
      IIf(token, GdiplusShutdown(token))
      IIf(hdc, ReleaseDC_(0, hdc))
      PokeL(*width, width)
      PokeL(*height, height)
      ProcedureReturn result
    EndProcedure

    Structure SplashInfo
      title.s
      text.s
      file.s
      width.l
      height.l
      x.l
      y.l
      opt.l
      fontname.s
      fontsz.l
      fontwt.l
      done.l
      pgui.i
    EndStructure

    Procedure SplashFont(*gui.PureAutoItGUI, fontname.s = "", fontsz.l = 12, fontwt.l = #FW_DONTCARE)
      Define hwnd.i = GadgetID(*gui\gadget_splashtext)
      Define font.i = SendMessage_(hwnd, #WM_GETFONT, 0, 0), text_font.i = 0
      If font And GetObject_(font, SizeOf(LOGFONT), @lf.LOGFONT)
        hdc.i = GetDC_(0)
        logy.l = GetDeviceCaps_(hdc, #LOGPIXELSY)
        ReleaseDC_(0, hdc)

        lf\lfWeight = fontwt
        lf\lfHeight = fontsz * logy / 72
        If fontname <> ""
          lf\lfCharSet = 0
          PokeS(@lf\lfFaceName, Left(fontname, 31))
        EndIf
        text_font = CreateFontIndirect_(@lf)
      EndIf

      IIf(*gui\splash_font, DeleteObject_(*gui\splash_font))
      *gui\splash_font = 0

      If text_font
        *gui\splash_font = text_font
        SendMessage_(hwnd, #WM_SETFONT, text_font, #True)
      EndIf
    EndProcedure

    Procedure SplashContent(*gui.PureAutoItGUI, file.s, text.s, *width, *height)
      IIf(*gui\splash_bmp, DeleteObject_(*gui\splash_bmp))
      *gui\splash_bmp = 0

      Define bmp.i = 0
      If file <> "" And PathFileExists_(file)
        bmp = SplashImageLoad(file, *width, *height)
        *gui\splash_bmp = bmp
        IIf(bmp = 0, text = file)
      EndIf

      SetGadgetState(*gui\gadget_image, bmp)
      SetGadgetText(*gui\gadget_splashtext, text)

      If bmp
        HideGadget(*gui\gadget_image, #False)
        HideGadget(*gui\gadget_splashtext, #True)
      Else
        HideGadget(*gui\gadget_splashtext, #False)
        HideGadget(*gui\gadget_image, #True)
      EndIf
    EndProcedure

    Procedure SplashState(*gui.PureAutoItGUI, title.s = "", text.s = "", file.s = "", width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = -1, fontname.s = "", fontsz.l = 12, fontwt.l = #FW_DONTCARE)
      SplashFont(*gui, fontname, fontsz, fontwt)
      SplashContent(*gui, file, text, @width, @height) ; width or height < 0, it will modify it to image width/height

      Define hwnd.i = WindowID(*gui\splash\win), visible.l = IsWindowVisible_(hwnd)
      If opt >= 0
        IIf(opt & #DLG_NOTONTOP = 0, sticky.l = 1, sticky = 0)
        IIf(opt & #DLG_MOVEABLE = 0, disable.l = 1, disable.l = 0)
        IIf(sticky, SetForegroundWindow_(hwnd)) ; sometimes StickyWindow not work if window is not foregrounded
        StickyWindow(*gui\splash\win, sticky)
        DisableWindow(*gui\splash\win, disable)

        style.l = GetWindowLong_(hwnd, #GWL_STYLE)
        IIf(opt & #DLG_NOTITLE, style & (~#WS_CAPTION), style | #WS_CAPTION)
        SetWindowLong_(hwnd, #GWL_STYLE, style)
        SetWindowLong_(hwnd, #GWL_EXSTYLE, GetWindowLong_(hwnd, #GWL_EXSTYLE) | #WS_EX_TOOLWINDOW)

        gadget_hwnd.i = GadgetID(*gui\gadget_splashtext)
        style.l = GetWindowLong_(gadget_hwnd, #GWL_STYLE)
        style & ~(#SS_RIGHT | #SS_CENTER | #SS_CENTERIMAGE)
        If opt & #DLG_TEXTRIGHT
          style | #SS_RIGHT
        ElseIf opt & #DLG_TEXTLEFT = 0
          style | #SS_CENTER
        EndIf
        IIf(opt & #DLG_TEXTVCENTER, style | #SS_CENTERIMAGE)
        SetWindowLong_(gadget_hwnd, #GWL_STYLE, style)
      EndIf

      IIf(width < 0, width = WindowWidth(*gui\splash\win))
      IIf(height < 0, height = WindowHeight(*gui\splash\win))
      GUISetXY(@x, @y, width, height, visible)
      ResizeWindow(*gui\splash\win, x, y, width, height) ; always call ResizeWindow so that style change work

      ResizeGadget(*gui\gadget_splashtext, 10, 10, width - 20, height - 20)
      ResizeGadget(*gui\gadget_image, 0, 0, width, height)
      SetWindowTitle(*gui\splash\win, title)
    EndProcedure

    Procedure SplashMain(*info.SplashInfo)
      Define title.s, text.s, file.s, width.l, height.l, x.l, y.l, opt.l, fontname.s, fontsz.l, fontwt.l
      Do(title = *info\title, text = *info\text, file = *info\file, width = *info\width, height = *info\height, x = *info\x, y = *info\y)
      Do(opt = *info\opt, fontname = *info\fontname, fontsz = *info\fontsz, fontwt = *info\fontwt)
      *gui.PureAutoItGUI = *info\pgui

      DisableDebugger
      *gui\splash\win = OpenWindow(#PB_Any, 0, 0, 500, 400, "", #PB_Window_Invisible)
      EnableDebugger
      IIf(*gui\splash\win = 0, ProcedureReturn)

      *gui\gadget_image = ImageGadget(#PB_Any, 0, 0, 0, 0, 0)
      *gui\gadget_splashtext = TextGadget(#PB_Any, 0, 0, 0, 0, "")

      SplashState(*gui, title, text, file, width, height, x, y, opt, fontname, fontsz, fontwt)
      HideWindow(*gui\splash\win, #False)

      GUIEventLoop(@*gui\splash)
    EndProcedure

    Procedure SplashTextImageOn(title.s = "", text.s = "", file.s = "", width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = 0, fontname.s = "", fontsz.l = 12, fontwt.l = #FW_DONTCARE)
      FunctionInit()
      If gui\splash\win
        SplashState(@gui, title, text, file, width, height, x, y, opt, fontname, fontsz, fontwt)
      Else
        info.SplashInfo
        Do(info\title = title, info\text = text, info\file = file, info\width = width, info\height = height, info\x = x, info\y = y)
        Do(info\opt = opt, info\fontname = fontname, info\fontsz = fontsz, info\fontwt = fontwt, info\pgui = @gui)

        GUICreateThread(@gui\splash, @SplashMain(), @info)
      EndIf
    EndProcedure

    Procedure SplashImageOn(title.s, file.s, width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = 0)
      SplashTextImageOn(title, "", file, width, height, x, y, opt)
    EndProcedure

    Procedure SplashTextOn(title.s = "", text.s = "", width.l = -1, height.l = -1, x.l = -1, y.l = -1, opt.l = 0, fontname.s = "", fontsz.l = 12, fontwt.l = #FW_DONTCARE)
      SplashTextImageOn(title, text, "", width, height, x, y, opt, fontname, fontsz, fontwt)
    EndProcedure

    Procedure SplashOff()
      FunctionInit()
      GUIClose(@gui\splash)
    EndProcedure

  ; ===========================================================================
  ;  Misc. functions
  ; ===========================================================================

    Structure AdlibFuncInfo
      addr.i
      time.l
      last_execute.q
    EndStructure

    Structure AdlibInfo
      init.l
      thread_pbid.i
      mutex.i
      Map queue.AdlibFuncInfo()
    EndStructure

    Procedure AdlibExecute(*info.AdlibInfo)
      Repeat
        now.q = ElapsedMilliseconds()
        LockMutex(*info\mutex)
        ForEach *info\queue()
          If now - *info\queue()\last_execute > *info\queue()\time
            *info\queue()\last_execute = now
            CallFunctionFast(*info\queue()\addr) ; callback maybe unset the adlib, so dont access queue after call
          EndIf
        Next
        UnlockMutex(*info\mutex)
        Delay(1)
      ForEver
    EndProcedure

    Procedure.l AdlibRegister(*callback, time.l = 250)
      FunctionInit()
      Static info.AdlibInfo, *last_callback

      ; only one AdlibExecute will be started
      IIf(__global_mutex, LockMutex(__global_mutex))
      If Not info\init And Not IsThread(info\thread_pbid)
        info\mutex = CreateMutex()
        IIf(info\mutex = 0, ProcedureReturn 0)

        info\thread_pbid = CreateThread(@AdlibExecute(), @info)
        IIf(info\thread_pbid = 0, ProcedureReturn 0)
        info\init = #True
      EndIf
      IIf(__global_mutex, UnlockMutex(__global_mutex))

      IIf(*callback = 0, *callback = *last_callback)
      IIf(*callback = 0 And time >= 0, ProcedureReturn 0)

      LockMutex(info\mutex)
      If time >= 0
        AddMapElement(info\queue(), Str(*callback))
        info\queue()\addr = *callback
        info\queue()\time = time
        info\queue()\last_execute = ElapsedMilliseconds()
        *last_callback = *callback
        result.l = 1
      Else
        DeleteMapElement(info\queue(), Str(*callback))
        result.l = MapSize(info\queue())
      EndIf
      UnlockMutex(info\mutex)
      ProcedureReturn result
    EndProcedure

    Procedure.l AdlibUnRegister(*callback = 0)
      ProcedureReturn AdlibRegister(*callback, -1)
    EndProcedure

    Procedure.l Opt(option.s, value.l = #DEADBEFF)
      If Not __options("")
        __options("") = 1
        __options("winsearchchildren") = 0
        __options("wintitlematchmode") = #MatchStart
        __options("windetecthiddentext") = 0
        __options("winwaitdelay") = 250
        __options("caretcoordmode") = #AbsoluteScreen
        __options("windelay") = 10
        __options("controldelay") = 10
        __options("mousedelay") = 10
        __options("sendkeydelay") = 5
        __options("sendkeydowndelay") = 5
        __options("sendattachmode") = 0
        __options("sendcapslockmode") = 1
        __options("mouseclickdelay") = 10
        __options("mouseclickdowndelay") = 10
        __options("mouseclickdragdelay") = 250
        __options("mousecoordmode") = 1
        __options("pixelcoordmode") = 1
      EndIf

      IIf(option = "", ProcedureReturn 0)

      option = LCase(option)
      prev.l = __options(option)
      IIf(value <> #DEADBEFF, __options(option) = value)
      ProcedureReturn prev
    EndProcedure

    Procedure.l CDTray(drive.s, status.s)
      FunctionInit()
      IIf(mciSendString_("set cdaudio!" + drive + " door " + status + " wait", 0, 0, 0) <> 0, ErrorReturn(1, 0), ProcedureReturn 1)
    EndProcedure

    Procedure.l OnAutoItExitRegister(*callback)
      FunctionInit()
      ForEach atexit()
        If atexit() = *callback
          Extended(1)
          ProcedureReturn 1
        EndIf
      Next

      AddElement(atexit())
      atexit() = *callback
      ProcedureReturn 1
    EndProcedure

    Procedure.l OnAutoItExitUnRegister(*callback)
      FunctionInit()
      ForEach atexit()
        If atexit() = *callback
          DeleteElement(atexit())
          ProcedureReturn 1
        EndIf
      Next
      ProcedureReturn 0
    EndProcedure

    Procedure AtExitHandler()
      ForEach atexit()
        CallFunctionFast(atexit())
      Next
    EndProcedure

    ; use fasm macro magic to add code below _PB_EndFunctions label
    !macro _PB_EndFunctions nul
    !{
       !label _PB_EndFunctions
       EnableASM
         mov eax, dword [esp+4]
         mov exit_code, eax
         call *exit_handler
       DisableASM
    !}

    !macro PB_EndFunctions nul
    !{
      !label PB_EndFunctions
      EnableASM
        mov rax, [PB_ExitCode]
        mov exit_code, rax
        sub rsp, 40
        call *exit_handler
        add rsp, 40
      DisableASM
    !}

  ; ===========================================================================
  ;  Mouse functions
  ; ===========================================================================

    Procedure mouse_event_send(flag.l, x.l, y.l, mousedata.l = 0, extra.i = 0)
      input.INPUT
      input\type = #INPUT_MOUSE
      input\mi\dx = x
      input\mi\dy = y
      input\mi\mouseData = mousedata
      input\mi\dwFlags = flag
      input\mi\dwExtraInfo = extra
      SendInput_(1, @input, SizeOf(INPUT))
    EndProcedure

    ; 0 relative coords to the active window
    ; 1 absolute screen coordinates (default)
    ; 2 relative coords to the client area
    ; flag = 1: ToScreen, flag = -1: ToRelative
    Procedure CoordAdjust(mode, x.l, y.l, *x, *y, flag = 1, hwnd.i = 0)
      Repeat
        IIf(mode <> 0 And mode <> 2, Break)
        IIf(hwnd = 0, hwnd = GetForegroundWindow_())
        IIf(hwnd = 0 Or IsIconic_(hwnd), Break)

        If mode = 0 And GetWindowRect_(hwnd, @r.RECT)
          x + (r\left * flag) : y + (r\top * flag)
        ElseIf ClientToScreen_(hwnd, @p.POINT)
          x + (p\x * flag) : y + (p\y * flag)
        EndIf
      Until #True
      PokeL(*x, x)
      PokeL(*y, y)
    EndProcedure

    Macro CoordAbs(coord, wh)
      (((65535 * coord) / (wh - 1)) + 1)
    EndMacro

    Macro CoordJoin(x, y)
      ((y) << 16 | (x))
    EndMacro

    Macro CoordSplit(coord, cx, cy)
      cx = coord & $FFFF
      cy = coord >> 16
    EndMacro

    ; min speed = 32
    Macro MouseMoveIncrementalCount(n1, n2)
      If n1 < n2
        delta = (n2 - n1) / speed
        IIf(delta = 0 Or delta < 32, delta = 32)
        IIf(n1 + delta > n2, n1 = n2, n1 + delta)
      ElseIf n1 > n2
        delta = (n1 - n2) / speed
        IIf(delta = 0 Or delta < 32, delta = 32)
        IIf(n1 - delta < n2, n1 = n2, n1 - delta)
      EndIf
    EndMacro

    Macro MouseMoveIncremental(x1, y1, x2, y2, speed)
      If speed = 0
        mouse_event_send(#MOUSEEVENTF_MOVE | #MOUSEEVENTF_ABSOLUTE, x2, y2)
      Else
        While x1 <> x2 Or y1 <> y2
          MouseMoveIncrementalCount(x1, x2)
          MouseMoveIncrementalCount(y1, y2)
          mouse_event_send(#MOUSEEVENTF_MOVE | #MOUSEEVENTF_ABSOLUTE, x1, y1)
          Delay(Opt("MouseDelay"))
        Wend
      EndIf
    EndMacro

    ; AutoIt MouseMove return None, but here return last coord to use in MouseClick
    Procedure.l MouseMove(x.l = #PB_Ignore, y.l = #PB_Ignore, speed.l = 10)
      FunctionInit()
      IIf(speed < 0, speed = 0)
      IIf(speed > 100, speed = 100)
      CoordAdjust(Opt("MouseCoordMode"), x, y, @xx.l, @yy.l)

      scr_w = GetSystemMetrics_(#SM_CXSCREEN)
      scr_h = GetSystemMetrics_(#SM_CYSCREEN)
      GetCursorPos_(@p.POINT)
      Define old_x.l = CoordAbs(p\x, scr_w), old_y.l = CoordAbs(p\y, scr_h)
      Define new_x.l = CoordAbs(xx, scr_w), new_y.l = CoordAbs(yy, scr_h)

      IIf(x = #PB_Ignore, new_x = old_x)
      IIf(y = #PB_Ignore, new_y = old_y)
      MouseMoveIncremental(old_x, old_y, new_x, new_y, speed)
      ProcedureReturn CoordJoin(new_x, new_y)
    EndProcedure

    Procedure.l MouseGetPos(*x, *y = 0)
      FunctionInit()
      GetCursorPos_(@p.POINT)
      CoordAdjust(Opt("MouseCoordMode"), p\x, p\y, @x, @y, -1)
      IIf(*y <> 0, PokeL(*y, y))
      IIf(*x = 0, ProcedureReturn x)
      IIf(*x = 1, ProcedureReturn y)
      PokeL(*x, x)
    EndProcedure

    ; flag = 0, MOUSEEVENTF_XXXX
    ; flag = 1, WM_XXXX
    Procedure.l MouseButtonToMsg(button.s, *msg_down, *msg_up, *wparam = 0, flag = 0)
      Select LCase(button)
      Case "primary", "main"
        IIf(GetSystemMetrics_(#SM_SWAPBUTTON), button = "right", button = "left")
      Case "secondary", "menu"
        IIf(GetSystemMetrics_(#SM_SWAPBUTTON), button = "left", button = "right")
      EndSelect

      Select LCase(button + Str(flag))
      Case "0", "left0"
        Define.l msg_down = #MOUSEEVENTF_LEFTDOWN, msg_up = #MOUSEEVENTF_LEFTUP
      Case "middle0"
        Define.l msg_down = #MOUSEEVENTF_MIDDLEDOWN, msg_up = #MOUSEEVENTF_MIDDLEUP
      Case "right0"
        Define.l msg_down = #MOUSEEVENTF_RIGHTDOWN, msg_up = #MOUSEEVENTF_RIGHTUP
      Case "1", "left1"
        Define.l msg_down = #WM_LBUTTONDOWN, msg_up = #WM_LBUTTONUP, wparam = #MK_LBUTTON
      Case "middle1"
        Define.l msg_down = #WM_MBUTTONDOWN, msg_up = #WM_MBUTTONUP, wparam = #MK_MBUTTON
      Case "right1"
        Define.l msg_down = #WM_RBUTTONDOWN, msg_up = #WM_RBUTTONUP, wparam = #MK_RBUTTON
      Default
        ProcedureReturn 0
      EndSelect

      PokeL(*msg_down, msg_down)
      PokeL(*msg_up, msg_up)
      IIf(*wparam <> 0, PokeL(*wparam, wparam))
      ProcedureReturn 1
    EndProcedure

    Procedure.l MouseClick(button.s, x.l = #PB_Ignore, y.l = #PB_Ignore, clicks.l = 1, speed.l = 10)
      FunctionInit()
      IIf(MouseButtonToMsg(button, @msg_down.l, @msg_up.l) = 0, ProcedureReturn 0)
      coord.l = MouseMove(x, y, speed)
      CoordSplit(coord, cx.u, cy.u)

      For i = 1 To clicks
        mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_down, cx, cy)
        Delay(Opt("MouseClickDownDelay"))
        mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_up, cx, cy)
        Delay(Opt("MouseClickDelay"))
      Next
      ProcedureReturn 1
    EndProcedure

    Procedure.l MouseDown(button.s)
      FunctionInit()
      IIf(MouseButtonToMsg(button, @msg_down.l, @msg_up.l) = 0, ProcedureReturn 0)
      coord.l = MouseMove(#PB_Ignore, #PB_Ignore, 0)
      CoordSplit(coord, cx.u, cy.u)

      mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_down, cx, cy)
      Delay(Opt("MouseClickDownDelay"))
      ProcedureReturn 1
    EndProcedure

    Procedure.l MouseUp(button.s)
      FunctionInit()
      IIf(MouseButtonToMsg(button, @msg_down.l, @msg_up.l) = 0, ProcedureReturn 0)
      coord.l = MouseMove(#PB_Ignore, #PB_Ignore, 0)
      CoordSplit(coord, cx.u, cy.u)

      mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_up, cx, cy)
      Delay(Opt("MouseClickDelay"))
      ProcedureReturn 1
    EndProcedure

    Procedure.l MouseClickDrag(button.s, x1.l, y1.l, x2.l, y2.l, speed.l = 10)
      FunctionInit()
      IIf(MouseButtonToMsg(button, @msg_down.l, @msg_up.l) = 0, ProcedureReturn 0)

      If x2 = #PB_Ignore Or y2 = #PB_Ignore
        GetCursorPos_(@p.POINT)
        CoordAdjust(Opt("MouseCoordMode"), p\x, p\y, @x.l, @y.l, -1)
        IIf(x2 = #PB_Ignore, x2 = x)
        IIf(y2 = #PB_Ignore, y2 = y)
      EndIf

      coord.l = MouseMove(x1, y1, speed)
      CoordSplit(coord, cx.u, cy.u)
      mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_down, cx, cy)
      Delay(Opt("MouseClickDragDelay"))

      coord.l = MouseMove(x2, y2, speed)
      CoordSplit(coord, cx.u, cy.u)
      mouse_event_send(#MOUSEEVENTF_ABSOLUTE | msg_up, cx, cy)
      Delay(Opt("MouseClickDragDelay"))
      ProcedureReturn 1
    EndProcedure

    Procedure.l MouseGetCursor()
      FunctionInit()
      DataSection
        MouseCursorIDC:
        Data.l #IDC_APPSTARTING, #IDC_ARROW, #IDC_CROSS, #IDC_HELP, #IDC_IBEAM, #IDC_ICON, #IDC_NO, #IDC_SIZE, #IDC_SIZEALL, #IDC_SIZENESW, #IDC_SIZENS, #IDC_SIZENWSE, #IDC_SIZEWE, #IDC_UPARROW, #IDC_WAIT, #IDC_HAND
      EndDataSection

      Static NewMap cursor_table.l()

      If MapSize(cursor_table()) = 0
        Restore MouseCursorIDC
        For i = 1 To 16
          Read.l id
          handle.i = LoadCursor_(0, id)
          IIf(handle, cursor_table(Str(handle)) = i)
        Next
      EndIf

      AttachThreadInput(GetForegroundWindow_())
      cursor.i = GetCursor_()
      DetachThreadInput()
      ProcedureReturn cursor_table(Str(cursor))
    EndProcedure

    Procedure.l MouseWheel_(direction.s, clicks.l = 1)
      FunctionInit()
      Select LCase(direction)
        Case "up":   delta.l = #WHEEL_DELTA
        Case "down": delta.l = -#WHEEL_DELTA
        Default:     ProcedureReturn 0
      EndSelect

      coord.l = MouseMove(#PB_Ignore, #PB_Ignore, 0)
      CoordSplit(coord, cx.u, cy.u)

      For i = 1 To clicks
        mouse_event_send(#MOUSEEVENTF_ABSOLUTE | #MOUSEEVENTF_WHEEL, cx, cy, delta)
        Delay(Opt("MouseClickDelay"))
      Next
      ProcedureReturn 1
    EndProcedure

  ; ===========================================================================
  ;  Network functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Obj/COM functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Process functions
  ; ===========================================================================

    Procedure.i ProcessExists(process.s)
      FunctionInit()
      IIf(StringIsDigit(process), pid.i = Val(process), pid.i = 0)
      snapshot.i = CreateToolhelp32Snapshot_(#TH32CS_SNAPPROCESS, 0)
      IIf(snapshot = #INVALID_HANDLE_VALUE, ProcedureReturn 0)

      result.i = 0
      proc.PROCESSENTRY32\dwSize = SizeOf(proc)
      Process32First_(snapshot, @proc)
      Repeat
        IIf(proc\th32ProcessID = 0, Continue) ; skip [System Process]
        IIf(pid <> 0 And proc\th32ProcessID = pid, ReturnBreak(pid))
        IIf(process = PeekS(@proc\szExeFile), ReturnBreak(proc\th32ProcessID))
      Until Not Process32Next_(snapshot, @proc)
      CloseHandle_(snapshot)

      ProcedureReturn result
    EndProcedure

    Procedure.l ProcessList(List procs.ProcessInfo(), name.s = "")
      FunctionInit()
      ClearList(procs())
      snapshot.i = CreateToolhelp32Snapshot_(#TH32CS_SNAPPROCESS, 0)
      IIf(snapshot = #INVALID_HANDLE_VALUE, ProcedureReturn 0)

      proc.PROCESSENTRY32\dwSize = SizeOf(proc)
      Process32First_(snapshot, @proc)
      Repeat
        If name = ""
          AddElement(procs())
          procs()\name = PeekS(@proc\szExeFile)
          procs()\pid = proc\th32ProcessID
        ElseIf PeekS(@proc\szExeFile) = name
          AddElement(procs())
          procs()\name = name
          procs()\pid = proc\th32ProcessID
        EndIf
      Until Not Process32Next_(snapshot, @proc)
      CloseHandle_(snapshot)

      ProcedureReturn ListSize(procs())
    EndProcedure

    Procedure.l ProcessClose(process.s)
      FunctionInit()
      pid.i = ProcessExists(process)
      IIf(pid = 0, ErrorReturn(4, 0))

      SetPrivilege("SeDebugPrivilege")
      proc.i = OpenProcess_(#PROCESS_TERMINATE, 0, pid)
      IIf(proc = 0, ErrorReturn(1, 0))
      result.l = TerminateProcess_(proc, 0)
      CloseHandle_(proc)

      IIf(result = 0, ErrorReturn(3, 0), ProcedureReturn 1)
    EndProcedure
    
    CompilerIf Defined(PROCESS_MEMORY_COUNTERS, #PB_Structure) = 0
    Structure PROCESS_MEMORY_COUNTERS
      cb.l
      PageFaultCount.l
      PeakWorkingSetSize.i
      WorkingSetSize.i
      QuotaPeakPagedPoolUsage.i
      QuotaPagedPoolUsage.i
      QuotaPeakNonPagedPoolUsage.i
      QuotaNonPagedPoolUsage.i
      PagefileUsage.i
      PeakPagefileUsage.i
    EndStructure
    CompilerEndIf

    Structure IO_COUNTERS
      ReadOperationCount.q
      WriteOperationCount.q
      OtherOperationCount.q
      ReadTransferCount.q
      WriteTransferCount.q
      OtherTransferCount.q
    EndStructure

    Procedure.s ProcessGetStats(process.s = "", type.l = 0)
      FunctionInit()
      Define result.s = "", proc.i = 0, lib.i = 0, pid.i
      IIf(process = "", pid = GetCurrentProcessId_(), pid = ProcessExists(process))

      Repeat
        IIf(pid = 0, Break)
        SetPrivilege("SeDebugPrivilege")

        proc = OpenProcess_(#PROCESS_QUERY_INFORMATION, 0, pid)
        IIf(proc = 0, Break)

        If type = 1
          GetProcessIoCounters_.GetProcessIoCounters = GetProcAddress_(GetModuleHandle_("kernel32"), "GetProcessIoCounters")
          IIf(GetProcessIoCounters_ = 0, Break)
          IIf(GetProcessIoCounters_(proc, @ic.IO_COUNTERS) = 0, Break)
          result = Str(ic\ReadOperationCount) + "," + Str(ic\WriteOperationCount) + "," + Str(ic\OtherOperationCount) + "," + Str(ic\ReadTransferCount) + "," + Str(ic\WriteTransferCount) + "," + Str(ic\OtherTransferCount)
        Else
          GetProcessMemoryInfo_.GetProcessMemoryInfo = GetProcAddress_(GetModuleHandle_("kernel32"), "GetProcessMemoryInfo")
          If GetProcessMemoryInfo_ = 0
            lib.i = LoadLibrary_("psapi.dll")
            GetProcessMemoryInfo_ = GetProcAddress_(lib, "GetProcessMemoryInfo")
          EndIf

          IIf(GetProcessMemoryInfo_ = 0, Break)
          IIf(GetProcessMemoryInfo_(proc, @pmc.PROCESS_MEMORY_COUNTERS, SizeOf(pmc)) = 0, Break)
          result = Str(pmc\WorkingSetSize) + "," + Str(pmc\PeakWorkingSetSize)
        EndIf
      Until #True
      IIf(lib, FreeLibrary_(lib))
      IIf(proc, CloseHandle_(proc))
      IIf(result = "", ErrorReturn(1, ""), ProcedureReturn result)
    EndProcedure

    Procedure.l ProcessSetPriority(process.s, priority.l)
      FunctionInit()
      Select priority
        Case 0: priority = #IDLE_PRIORITY_CLASS
        Case 1: priority = #BELOW_NORMAL_PRIORITY_CLASS
        Case 2: priority = #NORMAL_PRIORITY_CLASS
        Case 3: priority = #ABOVE_NORMAL_PRIORITY_CLASS
        Case 4: priority = #HIGH_PRIORITY_CLASS
        Case 5: priority = #REALTIME_PRIORITY_CLASS
        Default: ErrorReturn(2, 0)
      EndSelect

      pid.i = ProcessExists(process)
      IIf(pid = 0, ErrorReturn(1, 0))

      SetPrivilege("SeDebugPrivilege")
      proc = OpenProcess_(#PROCESS_SET_INFORMATION , 0, pid)
      IIf(proc = 0, ErrorReturn(1, 0))

      result.l = SetPriorityClass_(proc, priority)
      CloseHandle_(proc)
      IIf(result = 0, ErrorReturn(1, 0), ProcedureReturn 1)
    EndProcedure

    Procedure.i ProcessWait(process.s, timeout.l = 0)
      FunctionInit()
      timeout * 1000
      Define pid.i = 0, begin.i = ElapsedMilliseconds()
      Repeat
        IIf(timeout <> 0 And ElapsedMilliseconds() - begin > timeout, Break)
        pid = ProcessExists(process)
        IIf(pid <> 0, Break)
        Delay(250)
      ForEver
      ProcedureReturn pid
    EndProcedure

    Procedure.l ProcessWaitClose(process.s, timeout.l = 0)
      FunctionInit()
      timeout * 1000
      IIf(timeout = 0, timeout = #INFINITE)
      pid.i = ProcessExists(process)
      If pid = 0
        Extended($CCCCCCCC)
        ErrorReturn(1, 0)
      EndIf

      SetPrivilege("SeDebugPrivilege")
      proc = OpenProcess_(#PROCESS_QUERY_INFORMATION | #SYNCHRONIZE, 0, pid)
      IIf(proc = 0, ErrorReturn(1, 0))

      signal.l = WaitForSingleObject_(proc, timeout)
      GetExitCodeProcess_(proc, @code)
      CloseHandle_(proc)
      IIf(signal = #WAIT_TIMEOUT, ProcedureReturn 0)
      Extended(code)
      ProcedureReturn 1
    EndProcedure

    Procedure.i Run(program.s, workingdir.s = "", show_flag.l = #SW_NORMAL)
      Protected *dir=0
      FunctionInit()
      ;IIf(workingdir = "", workingdir = #Null$)
      
      If workingdir
        *dir=@workingdir
      EndIf
      
      s.STARTUPINFO
      p.PROCESS_INFORMATION
      s\dwFlags = #STARTF_USESHOWWINDOW
      s\wShowWindow = show_flag
      If CreateProcess_(0, program, 0, 0, 0, #NORMAL_PRIORITY_CLASS, 0, *dir, @s, @p)
        CloseHandle_(p\hProcess)
        CloseHandle_(p\hThread)
        ProcedureReturn p\dwProcessId
      EndIf
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.i RunWait(program.s, workingdir.s = "", showflag.l = #SW_NORMAL)
      Protected *dir=0
      FunctionInit()
      ;IIf(workingdir = "", workingdir = #Null$)
      If workingdir
        *dir=@workingdir
      EndIf
      s.STARTUPINFO
      p.PROCESS_INFORMATION
      s\dwFlags = #STARTF_USESHOWWINDOW
      s\wShowWindow = showflag
      If CreateProcess_(0, program, 0, 0, 0, #NORMAL_PRIORITY_CLASS, 0, *dir, @s, @p)
        WaitForSingleObject_(p\hProcess, #INFINITE)
        GetExitCodeProcess_(p\hProcess, @code)
        CloseHandle_(p\hProcess)
        CloseHandle_(p\hThread)
        ProcedureReturn code
      EndIf
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.i ShellExecute(filename.s, parameters.s = "", workingdir.s = "", verb.s = "", showflag.l = #SW_NORMAL)
      info.SHELLEXECUTEINFO\cbSize = SizeOf(info)
      info\lpFile = @filename
      IIf(parameters <> "", info\lpParameters = @parameters)
      IIf(workingdir <> "", info\lpDirectory = @workingdir)
      IIf(verb <> "", info\lpVerb = @verb)
      info\nShow = showflag
      info\fMask = #SEE_MASK_NOCLOSEPROCESS

      If ShellExecuteEx_(@info)
        If info\hProcess
          pid.i = GetProcessId_(info\hProcess)
          CloseHandle_(info\hProcess)
          ProcedureReturn pid
        Else
          ProcedureReturn -1
        EndIf
      EndIf
      ErrorReturn(1, 0)
    EndProcedure

    Procedure.i ShellExecuteWait(filename.s, parameters.s = "", workingdir.s = "", verb.s = "", showflag.l = #SW_NORMAL)
      info.SHELLEXECUTEINFO\cbSize = SizeOf(info)
      info\lpFile = @filename
      IIf(parameters <> "", info\lpParameters = @parameters)
      IIf(workingdir <> "", info\lpDirectory = @workingdir)
      IIf(verb <> "", info\lpVerb = @verb)
      info\nShow = showflag
      info\fMask = #SEE_MASK_NOCLOSEPROCESS

      If ShellExecuteEx_(@info)
        If info\hProcess
          WaitForSingleObject_(info\hProcess, #INFINITE)
          GetExitCodeProcess_(info\hProcess, @code)
          CloseHandle_(info\hProcess)
          ProcedureReturn code
        Else
          ProcedureReturn -1
        EndIf
      EndIf
      ErrorReturn(1, 0)
    EndProcedure

  ; ===========================================================================
  ;  Registry functions
  ; ===========================================================================

    Procedure.s RegRoot(keyname.s, *handle, *flag)
      root.s = UCase(StringField(keyname, 1, "\"))
      If Right(root, 2) = "64"
        PokeL(*flag, #KEY_WOW64_64KEY)
        root = StringTrimRight(root, 2)
      EndIf

      Select root
      Case "HKEY_LOCAL_MACHINE", "HKLM"
        PokeI(*handle, #HKEY_LOCAL_MACHINE)
      Case "HKEY_USERS", "HKU"
        PokeI(*handle, #HKEY_USERS)
      Case "HKEY_CURRENT_USER", "HKCU"
        PokeI(*handle, #HKEY_CURRENT_USER)
      Case "HKEY_CLASSES_ROOT", "HKCR"
        PokeI(*handle, #HKEY_CLASSES_ROOT)
      Case "HKEY_CURRENT_CONFIG", "HKCC"
        PokeI(*handle, #HKEY_CURRENT_CONFIG)
      Default
        PokeI(*handle, 0)
        ProcedureReturn ""
      EndSelect
      ProcedureReturn Mid(keyname, Len(root) + 2)
    EndProcedure

    ; tested under windows 8, max key name: 256, max value name: 16383
    Procedure.s RegEnumKey(keyname.s, instance.l)
      FunctionInit()
      Define buffer.s{512}, handle.i = 0 ;  use fixed string in small buffer is faster than memory alloc
      Repeat
        keyname = RegRoot(keyname, @root_key.i, @flag.l)
        IIf(root_key = 0, ErrorBreak(2))

        IIf(RegOpenKeyEx_(root_key, keyname, i, #KEY_READ | flag, @handle) <> #ERROR_SUCCESS, ErrorBreak(1))
        IIf(instance <= 0 Or RegEnumKey_(handle, instance - 1, @buffer, LengthOf(buffer)) <> #ERROR_SUCCESS, ErrorBreak(-1))
      Until #True
      IIf(handle, RegCloseKey_(handle))
      ProcedureReturn buffer
    EndProcedure

    Procedure.s RegEnumVal(keyname.s, instance.l)
      FunctionInit()
      Define *buffer = 0, handle.i = 0, length.l = 32766, result.s = ""
      Repeat
        *buffer = AllocateMemory(length * SizeOf(Character), #PB_Memory_NoClear)
        IIf(*buffer = 0, ErrorBreak(2))

        keyname = RegRoot(keyname, @root_key.i, @flag.l)
        IIf(root_key = 0, ErrorBreak(2))

        IIf(RegOpenKeyEx_(root_key, keyname, i, #KEY_READ | flag, @handle) <> #ERROR_SUCCESS, ErrorBreak(1))
        IIf(instance <= 0 Or RegEnumValue_(handle, instance - 1, *buffer, @length, 0, @type.l, 0, 0) <> #ERROR_SUCCESS, ErrorBreak(-1))
        result = PeekS(*buffer)
        Extended(type)
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      IIf(handle, RegCloseKey_(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.s RegRead(keyname.s, valuename.s)
      FunctionInit()
      Define *buffer = 0, handle.i = 0, result.s = "", size.l = 0
      Repeat
        keyname = RegRoot(keyname, @root_key.i, @flag.l)
        IIf(root_key = 0, ErrorBreak(2))

        IIf(RegOpenKeyEx_(root_key, keyname, i, #KEY_READ | flag, @handle) <> #ERROR_SUCCESS, ErrorBreak(1))
        IIf(RegQueryValueEx_(handle, valuename, 0, @type.l, 0, @size) <> #ERROR_SUCCESS, ErrorBreak(-1))

        *buffer = AllocateMemory(size + 2)
        IIf(*buffer = 0, ErrorBreak(-1))

        IIf(RegQueryValueEx_(handle, valuename, 0, @type.l, *buffer, @size) <> #ERROR_SUCCESS, ErrorBreak(-1))
        Extended(type)

        Select type
        Case #REG_SZ, #REG_EXPAND_SZ
          result = PeekS(*buffer)

        Case #REG_MULTI_SZ
          *ptr = *buffer
          For i = 0 To size - 1 - (2 * SizeOf(Character))
            IIf(PeekC(*ptr) = 0, PokeC(*ptr, $0A))
            *ptr + SizeOf(Character)
          Next
          result = PeekS(*buffer)

        Case #REG_DWORD
          result = StrU(PeekL(*buffer), #PB_Long)

        Case #REG_QWORD
          result = StrU(PeekQ(*buffer), #PB_Quad)

        Case #REG_DWORD_BIG_ENDIAN
          tmp.l = PeekL(*buffer)
          EndianSwitch(tmp)
          result = StrU(tmp)

        Case #REG_BINARY
          result = MemoryToHex(*buffer, size)

        Default
          result = ""
          ErrorBreak(-2)
        EndSelect
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      IIf(handle, RegCloseKey_(handle))
      ProcedureReturn result
    EndProcedure

    Procedure.l RegWrite(keyname.s, valuename.s = Chr(1), type.l = 0, value.s = "")
      FunctionInit()
      Define handle.i = 0
      Repeat
        keyname = RegRoot(keyname, @root_key.i, @flag.l)
        IIf(root_key = 0, ErrorBreak(2))

        IIf(RegCreateKeyEx_(root_key, keyname, 0, 0, 0, #KEY_WRITE, 0, @handle, 0) <> #ERROR_SUCCESS, ErrorBreak(1))
        If valuename <> Chr(1)
          Select type
          Case #REG_SZ, #REG_EXPAND_SZ
            IIf(RegSetValueEx_(handle, valuename, 0, type, @value, (Len(value) + 1) * SizeOf(Character)) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Case #REG_MULTI_SZ
            copy.s = value + #LF$
            size.l = (Len(copy) + 1) * SizeOf(Character)
            *ptr = @copy
            For i = 0 To size - 1
              IIf(PeekC(*ptr) = $0A, PokeC(*ptr, 0))
              *ptr + SizeOf(Character)
            Next
            IIf(RegSetValueEx_(handle, valuename, 0, type, @copy, size) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Case #REG_DWORD
            buffer_d.l = Val(value)
            IIf(RegSetValueEx_(handle, valuename, 0, type, @buffer_d, SizeOf(LONG)) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Case #REG_QWORD
            buffer_q.q = Val(value)
            IIf(RegSetValueEx_(handle, valuename, 0, type, @buffer_q, SizeOf(QUAD)) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Case #REG_DWORD_BIG_ENDIAN
            buffer_d.l = Val(value)
            EndianSwitch(buffer_d)
            IIf(RegSetValueEx_(handle, valuename, 0, type, @buffer_d, SizeOf(LONG)) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Case #REG_BINARY
            size.l = HexToMemory_InPlace(value, @value)
            IIf(RegSetValueEx_(handle, valuename, 0, type, @value, size) <> #ERROR_SUCCESS, ErrorBreak(-1))

          Default
            ErrorBreak(-2)
          EndSelect
        EndIf
      Until #True
      IIf(handle, RegCloseKey_(handle))
      IIf(_Error(), ProcedureReturn 0, ProcedureReturn 1)
    EndProcedure

    Procedure.l RegDelete(keyname.s, valuename.s = Chr(1))
      FunctionInit()
      Define handle.i = 0
      Repeat
        If valuename <> Chr(1)
          keyname = RegRoot(keyname, @root_key.i, @flag.l)
          IIf(root_key = 0, ErrorBreak(2))

          IIf(RegOpenKeyEx_(root_key, keyname, i, #KEY_WRITE | flag, @handle) <> #ERROR_SUCCESS, ErrorBreak(1))
          IIf(RegDeleteValue_(handle, valuename) <> #ERROR_SUCCESS, ErrorBreak(-1))
        Else
          keyname = RegRoot(keyname, @root_key.i, @flag.l)
          IIf(root_key = 0, ErrorBreak(2))

          delkey.s = StringField(keyname, CountString(keyname, "\") + 1, "\")
          keyname = StringTrimRight(keyname, Len(delkey) + 1)

          IIf(RegOpenKeyEx_(root_key, keyname, i, #KEY_WRITE | flag, @handle) <> #ERROR_SUCCESS, ErrorBreak(1))
          IIf(RegDeleteKey_(handle, delkey) <> #ERROR_SUCCESS, ErrorBreak(-2))
        EndIf
      Until #True
      IIf(handle, RegCloseKey_(handle))
      IIf(_Error(), ProcedureReturn 0, ProcedureReturn 1)
    EndProcedure

  ; ===========================================================================
  ;  String functions
  ; ===========================================================================

    CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
      ImportC "msvcrt.lib"
        CompilerIf #PB_Compiler_Unicode
          snwprintf(*ptr, count.i, str.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0) As "__snwprintf"
        CompilerElse
          snwprintf(*ptr, count.i, str.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0) As "__snprintf"
        CompilerEndIf
      EndImport
    CompilerElse
      ImportC "msvcrt.lib"
        CompilerIf #PB_Compiler_Unicode
          snwprintf(*ptr, count.i, str.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0) As "_snwprintf"
        CompilerElse
          snwprintf(*ptr, count.i, str.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0) As "_snprintf"
        CompilerEndIf
      EndImport
    CompilerEndIf

    Procedure.s StringReplaceFrom(str.s, from.l, replace.s)
      FunctionInit()
      from - 1
      IIf(from < 0, ProcedureReturn "")
      IIf(Len(str) < Len(replace) + from, ProcedureReturn "")
      PokeS(@str + from * SizeOf(Character), replace, -1, #PB_String_NoZero)
      ProcedureReturn str
    EndProcedure

    Procedure.s StringReplace(str.s, search.s, replace.s, occur.l = 0, casesense.l = #False)
      FunctionInit()
      IIf(replace = "", ErrorReturn(1, ""))
      IIf(search = "", ProcedureReturn "")

      char.c = StringReplace_Inplace(str, search, occur, casesense, @count.l, @search_len.l)
      If char
        Extended(count)
        ProcedureReturn ReplaceString(str, search, replace)
      ElseIf occur < 0
        ErrorReturn(1, "")
      Else
        IIf(occur = 0, occur = -1)
        IIf(casesense, mode = #PB_String_CaseSensitive, mode = #PB_String_NoCase)
        ProcedureReturn ReplaceString(str, search, replace, mode, 1, occur)
      EndIf
    EndProcedure

    Procedure.s StringFormat(format.s, v01.i = 0, v02.i = 0, v03.i = 0, v04.i = 0, v05.i = 0, v06.i = 0, v07.i = 0, v08.i = 0, v09.i = 0, v10.i = 0, v11.i = 0, v12.i = 0, v13.i = 0, v14.i = 0, v15.i = 0, v16.i = 0, v17.i = 0, v18.i = 0, v19.i = 0, v20.i = 0, v21.i = 0, v22.i = 0, v23.i = 0, v24.i = 0, v25.i = 0, v26.i = 0, v27.i = 0, v28.i = 0, v29.i = 0, v30.i = 0, v31.i = 0, v32.i = 0)
      *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
      IIf(*buffer = 0, ProcedureReturn "")

      snwprintf(*buffer, #TextBufferLength, UnescapeString(format), v01, v02, v03, v04, v05, v06, v07, v08, v09, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32)
      result.s = PeekS(*buffer)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.l StringSplit(Array arr.s(1), string.s, delimiters.s, flag = 0)
      If delimiters = ""
        length.l = Len(string)
        ReDim arr(length)

        If flag & #STR_NOCOUNT
          index_base.l = -1
        Else
          index_base.l = 0
          arr(0) = Str(length)
        EndIf

        For i = 1 To length
          arr(index_base + i) = Mid(string, i, 1)
        Next
        ProcedureReturn length
      Else
        If flag & #STR_ENTIRESPLIT
          length.l = CountString(string, delimiters) + 1
        Else
          length.l = StringSplit_InPlace(string, delimiters)
        EndIf
        ReDim arr(length)

        If flag & #STR_NOCOUNT
          index_base.l = -1
        Else
          index_base.l = 0
          arr(0) = Str(length)
        EndIf

        For i = 1 To length
          arr(index_base + i) = StringField(string, i, delimiters)
        Next
        ProcedureReturn length
      EndIf
    EndProcedure

    Procedure.s StringStripWS(str.s, flag.l)
      StringStripWS_InPlace(str, flag)
      ProcedureReturn str
    EndProcedure

    Procedure.s StringRepeat(string.s, count.l)
      IIf(count <= 0 Or string = "", ProcedureReturn "")

      length.l = Len(string)
      *buffer = AllocateMemory((length * count + 1) * SizeOf(Character), #PB_Memory_NoClear)
      IIf(*buffer = 0, ProcedureReturn "")

      StringRepeat_InPlace(string, length, *buffer, count)
      result.s = PeekS(*buffer)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.s StringInsert(string.s, insert.s, position.l = 0)
      len.l = Len(string)
      IIf(position >= 0 And position <= len, ProcedureReturn InsertString(string, insert, position + 1))
      IIf(position < 0 And position >= -len, ProcedureReturn InsertString(string, insert, len + position + 1))
      ProcedureReturn string
    EndProcedure

    Procedure.l StringToASCIIArray(Array char.u(1), string.s, start.l = 0, last.l = -1, encoding.l = 0)
      ; start and last is 0-based position
      IIf(start < 0, start = 0)
      IIf(last < 0, string = Mid(string, start + 1), string = Mid(string, start + 1, last - start + 1))

      Select encoding
      Case 1, #PB_Ascii
        encoding = #PB_Ascii
        char_size.l = 1
      Case 2, #PB_UTF8
        encoding = #PB_UTF8
        char_size = 1
      Default ; 0, #PB_Unicode
        encoding = #PB_Unicode
        char_size = 2
      EndSelect

      size.l = StringByteLength(string, encoding)
      IIf(size = 0, ProcedureReturn 0)
      *buffer = AllocateMemory(size, #PB_Memory_NoClear)
      IIf(*buffer = 0, ProcedureReturn 0)

      PokeS(*buffer, string, -1, encoding | #PB_String_NoZero)

      length.l = size / char_size
      ReDim char(length)

      *ptr = *buffer
      index.l = 0
      While *ptr < *buffer + size
        IIf(char_size = 1, char(index) = PeekA(*ptr), char(index) = PeekU(*ptr))
        *ptr + char_size
        index + 1
      Wend
      FreeMemory(*buffer)
      ProcedureReturn length
    EndProcedure

    Procedure.s StringFromASCIIArray(Array char.u(1), start = 0, last = -1, encoding = 0)
      length.l = ArraySize(char())
      IIf(start < 0, start = 0)
      IIf(last < 0 Or last >= length, last = length - 1)
      IIf(start >= length Or last < start, ProcedureReturn "")

      Select encoding
      Case 1, #PB_Ascii
        encoding = #PB_Ascii
      Case 2, #PB_UTF8
        encoding = #PB_UTF8
      Default ; 0, #PB_Unicode
        ProcedureReturn PeekS(@char() + start * 2, last - start + 1, #PB_Unicode)
      EndSelect

      *buffer = AllocateMemory(last - start + 2, #PB_Memory_NoClear)
      IIf(*buffer = 0, ProcedureReturn "")
      *ptr = *buffer

      For i = start To last
        PokeA(*ptr, char(i))
        *ptr + 1
      Next
      PokeA(*ptr, 0)

      result.s = PeekS(*buffer, -1, encoding)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.s MemoryToHex(*src, length)
      *buffer = AllocateMemory(length * 2 * SizeOf(Character) + 2)
      IIf(*buffer = 0, ProcedureReturn "")

      MemoryToHex_Inplace(*src, length, *buffer)
      result.s = PeekS(*buffer)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.s StringToHex(string.s, format.l = #PB_Ascii)
      length.l = StringByteLength(string, format)
      IIf(length = 0, ProcedureReturn "")
      *binary = AllocateMemory(length, #PB_Memory_NoClear)
      IIf(*binary = 0, ProcedureReturn "")

      PokeS(*binary, string, -1, format | #PB_String_NoZero)
      result.s = MemoryToHex(*binary, length)
      FreeMemory(*binary)
      ProcedureReturn result
    EndProcedure

    Procedure.s HexToString(hex.s, format.l = #PB_Ascii)
      *buffer = AllocateMemory(Len(hex) / 2 * SizeOf(Character) + 2)
      IIf(*buffer = 0, ProcedureReturn "")

      HexToMemory_InPlace(hex, *buffer)
      result.s = PeekS(*buffer, -1, format)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.i HexToMemory(hex.s, *buffer = 0)
      FunctionInit()
      length.l = Len(hex)
      IIf(length = 0 Or length & 1 Or Not StringIsXDigit(hex), ProcedureReturn 0)

      If *buffer = 0
        *buffer = AllocateMemory(length / 2)
        IIf(*buffer = 0, ProcedureReturn 0)
        HexToMemory_InPlace(hex, *buffer)
        ProcedureReturn *buffer
      Else
        ProcedureReturn HexToMemory_InPlace(hex, *buffer)
      EndIf
    EndProcedure

    Procedure.l MemoryStringFormat_CheckBOM(*ptr, size.l)
      IIf(size >= 2 And PeekU(*ptr) = $FEFF, ProcedureReturn #ENCODING_UTF16_LE_BOM)
      IIf(size >= 2 And PeekU(*ptr) = $FFFE, ProcedureReturn #ENCODING_UTF16_BE_BOM)
      IIf(size >= 3 And PeekU(*ptr) = $BBEF And PeekA(*ptr + 2) = $BF, ProcedureReturn #ENCODING_UTF8_BOM)
      ProcedureReturn #ENCODING_UNKNOWN
    EndProcedure

    Procedure.l MemoryStringFormat_CheckUTF8(*ptr, size.l)
      Define ascii_range.l = #True, pos.l = 0, more.l
      While pos < size
        ch.a = PeekA(*ptr + pos)
        pos + 1
        Select ch
          Case   1 To 127: more = 0
          Case 194 To 223: more = 1
          Case 224 To 239: more = 2
          Case 240 To 244: more = 3
          Default: ProcedureReturn #ENCODING_UNKNOWN ; 0 or others
        EndSelect

        While more And pos < size
          ascii_range = #False
          ch = PeekA(*ptr + pos)
          pos + 1
          IIf(ch < 128 Or ch > 191, ProcedureReturn #ENCODING_UNKNOWN)
          more - 1
        Wend
      Wend
      IIf(ascii_range, ProcedureReturn #ENCODING_ASCII, ProcedureReturn #ENCODING_UTF8_NOBOM)
    EndProcedure

    Procedure.l MemoryStringFormat_CheckUTF16CRLF(*ptr, size.l)
      IIf(size < 2, ProcedureReturn #ENCODING_UNKNOWN)
      size - 1

      Define le.l = 0 , be.l = 0, pos.l = 0
      While pos < size
        ch.u = PeekU(*ptr + pos)
        pos + 2
        Select ch
          Case $0D00, $0A00: be + 1
          Case $000D, $000A: le + 1
        EndSelect
        IIf(le And be, ProcedureReturn #ENCODING_UNKNOWN)
      Wend

      IIf(le, ProcedureReturn #ENCODING_UTF16_LE_NOBOM)
      IIf(be, ProcedureReturn #ENCODING_UTF16_BE_NOBOM)
      ProcedureReturn #ENCODING_UNKNOWN
    EndProcedure

    Procedure.l MemoryStringFormat_CheckUTF16ASCII(*ptr, size.l)
      size - 1
      Define even_nulls.l = 0, odd_nulls.l = 0, pos.l = 0
      While pos < size
        IIf(PeekA(*ptr + pos) = 0, even_nulls + 1)
        IIf(PeekA(*ptr + pos + 1) = 0, odd_nulls + 1)
        pos + 2
      Wend

      even_rate.d = even_nulls * 2 / size * 100
      odd_rate.d = odd_nulls * 2 / size * 100
      IIf((odd_rate > 70 And even_rate < 10) Or (odd_rate > 50 And even_rate < 2) Or (odd_rate > 10 And even_rate = 0), ProcedureReturn #ENCODING_UTF16_LE_NOBOM)
      IIf((even_rate > 70 And odd_rate < 10) Or (even_rate > 50 And odd_rate < 2) Or (even_rate > 10 And odd_rate = 0), ProcedureReturn #ENCODING_UTF16_BE_NOBOM)
      ProcedureReturn #ENCODING_UNKNOWN
    EndProcedure

    Procedure.l MemoryStringFormat_CheckNull(*ptr, size.l)
      pos.l = 0
      While pos < size
        IIf(PeekA(*ptr + pos) = 0, ProcedureReturn #True)
        pos + 1
      Wend
      ProcedureReturn #False
    EndProcedure

    Procedure.l MemoryStringFormat(*ptr, size.l)
      format.l = MemoryStringFormat_CheckBOM(*ptr, size)
      IIf(format <> #ENCODING_UNKNOWN, ProcedureReturn format)

      format = MemoryStringFormat_CheckUTF8(*ptr, size)
      IIf(format <> #ENCODING_UNKNOWN, ProcedureReturn format)

      format = MemoryStringFormat_CheckUTF16CRLF(*ptr, size)
      IIf(format <> #ENCODING_UNKNOWN, ProcedureReturn format)

      format = MemoryStringFormat_CheckUTF16ASCII(*ptr, size)
      IIf(format <> #ENCODING_UNKNOWN, ProcedureReturn format)

      IIf(MemoryStringFormat_CheckNull(*ptr, size), ProcedureReturn #ENCODING_UNKNOWN)
      ProcedureReturn #ENCODING_ANSI
    EndProcedure

    CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp

      ; Returns 1 (match) or 0 (no match)
      Procedure.l StringRegExp0(string.s, pattern.s)
        FunctionInit()
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern
          result = MatchRegularExpression(re, string)
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in array will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExp1(Array match.s(1), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          result.l = CountRegularExpressionGroups(re)
          If result = 0 ; flag = 1, it will return full match if there is no group
            ReDim match(1)
            match(0) = RegularExpressionMatchString(re)
            Extended(offset + RegularExpressionMatchPosition(re) + Len(match(0)) - 1)
            ReturnBreak(1)
          EndIf

          ReDim match(result)
          For i = 0 To result - 1
            match(i) = RegularExpressionGroup(re, i + 1)
          Next
          Extended(offset + RegularExpressionMatchPosition(re) + Len(RegularExpressionMatchString(re)) - 1)
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in list will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExpList1(List match.s(), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        ClearList(match())
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          result.l = CountRegularExpressionGroups(re)
          If result = 0 ; flag = 1, it will return full match if there is no group
            AddElement(match())
            match() = RegularExpressionMatchString(re)
            Extended(offset + RegularExpressionMatchPosition(re) + Len(match()) - 1)
            ReturnBreak(1)
          EndIf

          For i = 1 To result
            AddElement(match())
            match() = RegularExpressionGroup(re, i)
          Next
          Extended(offset + RegularExpressionMatchPosition(re) + Len(RegularExpressionMatchString(re)) - 1)
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in array will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExp2(Array match.s(1), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          groups.l = CountRegularExpressionGroups(re)
          ReDim match(groups + 1)
          match(0) = RegularExpressionMatchString(re)
          Extended(offset + RegularExpressionMatchPosition(re) + Len(match(0)) - 1)

          For i = 0 To groups - 1
            match(i + 1) = RegularExpressionGroup(re, i + 1)
          Next
          result = groups + 1
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in list will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExpList2(List match.s(), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        ClearList(match())
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          groups.l = CountRegularExpressionGroups(re)
          AddElement(match())
          match() = RegularExpressionMatchString(re)
          Extended(offset + RegularExpressionMatchPosition(re) + Len(match()) - 1)

          For i = 1 To groups
            AddElement(match())
            match() = RegularExpressionGroup(re, i)
          Next
          result = groups + 1
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in array will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExp3(Array match.s(1), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          Define index.l = 0, size.l = 10, groups.l = CountRegularExpressionGroups(re)

          If groups = 0 ; flag = 3, it will return all full match if there is no group
            count.l = ExtractRegularExpression(re, string, match())
            ReDim match(count)
            ReturnBreak(count)
          EndIf

          ReDim match(size)
          Repeat
            For i = 1 To groups
              match(index) = RegularExpressionGroup(re, i)
              index + 1

              If size - index < 1
                size * 2
                ReDim match(size)
              EndIf
            Next
          Until NextRegularExpressionMatch(re) = 0
          ReDim match(index)
          result = index
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      ; how many matches in list will be returned, return 0 if no match or error occurred
      Procedure.l StringRegExpList3(List match.s(), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        ClearList(match())
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2)) ; Error 2: Bad pattern, array is invalid
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          Define groups.l = CountRegularExpressionGroups(re)
          Repeat
            If groups = 0 ; flag = 3, it will return all full match if there is no group
              AddElement(match())
              match() = RegularExpressionMatchString(re)
            Else
              For i = 1 To groups
                AddElement(match())
                match() = RegularExpressionGroup(re, i)
              Next
            EndIf
          Until NextRegularExpressionMatch(re) = 0
          result = ListSize(match())
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      Procedure.l StringRegExp4(Array match.StringRegExpMatch(1), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2))
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          Define index.l = 0, size.l = 10, groups.l = CountRegularExpressionGroups(re)
          ReDim match(size)
          Repeat
            ReDim match(index)\group(groups + 1)
            match(index)\group(0) = RegularExpressionMatchString(re)
            For i = 1 To groups
              match(index)\group(i) = RegularExpressionGroup(re, i)
            Next
            index + 1

            If size - index < 1
              size * 2
              ReDim match(size)
            EndIf
          Until NextRegularExpressionMatch(re) = 0
          ReDim match(index)
          result = index
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      Procedure.l StringRegExpList4(List match.StringRegExpMatch(), string.s, pattern.s, offset.l = 1)
        FunctionInit()
        ClearList(match())
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, pattern)
        IIf(offset > 1, string = Mid(string, offset))
        Repeat
          IIf(re = 0, ErrorBreak(2))
          IIf(ExamineRegularExpression(re, string) = 0, ErrorBreak(2))
          IIf(NextRegularExpressionMatch(re) = 0, ErrorBreak(1)) ; Error 1: Array is invalid. No matches

          Define groups.l = CountRegularExpressionGroups(re)
          Repeat
            AddElement(match())
            ReDim match()\group(groups + 1)
            match()\group(0) = RegularExpressionMatchString(re)
            For i = 1 To groups
              match()\group(i) = RegularExpressionGroup(re, i)
            Next
          Until NextRegularExpressionMatch(re) = 0
          result = ListSize(match())
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      Structure BackRefInfo
        pos.l
        len.l
        number.l
      EndStructure

      Procedure.l StringRegExpReplace_Backref(List backref.BackRefInfo(), replace.s)
        Define result.l = 0, re.i = CreateRegularExpression(#PB_Any, "(?|(?<!\\)\\(\d+)|\$(\d+)|\${(\d+)}|\\(\\)|\\(\$))")
        ; (?|...)    Non-capturing group
        ; (?<!...)   Negative look-behind
        Repeat
          IIf(re = 0, ReturnBreak(0))
          IIf(ExamineRegularExpression(re, replace) = 0, ReturnBreak(0))

          While NextRegularExpressionMatch(re)
            AddElement(backref())
            backref()\pos = RegularExpressionMatchPosition(re)
            backref()\len = RegularExpressionMatchLength(re)
            match.s = RegularExpressionGroup(re, 1)
            Select match
              Case "\": backref()\number = -1
              Case "$": backref()\number = -2
              Default:  backref()\number = Val(match) ; \d+
            EndSelect
          Wend
          ReturnBreak(ListSize(backref()))
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

      Macro StringRegExpReplace_Prepare(re, groups, replace, new_replace, backref, backref_count)
        If backref_count
          last_pos.l = 1
          ForEach backref()
            new_replace + Mid(replace, last_pos, backref()\pos - last_pos)
            Select backref()\number
              Case -1: new_replace + "\"
              Case -2: new_replace + "$"
              Case 0:  new_replace + RegularExpressionMatchString(re)
              Case 1 To groups: new_replace + RegularExpressionGroup(re, backref()\number)
            EndSelect
            last_pos = backref()\pos + backref()\len
          Next
          new_replace + Mid(replace, last_pos)
        Else
          new_replace = replace
        EndIf
      EndMacro

      Procedure.s StringRegExpReplace(string.s, pattern.s, replace.s, count = 0)
        FunctionInit()
        Define result.s = "", re.i = CreateRegularExpression(#PB_Any, pattern)
        Repeat
          IIf(re = 0, ReturnErrorBreak(string, 2))
          IIf(ExamineRegularExpression(re, string) = 0, ReturnErrorBreak(string, 2))

          NewList backref.BackRefInfo()
          backref_count.l = StringRegExpReplace_Backref(backref(), replace)

          Define last.l = 1, groups.l = CountRegularExpressionGroups(re), replace_count.l = 0
          While NextRegularExpressionMatch(re)
            new_replace.s = ""
            StringRegExpReplace_Prepare(re, groups, replace, new_replace, backref, backref_count)

            pos.l = RegularExpressionMatchPosition(re)
            len.l = RegularExpressionMatchLength(re)

            result + Mid(string, last, pos - last)
            result + new_replace
            last = pos + len

            replace_count + 1
            IIf(count > 0 And replace_count >= count, Break)
          Wend
          result + Mid(string, last)
          Extended(replace_count)
        Until #True
        IIf(re, FreeRegularExpression(re))
        ProcedureReturn result
      EndProcedure

    CompilerEndIf: CompilerEndIf

  ; ===========================================================================
  ;  Timer and Delay functions
  ; ===========================================================================

    Procedure Sleep(delay.l)
      FunctionInit()
      Delay(delay)
    EndProcedure

    Procedure.i TimerInit()
      FunctionInit()
      ProcedureReturn ElapsedMilliseconds()
    EndProcedure

    Procedure.l TimerDiff(init.i)
      FunctionInit()
      ProcedureReturn ElapsedMilliseconds() - init
    EndProcedure

  ; ===========================================================================
  ;  Tray functions
  ; ===========================================================================

  ; ===========================================================================
  ;  Variables and Conversions
  ; ===========================================================================

  ; ===========================================================================
  ;  Window and control helpers
  ; ===========================================================================

    Procedure CheckMatch(string.s, pattern.s)
      Select Opt("WinTitleMatchMode")
      Case -#MatchSubstring
        IIf(FindString(string, pattern, 1, #PB_String_NoCase) <> 0, ProcedureReturn #True)
      Case -#MatchExact
        IIf(LCase(string) = LCase(pattern), ProcedureReturn #True)
      Case -#MatchStart
        IIf(LCase(Left(string, Len(pattern))) = LCase(pattern), ProcedureReturn #True)
      Case #MatchSubstring
        IIf(FindString(string, pattern, 1, #PB_String_CaseSensitive) <> 0, ProcedureReturn #True)
      Case #MatchExact
        IIf(string = pattern, ProcedureReturn #True)
      Default
        IIf(Left(string, Len(pattern)) = pattern, ProcedureReturn #True)
      EndSelect
      ProcedureReturn #False
    EndProcedure

    Procedure.i AllocateMemoryRemote(hwnd.i, size.l, *handle)
      Repeat
        GetWindowThreadProcessId_(hwnd, @pid)
        IIf(pid = 0, Break)

        handle.i = OpenProcess_(#PROCESS_VM_OPERATION | #PROCESS_VM_READ | #PROCESS_VM_WRITE, #False, pid)
        IIf(handle = 0, Break)

        remote_buffer.i = VirtualAllocEx_(handle, 0, size, #MEM_RESERVE | #MEM_COMMIT, #PAGE_READWRITE)
        IIf(remote_buffer = 0, Break)

        PokeI(*handle, handle)
        ProcedureReturn remote_buffer
      Until #True
      PokeI(*handle, 0)
      ProcedureReturn 0
    EndProcedure

    Procedure.i CopyMemoryFromRemote(handle.i, *remote_ptr, *dest_ptr, size.i)
      ReadProcessMemory_(handle, *remote_ptr, *dest_ptr, size, @result.i)
      ProcedureReturn result
    EndProcedure

    Procedure.i CopyMemoryToRemote(handle.i, *src_ptr, *remote_ptr, size.i)
      WriteProcessMemory_(handle, *remote_ptr, *src_ptr, size, @result.i)
      ProcedureReturn result
    EndProcedure

    Procedure FreeMemoryRemote(handle.i, *remote_ptr)
      VirtualFreeEx_(handle, *remote_ptr, 0, #MEM_RELEASE)
      CloseHandle_(handle)
    EndProcedure

    Procedure.s GetWindowTextFast(hwnd.i)
      *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
      IIf(*buffer = 0, ProcedureReturn "")

      GetWindowText_(hwnd, *buffer, #TextBufferLength)
      result.s = PeekS(*buffer)
      FreeMemory(*buffer)
      ProcedureReturn result
    EndProcedure

    Procedure.s GetWindowTextFull(hwnd.i)
      result.s = ""
      If SendMessageTimeout_(hwnd, #WM_GETTEXTLENGTH, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i)
        outcome + 2
        *buffer = AllocateMemory(outcome * SizeOf(Character), #PB_Memory_NoClear)
        If *buffer
          If SendMessageTimeout_(hwnd, #WM_GETTEXT, outcome, *buffer, #SMTO_ABORTIFHUNG, 1000, @outcome.i)
            result = PeekS(*buffer)
          EndIf
        EndIf
        FreeMemory(*buffer)
      EndIf
      ProcedureReturn result
    EndProcedure

    Structure ControlParameter
      parent.i
      result.i
      id.l
      text.s
      class.s
      x.l
      y.l
      w.l
      h.l
      instance.l
      regexpclass.i
    EndStructure

    Procedure EnumChildFindClassnnCallback(hwnd, *m.ControlParameter)
      Define *buffer = 0, result.l = #ContinueEnum
      Repeat
        IIf(*m\class = "", ReturnBreak(#StopEnum))

        *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
        IIf(*buffer = 0, ReturnBreak(#ContinueEnum))

        GetClassName_(hwnd, *buffer, #TextBufferLength)
        text.s = PeekS(*buffer)
        If CheckMatch(text, *m\class)
          *m\instance + 1
          IIf(*m\instance >= 100, ReturnBreak(#StopEnum))

          If *m\result = hwnd
            *m\text = text + Str(*m\instance)
            ReturnBreak(#StopEnum)
          EndIf
        EndIf
        ReturnBreak(#ContinueEnum)
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Procedure EnumChildFindControlCallback(hwnd, *m.ControlParameter)
      Define *buffer = 0, result.l = #ContinueEnum
      Repeat
        match_count.l = 0

        If *m\id And *m\id = GetDlgCtrlID_(hwnd)
          *m\result = hwnd
          ReturnBreak(#StopEnum)
        EndIf

        *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
        IIf(*buffer = 0, ReturnBreak(#ContinueEnum))

        If *m\class <> "" Or *m\regexpclass
          GetClassName_(hwnd, *buffer, #TextBufferLength)
          class.s = PeekS(*buffer)

          If *m\class <> ""
            IIf(Not CheckMatch(class, *m\class), ReturnBreak(#ContinueEnum))
            match_count + 1
          EndIf

          CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
            If *m\regexpclass
              IIf(Not MatchRegularExpression(*m\regexpclass, class), ReturnBreak(#ContinueEnum))
              match_count + 1
            EndIf
          CompilerEndIf: CompilerEndIf
        EndIf

        If *m\text <> ""
          IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXT, #TextBufferLength, *buffer, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ReturnBreak(#ContinueEnum))
          IIf(Not CheckMatch(PeekS(*buffer), *m\text), ReturnBreak(#ContinueEnum))
          match_count + 1
        EndIf

        If *m\parent And (*m\x <> #PB_Ignore Or *m\y <> #PB_Ignore Or *m\w <> #PB_Ignore Or *m\h <> #PB_Ignore)
          GetWindowRect_(hwnd, @r.RECT)
          IIf(*m\w <> #PB_Ignore, IIf(*m\w <> r\right - r\left,  ReturnBreak(#ContinueEnum), match_count + 1))
          IIf(*m\h <> #PB_Ignore, IIf(*m\h <> r\bottom - r\top,  ReturnBreak(#ContinueEnum), match_count + 1))

          If *m\x <> #PB_Ignore Or *m\y <> #PB_Ignore
            p.POINT\x = r\left
            p\y = r\top
            IIf(Not ScreenToClient_(*m\parent, @p), ReturnBreak(#ContinueEnum))
            IIf(*m\x <> #PB_Ignore, IIf(*m\x <> p\x, ReturnBreak(#ContinueEnum), match_count + 1))
            IIf(*m\y <> #PB_Ignore, IIf(*m\y <> p\y, ReturnBreak(#ContinueEnum), match_count + 1))
          EndIf
        EndIf

        ; match_count = 0 means no match, not all pass!!
        IIf(match_count = 0, ReturnBreak(#ContinueEnum))

        ; instance >= 2, wait next match
        *m\instance - 1
        IIf(*m\instance > 0, ReturnBreak(#ContinueEnum))

        *m\result = hwnd
        ReturnBreak(#StopEnum)
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Structure WindowParameter
      result.i
      hwnd.i
      title.s
      text.s
      class.s
      x.l
      y.l
      w.l
      h.l
      visible_text.s
      hidden_text.s
      class_list.s
      regexptitle.i
      regexpclass.i
      instance.l
      active.b
      last.b
      get_list.b
      List windows.WinInfo()
      Map searched.i()
    EndStructure

    ; recieve class list and all texts (including hidden texts)
    Procedure EnumChildCallback(hwnd, *m.WindowParameter)
      *buffer = 0
      Repeat
        *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
        IIf(*buffer = 0, Break)

        class_name.s = ""
        If GetClassName_(hwnd, *buffer, #TextBufferLength)
          class_name = PeekS(*buffer)
          IIf(*m\class_list <> "", *m\class_list + #LF$)
          *m\class_list + class_name
        EndIf

        IIf(*m\text = "", Break)
        If Opt("WinTextMatchMode") = 2 ; Quick mode
          IIf(class_name <> "Button" And class_name <> "Static", Break)
        EndIf

        visible.l = IsWindowVisible_(hwnd)
        IIf(Not visible And Opt("WinDetectHiddenText") = 0, Break)

        IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXT, #TextBufferLength, *buffer, #SMTO_ABORTIFHUNG, 1000, @outcome.i), Break)
        text.s = Trim(PeekS(*buffer))
        IIf(text = "", Break)

        IIf(visible, *m\visible_text + text + #LF$, *m\hidden_text + text + #LF$)

      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn #ContinueEnum
    EndProcedure

    Procedure EnumWindowsCallback(hwnd, *m.WindowParameter)
      Define result = #ContinueEnum, *buffer = 0
      Repeat
        match_count.l = 0

        If *m\hwnd And *m\hwnd = hwnd
          *m\result = hwnd
          ReturnBreak(#StopEnum)
        EndIf

        *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
        IIf(*buffer = 0, ReturnBreak(#ContinueEnum))

        If *m\title <> "" Or *m\regexptitle
          GetWindowText_(hwnd, *buffer, #TextBufferLength)
          title.s = PeekS(*buffer)

          If *m\title <> ""
            IIf(Not CheckMatch(title, *m\title), ReturnBreak(#ContinueEnum))
            match_count + 1
          EndIf

          CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
            If *m\regexptitle
              IIf(Not MatchRegularExpression(*m\regexptitle, title), ReturnBreak(#ContinueEnum))
              match_count + 1
            EndIf
          CompilerEndIf: CompilerEndIf
        EndIf

        If *m\class <> "" Or *m\regexpclass
          GetClassName_(hwnd, *buffer, #TextBufferLength)
          class.s = PeekS(*buffer)

          If *m\class <> ""
            IIf(Not CheckMatch(class, *m\class), ReturnBreak(#ContinueEnum))
            match_count + 1
          EndIf

          CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
            If *m\regexpclass
              IIf(Not MatchRegularExpression(*m\regexpclass, class), ReturnBreak(#ContinueEnum))
              match_count + 1
            EndIf
          CompilerEndIf: CompilerEndIf
        EndIf

        GetWindowRect_(hwnd, @r.RECT)
        IIf(*m\x <> #PB_Ignore, IIf(*m\x <> r\left, ReturnBreak(#ContinueEnum), match_count + 1))
        IIf(*m\y <> #PB_Ignore, IIf(*m\y <> r\top,  ReturnBreak(#ContinueEnum), match_count + 1))
        IIf(*m\w <> #PB_Ignore, IIf(*m\w <> r\right - r\left,  ReturnBreak(#ContinueEnum), match_count + 1))
        IIf(*m\h <> #PB_Ignore, IIf(*m\h <> r\bottom - r\top,  ReturnBreak(#ContinueEnum), match_count + 1))

        ; before EnumChildWindows_, reset the string
        ; for text, always use CaseSensitive FindString
        If *m\text <> ""
          visible.l = IsWindowVisible_(hwnd)
          IIf(Not visible And Opt("WinDetectHiddenText") = 0, ReturnBreak(#ContinueEnum))

          *m\visible_text = ""
          *m\hidden_text = ""
          EnumChildWindows_(hwnd, @EnumChildCallback(), *m)

          IIf((Not FindString(*m\visible_text, *m\text)) And (Not FindString(*m\hidden_text, *m\text)), ReturnBreak(#ContinueEnum))
          match_count + 1
        EndIf

        If *m\get_list
          ; for #GetListInclude, add all items for title = "", text = "", class = ""
          IIf(match_count = 0 And *m\get_list = #GetListExclude, ReturnBreak(#ContinueEnum))

          AddElement(*m\windows())
          *m\windows()\hwnd = hwnd
          *m\windows()\title = GetWindowTextFast(hwnd)
          ReturnBreak(#ContinueEnum)
        Else
          ; match_count = 0 means no match, not all pass!!
          IIf(match_count = 0, ReturnBreak(#ContinueEnum))

          ; instance >= 2, wait next match
          *m\instance - 1
          IIf(*m\instance > 0, ReturnBreak(#ContinueEnum))

          *m\result = hwnd
          ReturnBreak(#StopEnum)
        EndIf

      Until #True
      IIf(*buffer, FreeMemory(*buffer))

      If result = #ContinueEnum And Opt("WinSearchChildren")
        EnumChildWindows_(hwnd, @EnumWindowsCallback(), *m)
        IIf(*m\result, ProcedureReturn #StopEnum)
      EndIf
      ProcedureReturn result
    EndProcedure

    Procedure.l WindowSearchCondition(*m.WindowParameter)
      IIf(*m\title <> "", ProcedureReturn #True)
      IIf(*m\class <> "", ProcedureReturn #True)
      IIf(*m\text <> "", ProcedureReturn #True)
      IIf(*m\x <> #PB_Ignore, ProcedureReturn #True)
      IIf(*m\y <> #PB_Ignore, ProcedureReturn #True)
      IIf(*m\w <> #PB_Ignore, ProcedureReturn #True)
      IIf(*m\h <> #PB_Ignore, ProcedureReturn #True)
      IIf(*m\regexptitle <> 0, ProcedureReturn #True)
      IIf(*m\regexpclass <> 0, ProcedureReturn #True)
      ProcedureReturn #False
    EndProcedure

    Procedure.l TitleParse(title.s, text.s, *m.WindowParameter)
      *m\x = #PB_Ignore
      *m\y = #PB_Ignore
      *m\w = #PB_Ignore
      *m\h = #PB_Ignore
      *m\text = Trim(text)

      If Left(title, 1) = "[" And Right(title, 1) = "]"
        title = Mid(title, 2, Len(title) - 2)
        count = CountString(title, ";")
        For i = 1 To count + 1
          part.s = StringField(title, i, ";")
          ; If a Value must contain a ";" it must be doubled.
          If i < count And Len(StringField(title, i + 1, ";")) = 0
            part + ";" + StringField(title, i + 2, ";")
            i + 2
          EndIf

          prop.s = StringField(part, 1, ":")
          text.s = Trim(Mid(part, Len(prop) + 2))

          Select Trim(LCase(prop))
          Case "title": *m\title = text
          Case "class": *m\class = text
          Case "x": *m\x = Val(text)
          Case "y": *m\y = Val(text)
          Case "w": *m\w = Val(text)
          Case "h": *m\h = Val(text)
          Case "hwnd": *m\hwnd = Val(text)
          Case "instance": *m\instance = Val(text)
          Case "active": *m\active = #True
          Case "last": *m\last = #True

          CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
            Case "regexptitle":
              *m\regexptitle = CreateRegularExpression(#PB_Any, text)
              IIf(*m\regexptitle = 0, ProcedureReturn 0)

            Case "regexpclass":
              *m\regexpclass = CreateRegularExpression(#PB_Any, text)
              IIf(*m\regexpclass = 0, ProcedureReturn 0)
          CompilerEndIf: CompilerEndIf

          EndSelect
        Next
      ElseIf title = "" And text = ""
        *m\active = #True
      Else
        If StringIsDigit(title)
          number.i = Val(title)
          If IsWindow_(number)
            *m\hwnd = number
            ProcedureReturn 1
          EndIf
        EndIf
        *m\title = Trim(title)
      EndIf

      ProcedureReturn 1
    EndProcedure

    Procedure ClassNNParse(classnn.s, *m.ControlParameter)
      If StringIsDigit(classnn)
        *m\id = Val(classnn)
      ElseIf StringIsDigit(Right(classnn, 2))
        *m\class = Left(classnn, Len(classnn) - 2)
        *m\instance = Val(Right(classnn, 2))
      ElseIf StringIsDigit(Right(classnn, 1))
        *m\class = Left(classnn, Len(classnn) - 1)
        *m\instance = Val(Right(classnn, 1))
      Else
        *m\class = classnn
      EndIf
    EndProcedure

    Procedure.l ControlParse(string.s, parent.i, *m.ControlParameter)
      *m\x = #PB_Ignore
      *m\y = #PB_Ignore
      *m\w = #PB_Ignore
      *m\h = #PB_Ignore
      *m\parent = parent

      If Left(string, 1) = "[" And Right(string, 1) = "]"
        string = Mid(string, 2, Len(string) - 2)
        count = CountString(string, ";")
        For i = 1 To count + 1
          part.s = StringField(string, i, ";")
          ; If a Value must contain a ";" it must be doubled.
          If i < count And Len(StringField(string, i + 1, ";")) = 0
            part + ";" + StringField(string, i + 2, ";")
            i + 2
          EndIf

          prop.s = StringField(part, 1, ":")
          text.s = Trim(Mid(part, Len(prop) + 2))

          Select Trim(LCase(prop))
          Case "id": *m\id = Val(text)
          Case "text": *m\text = text
          Case "class": *m\class = text
          Case "x": *m\x = Val(text)
          Case "y": *m\y = Val(text)
          Case "w": *m\w = Val(text)
          Case "h": *m\h = Val(text)
          Case "instance": *m\instance = Val(text)
          Case "classnn": ClassNNParse(text, *m)

          CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
            Case "regexpclass":
              *m\regexpclass = CreateRegularExpression(#PB_Any, text)
              IIf(*m\regexpclass = 0, ProcedureReturn 0)
          CompilerEndIf: CompilerEndIf

          EndSelect
        Next
      Else
        ClassNNParse(string, *m)
      EndIf

      ProcedureReturn 1
    EndProcedure

  ; ===========================================================================
  ;  Window functions
  ; ===========================================================================

    Procedure.i WinGetHandle(title.s, text.s = "")
      FunctionInit()
      Define m.WindowParameter, result.i = 0
      Repeat
        IIf(TitleParse(title, text, @m) = 0, ErrorBreak(2))

        If m\active
          __last_hwnd = GetForegroundWindow_()
          ReturnBreak(__last_hwnd)

        ElseIf m\last
          ReturnBreak(__last_hwnd)

        ElseIf m\hwnd Or WindowSearchCondition(@m)
          EnumWindows_(@EnumWindowsCallback(), @m)
          If m\result
            __last_hwnd = m\result
            ReturnBreak(__last_hwnd)
          EndIf
        EndIf
        __last_hwnd = 0
        Error(1)
      Until #True

      CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
        IIf(m\regexptitle, FreeRegularExpression(m\regexptitle))
        IIf(m\regexpclass, FreeRegularExpression(m\regexpclass))
      CompilerEndIf: CompilerEndIf
      ProcedureReturn result
    EndProcedure

    Procedure.i WinActivate(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        IIf(IsIconic_(hwnd), ShowWindow_(hwnd, #SW_RESTORE))
        SetForegroundWindow_(hwnd)
        Delay(Opt("WinDelay"))
      EndIf
      ProcedureReturn hwnd
    EndProcedure

    Procedure.i WinActive(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd = GetForegroundWindow_()
        ProcedureReturn hwnd
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinClose(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        PostMessage_(hwnd, #WM_CLOSE, 0, 0)
        Delay(Opt("WinDelay"))
      EndIf
      ProcedureReturn hwnd
    EndProcedure

    Procedure.i WinExists(title.s, text.s = "")
      ProcedureReturn WinGetHandle(title, text)
    EndProcedure

    Procedure.i WinFlash(title.s, text.s = "", flashes.l = 4, delay.l = 500, wait.l = #True)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        If wait
          For i = 1 To (flashes - 2) * 2 + 1
            FlashWindow_(hwnd, #True)
            Delay(delay)
          Next
        Else
          f.FLASHWINFO\cbSize = SizeOf(FLASHWINFO)
          f\hwnd = hwnd
          f\dwFlags = #FLASHW_ALL | #FLASHW_TIMERNOFG
          f\uCount = flashes
          f\dwTimeout = delay
          FlashWindowEx_(@f)
        EndIf
        Delay(Opt("WinDelay"))
      EndIf
      ProcedureReturn hwnd
    EndProcedure

    Procedure.s WinGetCaretPos()
      FunctionInit()
      hwnd.i = GetForegroundWindow_()
      GetWindowRect_(hwnd, @r1.RECT)

      AttachThreadInput(hwnd)
      GetCaretPos_(@p.POINT)
      GetWindowRect_(GetFocus_(), @r2.RECT)
      DetachThreadInput()

      Select Opt("CaretCoordMode")
      Case #RelativeToWindow
        x.l = p\x + r2\left - r1\left
        y.l = p\y + r2\top - r1\top
      Case #AbsoluteScreen
        x.l = p\x + r2\left
        y.l = p\y + r2\top
      Default ; Case #RelativeToClientArea
        x.l = p\x
        y.l = p\y
      EndSelect
      ProcedureReturn Str(x)+ "," + Str(y)
    EndProcedure

    Procedure.s WinGetClassList(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        EnumChildWindows_(hwnd, @EnumChildCallback(), @m.WindowParameter)
      EndIf
      ProcedureReturn m\class_list
    EndProcedure

    Procedure.s WinGetClientSize(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        GetClientRect_(hwnd, @r.RECT)
        ProcedureReturn Str(r\right) + "," + Str(r\bottom)
      EndIf
      ProcedureReturn ""
    EndProcedure

    Procedure.s WinGetPos(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        GetWindowRect_(hwnd, @r.RECT)
        ProcedureReturn Str(r\left) + "," + Str(r\top) + "," + Str(r\right - r\left) + "," + Str(r\bottom - r\top)
      EndIf
      ProcedureReturn ""
    EndProcedure

    Procedure.l WinGetProcess(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        GetWindowThreadProcessId_(hwnd, @pid.l)
        ProcedureReturn pid
      EndIf
    EndProcedure

    Procedure.l WinGetState(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        flag.l = 1
        IIf(IsWindowVisible_(hwnd), flag | #WINDIW_VISIBLE)
        IIf(IsWindowEnabled_(hwnd), flag | #WINDIW_ENABLED)
        IIf(hwnd = GetForegroundWindow_(), flag | #WINDIW_ACTIVE)
        IIf(IsIconic_(hwnd), flag | #WINDIW_MINIMIZED)
        IIf(IsZoomed_(hwnd), flag | #WINDIW_MAXIMIZED)
        ProcedureReturn flag
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.s WinGetTitle(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      IIf(hwnd, ProcedureReturn GetWindowTextFull(hwnd), ProcedureReturn "")
    EndProcedure

    Procedure.s WinGetText(title.s, text.s = "")
      Define result.s = "", hwnd.i = WinGetHandle(title, text)
      If hwnd
        ; set something to \text to avoid EnumChildCallback return before get text
        m.WindowParameter\text = " "
        EnumChildWindows_(hwnd, @EnumChildCallback(), @m)
        result = m\visible_text
        If m\hidden_text <> ""
          IIf(result <> "", result + #LF$)
          result + m\hidden_text
        EndIf
      EndIf
      ProcedureReturn result
    EndProcedure

    Procedure.l WinKill(title.s, text.s = "")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        SetPrivilege("SeDebugPrivilege")
        GetWindowThreadProcessId_(hwnd, @pid.l)
        process.i = OpenProcess_(#PROCESS_TERMINATE, 0, pid)
        If process
          result.l = TerminateProcess_(process, 0)
          CloseHandle_(process)
          Delay(Opt("WinDelay"))
          ProcedureReturn result
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.l WinList(List windows.WinInfo(), title.s = "", text.s = "")
      FunctionInit()
      ClearList(windows())
      Define m.WindowParameter, result.l = 0
      Repeat
        IIf(TitleParse(title, text, @m) = 0, ErrorBreak(2))
        IIf(WindowSearchCondition(@m), m\get_list = #GetListExclude, m\get_list = #GetListInclude)

        ClearList(windows())

        ; single result
        If m\active Or m\last Or m\hwnd
          hwnd.i = WinGetHandle(title)
          If hwnd
            AddElement(windows())
            windows()\hwnd = hwnd
            windows()\title = GetWindowTextFast(hwnd)
          EndIf
          ReturnBreak(1)
        EndIf

        EnumWindows_(@EnumWindowsCallback(), @m)
        MergeLists(m\windows(), windows())
        result = ListSize(windows())
      Until #True

      CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
        IIf(m\regexptitle, FreeRegularExpression(m\regexptitle))
        IIf(m\regexpclass, FreeRegularExpression(m\regexpclass))
      CompilerEndIf: CompilerEndIf
      ProcedureReturn result
    EndProcedure

    Procedure.l WinMenuSelectItem(title.s, text.s, item0.s, item1.s = "", item2.s ="", item3.s ="", item4.s ="", item5.s ="", item6.s ="", item7.s ="")
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        Dim items.s(8)
        items(0) = item0 : items(1) = item1 : items(2) = item2 : items(3) = item3
        items(4) = item4 : items(5) = item5 : items(6) = item6 : items(7) = item7
        items(8) = "" ; the end

        Define n.l = 0, menu.i = GetMenu_(hwnd), menu_id.l = -1, buffer.s{1024}
        Repeat
          menu_count.l = GetMenuItemCount_(menu)
          found.l = #False
          For i = 0 To menu_count - 1
            succeed.l = GetMenuString_(menu, i, @buffer, LengthOf(buffer), #MF_BYPOSITION)

            If succeed And (CheckMatch(buffer, items(n)) Or CheckMatch(ReplaceString(buffer, "&", ""), items(n)))
              found = #True
              Break
            EndIf
          Next
          IIf(Not found, Break)

          menu_id = GetMenuItemID_(menu, i) ; -1 = submenu
          IIf(menu_id <> -1, Break) ; already found menuid

          n + 1
          IIf(items(n) = "", Break) ; no more item to find

          menu = GetSubMenu_(menu, i) ; no more submenu can find
          IIf(menu = 0, Break)
        ForEver

        If menu_id <> -1
          PostMessage_(hwnd, #WM_COMMAND, menu_id, 0)
          Delay(Opt("WinDelay"))
          ProcedureReturn menu_id
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    DataSection
      CLSID_Shell_Application:  ; {13709620-C279-11CE-A49E-444553540000}
      Data.l $13709620
      Data.w $C279, $11CE
      Data.b $A4, $9E, $44, $45, $53, $54, $0, $0

      IID_IShellDispatch:  ; {D8F015C0-C278-11CE-A49E-444553540000}
      Data.l $D8F015C0
      Data.w $C278, $11CE
      Data.b $A4, $9E, $44, $45, $53, $54, $0, $0
    EndDataSection

    Procedure WinMinimizeAll()
      FunctionInit()
      CoInitialize_(0)
      If Not CoCreateInstance_(?CLSID_Shell_Application, 0, 1, ?IID_IShellDispatch, @obj.IShellDispatch)
        obj\MinimizeAll()
        obj\Release()
        Delay(Opt("WinDelay"))
      EndIf
      CoUninitialize_()
    EndProcedure

    Procedure WinMinimizeAllUndo()
      FunctionInit()
      CoInitialize_(0)
      If Not CoCreateInstance_(?CLSID_Shell_Application, 0, 1, ?IID_IShellDispatch, @obj.IShellDispatch)
        obj\UndoMinimizeALL()
        obj\Release()
        Delay(Opt("WinDelay"))
      EndIf
      CoUninitialize_()
    EndProcedure

    Procedure.i WinMove(title.s, text.s = "", x.l = #PB_Ignore, y.l = #PB_Ignore, width.l = #PB_Ignore, height.l = #PB_Ignore, speed.l = 0)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        GetWindowRect_(hwnd, @rect.RECT)
        old_x.l = rect\left
        old_y.l = rect\top
        old_w.l = rect\right - rect\left
        old_h.l = rect\bottom - rect\top
        IIf(x = #PB_Ignore, x = old_x)
        IIf(y = #PB_Ignore, y = old_y)
        IIf(width = #PB_Ignore, width = old_w)
        IIf(height = #PB_Ignore, height = old_h)

        If speed > 0
          times = speed * 10
          IIf(times > 1000, times = 1000)
          For i = 1 To times
            curr_x.l = old_x + (x - old_x) * i / times
            curr_y.l = old_y + (y - old_y) * i / times
            curr_w.l = old_w + (width - old_w) * i / times
            curr_h.l = old_h + (height - old_h) * i / times
            MoveWindow_(hwnd, curr_x, curr_y, curr_w, curr_h, #True)
            Delay(1)
          Next
        EndIf

        If MoveWindow_(hwnd, x, y, width, height, #True)
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinSetOnTop(title.s, text.s = "", flag.l = #True)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        IIf(flag, flag = #HWND_TOPMOST, flag = #HWND_NOTOPMOST)
        If SetWindowPos_(hwnd, flag, 0, 0, 0, 0, #SWP_NOMOVE | #SWP_NOSIZE | #SWP_SHOWWINDOW)
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinSetState(title.s, text.s, flag.l = #SW_RESTORE)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        Select flag
        Case #PB_Window_Minimize  : flag = #SW_MINIMIZE
        Case #PB_Window_Maximize  : flag = #SW_MAXIMIZE
        Case #PB_Window_Invisible : flag = #SW_HIDE
        EndSelect

        Select flag
        Case #SW_HIDE, #SW_SHOW, #SW_MINIMIZE, #SW_MAXIMIZE, #SW_RESTORE
          ShowWindow_(hwnd, flag)
        Case #SW_DISABLE
          EnableWindow_(hwnd, #False)
        Case #SW_ENABLE
          EnableWindow_(hwnd, #True)
        Default
          ProcedureReturn 0
        EndSelect

        Delay(Opt("WinDelay"))
        ProcedureReturn hwnd
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinSetTitle(title.s, text.s, string.s)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        If SendMessageTimeout_(hwnd, #WM_SETTEXT, 0, @string, #SMTO_ABORTIFHUNG, 1000, @outcome.i) And outcome
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinSetTrans(title.s, text.s = "", transparency.l = 128)
      hwnd.i = WinGetHandle(title, text)
      If hwnd
        IIf(transparency < 0, transparency = 0)
        IIf(transparency > 255, transparency = 255)
        exstyle.l = GetWindowLong_(hwnd, #GWL_EXSTYLE)
        SetWindowLong_(hwnd, #GWL_EXSTYLE, exstyle | #WS_EX_LAYERED)
        If SetLayeredWindowAttributes_(hwnd, 0, transparency, #LWA_ALPHA)
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
      EndIf
      ProcedureReturn 0
    EndProcedure

    Procedure.i WinWait(title.s, text.s = "", timeout.l = 0) ; timeout in sec
      time_end.l = ElapsedMilliseconds() + (timeout * 1000)
      Repeat
        hwnd.i = WinGetHandle(title, text)
        If hwnd
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
        IIf(timeout And ElapsedMilliseconds() > time_end, ProcedureReturn 0)
        Delay(Opt("WinWaitDelay"))
      ForEver
    EndProcedure

    Procedure.i WinWaitActive(title.s, text.s = "", timeout.l = 0) ; timeout in sec
      time_end.l = ElapsedMilliseconds() + (timeout * 1000)
      Repeat
        hwnd.i = WinGetHandle(title, text)
        If hwnd And hwnd = GetForegroundWindow_()
          Delay(Opt("WinDelay"))
          ProcedureReturn hwnd
        EndIf
        IIf(timeout And ElapsedMilliseconds() > time_end, ProcedureReturn 0)
        Delay(Opt("WinWaitDelay"))
      ForEver
    EndProcedure

    Procedure.i WinWaitClose(title.s, text.s = "", timeout.l = 0) ; timeout in sec
      time_end.l = ElapsedMilliseconds() + (timeout * 1000)
      Repeat
        hwnd.i = WinGetHandle(title, text)
        If Not hwnd
          Delay(Opt("WinDelay"))
          ProcedureReturn 1
        EndIf
        IIf(timeout And ElapsedMilliseconds() > time_end, ProcedureReturn 0)
        Delay(Opt("WinWaitDelay"))
      ForEver
    EndProcedure

    Procedure.i WinWaitNotActive(title.s, text.s = "", timeout.l = 0) ; timeout in sec
      time_end.l = ElapsedMilliseconds() + (timeout * 1000)
      Repeat
        hwnd.i = WinGetHandle(title, text)
        If Not hwnd Or hwnd <> GetForegroundWindow_()
          Delay(Opt("WinDelay"))
          ProcedureReturn 1
        EndIf
        IIf(timeout And ElapsedMilliseconds() > time_end, ProcedureReturn 0)
        Delay(Opt("WinWaitDelay"))
      ForEver
    EndProcedure

  ; ===========================================================================
  ;  Controls functions
  ; ===========================================================================

    Procedure.i ControlGetHandle(title.s, text.s, control_id.s)
      Define result.i = 0, m.ControlParameter, hwnd.i = WinGetHandle(title, text)
      Repeat
        IIf(ControlParse(control_id, hwnd, @m.ControlParameter) = 0, ErrorBreak(2))
        EnumChildWindows_(hwnd, @EnumChildFindControlCallback(), @m)

        IIf(m\result, result = m\result, Error(1))
      Until #True

      CompilerIf Defined(PureAutoIt_EnableRegExp, #PB_Constant): CompilerIf #PureAutoIt_EnableRegExp
        IIf(m\regexpclass, FreeRegularExpression(m\regexpclass))
      CompilerEndIf: CompilerEndIf
      ProcedureReturn result
    EndProcedure

    Procedure.i ControlClick(title.s, text.s, control_id.s, button.s = "", clicks.l = 1, x = #PB_Ignore, y = #PB_Ignore)
      hwnd.i = ControlGetHandle(title, text, control_id)
      IIf(Not hwnd, ProcedureReturn 0)
      IIf(MouseButtonToMsg(button, @msg_down.l, @msg_up.l, @wparam.l, 1) = 0, ProcedureReturn 0)

      If x < 0 Or y < 0 ; Default is center
        GetWindowRect_(hwnd, @r.RECT) ; even fail, result is 0
        IIf(x < 0, x = (r\right - r\left) / 2)
        IIf(y < 0, y = (r\bottom - r\top) / 2)
      EndIf
      lparam.i = y << 16 | x

      For i = 1 To clicks
        PostMessage_(hwnd, msg_down, wparam, lparam)
        Delay(Opt("MouseClickDownDelay"))
        PostMessage_(hwnd, msg_up, 0, lparam)
        Delay(Opt("MouseClickDelay"))
      Next
      ProcedureReturn #True
    EndProcedure

    Procedure.i ControlSetText(title.s, text.s, control_id.s, string.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        If SendMessageTimeout_(hwnd, #WM_SETTEXT, 0, @string, #SMTO_ABORTIFHUNG, 1000, @outcome.i) And outcome
          Delay(Opt("ControlDelay"))
          ProcedureReturn #True
        EndIf
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.s ControlGetText(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      IIf(hwnd, ProcedureReturn GetWindowTextFull(hwnd), ProcedureReturn "")
    EndProcedure

    Procedure.l ControlDisable(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        EnableWindow_(hwnd, #False)
        Delay(Opt("ControlDelay"))
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.l ControlEnable(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        EnableWindow_(hwnd, #True)
        Delay(Opt("ControlDelay"))
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.l ControlFocus(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        AttachThreadInput(hwnd)
        ret.i = SetFocus_(hwnd)
        DetachThreadInput()

        Delay(Opt("ControlDelay"))
        IIf(ret, ProcedureReturn #True)
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.s ControlGetPos(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        parent.i = GetParent_(hwnd)
        IIf(parent = 0, ProcedureReturn "")

        GetWindowRect_(hwnd, @r.RECT)
        p.POINT\x = r\left
        p\y = r\top
        IIf(Not ScreenToClient_(parent, @p), ProcedureReturn "")

        ProcedureReturn Str(p\x) + "," + Str(p\y) + "," + Str(r\right - r\left) + "," + Str(r\bottom - r\top)
      EndIf
      ProcedureReturn ""
    EndProcedure

    Procedure.l ControlHide(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        ShowWindow_(hwnd, #SW_HIDE)
        Delay(Opt("ControlDelay"))
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.l ControlShow(title.s, text.s, control_id.s)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        ShowWindow_(hwnd, #SW_SHOWNORMAL)
        Delay(Opt("ControlDelay"))
        ProcedureReturn #True
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.l ControlMove(title.s, text.s, control_id.s, x.l = #PB_Ignore, y.l = #PB_Ignore, width.l = #PB_Ignore, height.l = #PB_Ignore)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        If x = #PB_Ignore Or y = #PB_Ignore Or width = #PB_Ignore Or height = #PB_Ignore
          Define r.RECT, p.POINT
          GetWindowRect_(hwnd, @r)
          IIf(width = #PB_Ignore, width = r\right - r\left)
          IIf(height = #PB_Ignore, height = r\bottom - r\top)

          If x = #PB_Ignore Or y = #PB_Ignore
            parent.i = GetParent_(hwnd)
            IIf(parent = 0, ProcedureReturn #False)

            p\x = r\left
            p\y = r\top
            IIf(Not ScreenToClient_(parent, @p), ProcedureReturn #False)
            IIf(x = #PB_Ignore, x = p\x)
            IIf(y = #PB_Ignore, y = p\y)
          EndIf
        EndIf

        If MoveWindow_(hwnd, x, y, width, height, #True)
          Delay(Opt("ControlDelay"))
          ProcedureReturn #True
        EndIf
      EndIf
      ProcedureReturn #False
    EndProcedure

    Procedure.s StatusbarGetText(title.s, text.s = "", part.l = 1)
      Define handle.i, *remote_buffer, *buffer = 0, result.s = ""
      Repeat
        IIf(part < 1, Break)
        part - 1

        hwnd.i = WinGetHandle(title, text)
        IIf(hwnd = 0, Break)

        m.ControlParameter\class = "msctls_statusbar32"
        EnumChildWindows_(hwnd, @EnumChildFindControlCallback(), @m)
        IIf(m\result = 0, Break)

        sb_hwnd.i = m\result
        IIf(Not SendMessageTimeout_(sb_hwnd, #SB_GETPARTS, 0, 0, #SMTO_ABORTIFHUNG, 1000, @part_count.i), Break)
        IIf(part_count < part, Break)

        IIf(Not SendMessageTimeout_(sb_hwnd, #SB_GETTEXTLENGTH, part, 0, #SMTO_ABORTIFHUNG, 1000, @part_length.i), Break)
        part_length & $FFFF
        IIf(part_length = 0, Break)

        buffer_size.l = part_length * SizeOf(Character) + 8
        *remote_buffer = AllocateMemoryRemote(sb_hwnd, buffer_size, @handle)
        IIf(*remote_buffer = 0, Break)

        IIf(Not SendMessageTimeout_(sb_hwnd, #SB_GETTEXT, part, *remote_buffer, #SMTO_ABORTIFHUNG, 1000, @text_length.i), Break)
        text_length & $FFFF
        IIf(text_length = 0, Break)

        *buffer = AllocateMemory((text_length + 1) * SizeOf(Character), #PB_Memory_NoClear)
        IIf(*buffer = 0, Break)

        CopyMemoryFromRemote(handle, *remote_buffer, *buffer, text_length * SizeOf(Character))
        result = PeekS(*buffer)
        Delay(Opt("ControlDelay"))
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      IIf(*remote_buffer, FreeMemoryRemote(handle, *remote_buffer))
      ProcedureReturn result
    EndProcedure

    Procedure.s ControlGetFocus(title.s, text.s = "")
      Define result.s = "", *buffer = 0
      Repeat
        hwnd.i = WinGetHandle(title, text)
        IIf(hwnd = 0, ReturnBreak(""))

        AttachThreadInput(hwnd)
        focus_hwnd.i = GetFocus_()
        DetachThreadInput()
        IIf(focus_hwnd = 0, ReturnBreak(""))

        *buffer = AllocateMemory(#TextBufferSize, #PB_Memory_NoClear)
        IIf(*buffer = 0, ReturnBreak(""))

        GetClassName_(focus_hwnd, *buffer, #TextBufferLength)
        text.s = PeekS(*buffer)
        If text <> ""
          m.ControlParameter\class = text
          m\result = focus_hwnd
          m\instance = 0
          m\text = ""
          EnumChildWindows_(hwnd, @EnumChildFindClassnnCallback(), @m)
          IIf(m\text <> "", ReturnBreak(m\text))
        EndIf

        ; no class name, or no classnn found (not match or instance > 100), return id
        id.l = GetDlgCtrlID_(focus_hwnd)
        IIf(id, ReturnBreak(Str(id)))
      Until #True
      IIf(*buffer, FreeMemory(*buffer))
      ProcedureReturn result
    EndProcedure

    Procedure.s ControlCommand(title.s, text.s, control_id.s, command.s, option.s = "")
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        result.s = ""
        Select LCase(command)
        Case "isvisible"
          IIf(IsWindowVisible_(hwnd), ProcedureReturn "1", ProcedureReturn "")

        Case "isenabled"
          IIf(IsWindowEnabled_(hwnd), ProcedureReturn "1", ProcedureReturn "")

        Case "ischecked"
          IIf(Not SendMessageTimeout_(hwnd, #BM_GETCHECK, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          IIf(outcome = #BST_CHECKED, result = "1", result = "")

        Case "check"
          IIf(SendMessageTimeout_(hwnd, #BM_SETCHECK , #BST_CHECKED, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), result = "1")

        Case "uncheck"
          IIf(SendMessageTimeout_(hwnd, #BM_SETCHECK , #BST_UNCHECKED, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), result = "1")

        Case "editpaste"
          IIf(SendMessageTimeout_(hwnd, #EM_REPLACESEL, 0, @option, #SMTO_ABORTIFHUNG, 1000, @outcome.i), result = "1")

        Case "currenttab"
          IIf(Not SendMessageTimeout_(hwnd, #TCM_GETCURSEL, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i) Or outcome = -1, ProcedureReturn "")
          result = Str(outcome + 1)

        Case "tabright"
          IIf(Not SendMessageTimeout_(hwnd, #TCM_GETCURSEL, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i) Or outcome = -1, ProcedureReturn "")
          IIf(Not SendMessageTimeout_(hwnd, #TCM_SETCURSEL, outcome + 1, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          IIf(outcome = -1, result = "", result = Str(outcome + 1))

        Case "tableft"
          IIf(Not SendMessageTimeout_(hwnd, #TCM_GETCURSEL, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i) Or outcome = -1 Or outcome = 0, ProcedureReturn "")
          IIf(Not SendMessageTimeout_(hwnd, #TCM_SETCURSEL, outcome - 1, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          IIf(outcome = -1, result = "", result = Str(outcome + 1))

        Case "getlinecount"
          IIf(Not SendMessageTimeout_(hwnd, #EM_GETLINECOUNT, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          IIf(outcome = 0, result = "", result = Str(outcome))

        Case "getcurrentline"
          IIf(Not SendMessageTimeout_(hwnd, #EM_LINEFROMCHAR, -1, 0, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          result = Str(outcome + 1)

        Case "getcurrentcol"
          IIf(Not SendMessageTimeout_(hwnd, #EM_GETSEL, @start.l, @last.l, #SMTO_ABORTIFHUNG, 1000, @outcome.i), ProcedureReturn "")
          IIf(Not SendMessageTimeout_(hwnd, #EM_LINEFROMCHAR, -1, 0, #SMTO_ABORTIFHUNG, 1000, @line_number.i), ProcedureReturn "")

          If line_number = 0
            result = Str(start + 1)
          Else
            orig.l = start
            Repeat
              IIf(Not SendMessageTimeout_(hwnd, #EM_LINEFROMCHAR, start, 0, #SMTO_ABORTIFHUNG, 1000, @outcome), ProcedureReturn "")
              IIf(outcome <> line_number, Break)
              start - 1
            ForEver
            result = Str(orig - start)
          EndIf

        Case "getline"
          req_line.l = Val(option)
          IIf(req_line <= 0, ProcedureReturn "")
          *buffer = 0
          Repeat
            IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXTLENGTH, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome), Break)
            outcome + 2
            *buffer = AllocateMemory(outcome * SizeOf(Character), #PB_Memory_NoClear)
            IIf(*buffer = 0, Break)

            IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXT, outcome, *buffer, #SMTO_ABORTIFHUNG, 1000, @outcome), Break)
            IIf(outcome = 0, Break)

            result = StringField(PeekS(*buffer), req_line, #LF$)
          Until #True
          IIf(*buffer, FreeMemory(*buffer))

        Case "getselected"
          *buffer = 0
          Repeat
            IIf(Not SendMessageTimeout_(hwnd, #EM_GETSEL, @start.l, @last.l, #SMTO_ABORTIFHUNG, 1000, @outcome), Break)
            IIf(start = last, Break)

            IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXTLENGTH, 0, 0, #SMTO_ABORTIFHUNG, 1000, @outcome), Break)
            outcome + 2
            *buffer = AllocateMemory(outcome * SizeOf(Character), #PB_Memory_NoClear)
            IIf(*buffer = 0, Break)

            IIf(Not SendMessageTimeout_(hwnd, #WM_GETTEXT, outcome, *buffer, #SMTO_ABORTIFHUNG, 1000, @outcome), Break)
            IIf(outcome = 0 Or last > outcome, Break)

            result = PeekS(*buffer + start * SizeOf(Character), last - start)
          Until #True
          IIf(*buffer, FreeMemory(*buffer))

        EndSelect
        Delay(Opt("ControlDelay"))
        ProcedureReturn result
      EndIf
      ProcedureReturn ""
    EndProcedure

    Procedure.i ControlSend(title.s, text.s, control_id.s, keys.s, dbcs = #False)
      hwnd.i = ControlGetHandle(title, text, control_id)
      If hwnd
        AttachThreadInput(hwnd)
        For i = 1 To Len(keys)
          SendUnicode(Asc(Mid(keys, i, 1)), 1, hwnd, dbcs)
        Next
        DetachThreadInput()
        ProcedureReturn hwnd
      EndIf
      ProcedureReturn 0
    EndProcedure
EndModule

; IDE Options = PureBasic 5.70 LTS (Windows - x86)
; CursorPosition = 4565
; FirstLine = 4531
; Folding = ------------------------------------------------------------------
; EnableXP
; CompileSourceDirectory